@@ -37,16 +37,16 @@
 #include "include/layer4.h"
 #include "include/main.h"
 
-/* 
+/*
  * The global design of this checker is the following :
- * 
+ *
  * - All the actions are done asynchronously.
  * - All the actions handle timeout connection.
  * - All the actions handle error from low layer to upper
  *   layers.
- * 
+ *
  * The global synopsis of the inter-thread-call is :
- *     
+ *
  *     http_request_thread (send SSL GET request)
  *            v
  *     http_response_thread (initialize read stream step)
@@ -132,7 +132,7 @@ finalize(thread_t * thread)
 	if (req->verbose) {
 		printf("\n");
 		printf(HTML_HASH);
-		dump_buffer((char *) digest, digest_length);
+		dump_buffer((char *) digest, digest_length, stdout);
 
 		printf(HTML_HASH_FINAL);
 	}
@@ -152,7 +152,7 @@ http_dump_header(char *buffer, int size)
 {
 	int r;
 
-	dump_buffer(buffer, size);
+	dump_buffer(buffer, size, stdout);
 	printf(HTTP_HEADER_ASCII);
 	for (r = 0; r < size; r++)
 		printf("%c", buffer[r]);
@@ -178,7 +178,7 @@ http_process_stream(SOCK * sock_obj, int r)
 			if (r) {
 				if (req->verbose) {
 					printf(HTML_HEADER_HEXA);
-					dump_buffer(sock_obj->extracted, r);
+					dump_buffer(sock_obj->extracted, r, stdout);
 				}
 				memmove(sock_obj->buffer, sock_obj->extracted, r);
 				HASH_UPDATE(sock_obj, sock_obj->buffer, r);
@@ -199,7 +199,7 @@ http_process_stream(SOCK * sock_obj, int r)
 		}
 	} else if (sock_obj->size) {
 		if (req->verbose)
-			dump_buffer(sock_obj->buffer, r);
+			dump_buffer(sock_obj->buffer, r, stdout);
 		HASH_UPDATE(sock_obj, sock_obj->buffer, sock_obj->size);
 		sock_obj->size = 0;
 	}
@@ -311,26 +311,17 @@ http_request_thread(thread_t * thread)
 		*request_host_port = 0;
 	} else {
 		request_host = req->ipaddress;
-	
+
 		/* Allocate a buffer for the port string ( ":" [0-9][0-9][0-9][0-9][0-9] "\0" ) */
 		request_host_port = (char*) MALLOC(7);
 		snprintf(request_host_port, 7, ":%d",
 		 ntohs(req->addr_port));
 	}
-	
-	if(req->dst){
-		if(req->dst->ai_family == AF_INET6 && !req->vhost) {
-			snprintf(str_request, GET_BUFFER_LENGTH, REQUEST_TEMPLATE_IPV6,
-				req->url, request_host, request_host_port);
-		} else {
-			snprintf(str_request, GET_BUFFER_LENGTH, REQUEST_TEMPLATE,
-				req->url, request_host, request_host_port);
-		}
-	} else {
-		snprintf(str_request, GET_BUFFER_LENGTH, REQUEST_TEMPLATE,
-			req->url, request_host, request_host_port);
-	}
-	
+
+	snprintf(str_request, GET_BUFFER_LENGTH,
+		 (req->dst && req->dst->ai_family == AF_INET6 && !req->vhost) ? REQUEST_TEMPLATE_IPV6 : REQUEST_TEMPLATE,
+		  req->url, request_host, request_host_port);
+
 	FREE(request_host_port);
 
 	/* Send the GET request to remote Web server */
