@@ -58,8 +58,9 @@ dump_checker(void *data)
 }
 
 void
-dump_conn_opts (conn_opts_t *conn)
+dump_conn_opts(void *data)
 {
+	conn_opts_t *conn = data;
 	log_message(LOG_INFO, "   Connection dest = %s", inet_sockaddrtopair(&conn->dst));
 	if (conn->bindto.ss_family)
 		log_message(LOG_INFO, "   Bind to = %s", inet_sockaddrtopair(&conn->bindto));
@@ -113,17 +114,7 @@ queue_checker(void (*free_func) (void *), void (*dump_func) (void *)
 	}
 }
 
-/* Set dst */
-void
-checker_set_dst(struct sockaddr_storage *dst)
-{
-	virtual_server_t *vs = LIST_TAIL_DATA(check_data->vs);
-	real_server_t *rs = LIST_TAIL_DATA(vs->rs);
-
-	*dst = rs->addr;
-}
-
-void
+static void
 checker_set_dst_port(struct sockaddr_storage *dst, uint16_t port)
 {
 	if (dst->ss_family == AF_INET6) {
@@ -230,8 +221,7 @@ init_checkers_queue(void)
 void
 free_checkers_queue(void)
 {
-	free_list(checkers_queue);
-	checkers_queue = NULL;
+	free_list(&checkers_queue);
 	ncheckers = 0;
 }
 
