@@ -28,11 +28,10 @@
 #include "check_ssl.h"
 #include "check_api.h"
 #include "global_data.h"
-#include "ipwrapper.h"
-#include "ipvswrapper.h"
 #include "pidfile.h"
 #include "daemon.h"
 #include "signals.h"
+#include "process.h"
 #include "logger.h"
 #include "list.h"
 #include "main.h"
@@ -41,12 +40,10 @@
 #include "bitops.h"
 #include "vrrp_netlink.h"
 #include "vrrp_if.h"
-#ifdef _WITH_SNMP_
+#ifdef _WITH_SNMP_CHECKER_
   #include "check_snmp.h"
 #endif
 
-extern char *checkers_pidfile;
-
 /* Daemon stop sequence */
 static void
 stop_check(void)
@@ -59,8 +56,8 @@ stop_check(void)
 	if (!__test_bit(DONT_RELEASE_IPVS_BIT, &debug))
 		clear_services();
 	ipvs_stop();
-#ifdef _WITH_SNMP_
-	if (snmp)
+#ifdef _WITH_SNMP_CHECKER_
+	if (global_data->enable_snmp_checker)
 		check_snmp_agent_close();
 #endif
 
@@ -74,7 +71,7 @@ stop_check(void)
 	free_interface_queue();
 #endif
 
-#ifdef _DEBUG_
+#ifdef _MEM_CHECK_
 	keepalived_free_final("Healthcheck child process");
 #endif
 
@@ -82,6 +79,8 @@ stop_check(void)
 	 * Reached when terminate signal catched.
 	 * finally return to parent process.
 	 */
+	log_message(LOG_INFO, "Stopped");
+
 	closelog();
 	exit(0);
 }
@@ -95,15 +94,12 @@ start_check(void)
 		stop_check();
 		return;
 	}
+
 	init_checkers_queue();
 #ifdef _WITH_VRRP_
 	init_interface_queue();
 	kernel_netlink_init();
 #endif
-#ifdef _WITH_SNMP_
-	if (!reload && snmp)
-		check_snmp_agent_init(snmp_socket);
-#endif
 
 	/* Parse configuration file */
 	global_data = alloc_global_data();
@@ -116,7 +112,18 @@ start_check(void)
 	init_global_data(global_data);
 
 	/* Post initializations */
+#ifdef _DEBUG_
 	log_message(LOG_INFO, "Configuration is using : %lu Bytes", mem_allocated);
+#endif
+
+	/* Remove any entries left over from previous invocation */
+	if (!reload && global_data->lvs_flush)
+		ipvs_flush_cmd();
+
+#ifdef _WITH_SNMP_CHECKER_
+	if (!reload && global_data->enable_snmp_checker)
+		check_snmp_agent_init(global_data->snmp_socket);
+#endif
 
 	/* SSL load static data & initialize common ctx context */
 	if (!init_ssl_ctx()) {
@@ -130,6 +137,13 @@ start_check(void)
 	 */
 	link_vsg_to_vs();
 
+	/* Set the process priority and non swappable if configured */
+	if (global_data->checker_process_priority)
+		set_process_priority(global_data->checker_process_priority);
+
+	if (global_data->checker_no_swap)
+		set_process_dont_swap(4096);	/* guess a stack size to reserve */
+
 	/* Processing differential configuration parsing */
 	if (reload)
 		clear_diff_services();
@@ -156,15 +170,16 @@ start_check(void)
 }
 
 /* Reload handler */
-int reload_check_thread(thread_t *);
-void
+static int reload_check_thread(thread_t *);
+
+static void
 sighup_check(void *v, int sig)
 {
 	thread_add_event(master, reload_check_thread, NULL, 0);
 }
 
 /* Terminate handler */
-void
+static void
 sigend_check(void *v, int sig)
 {
 	if (master)
@@ -172,7 +187,7 @@ sigend_check(void *v, int sig)
 }
 
 /* CHECK Child signal handling */
-void
+static void
 check_signal_init(void)
 {
 	signal_handler_init();
@@ -183,7 +198,7 @@ check_signal_init(void)
 }
 
 /* Reload thread */
-int
+static int
 reload_check_thread(thread_t * thread)
 {
 	/* set the reloading flag */
@@ -191,16 +206,11 @@ reload_check_thread(thread_t * thread)
 
 	log_message(LOG_INFO, "Got SIGHUP, reloading checker configuration");
 
-	/* Signals handling */
-	signal_reset();
-	signal_handler_destroy();
-
 	/* Destroy master thread */
 #ifdef _WITH_VRRP_
 	kernel_netlink_close();
 #endif
-	thread_destroy_master(master);
-	master = thread_make_master();
+	thread_cleanup_master(master);
 	free_global_data(global_data);
 	free_checkers_queue();
 #ifdef _WITH_VRRP_
@@ -214,9 +224,9 @@ reload_check_thread(thread_t * thread)
 	check_data = NULL;
 
 	/* Reload the conf */
+#ifdef _DEBUG_
 	mem_allocated = 0;
-	check_signal_init();
-	signal_set(SIGCHLD, thread_child_handler, master);
+#endif
 	start_check();
 
 	/* free backup data */
@@ -227,7 +237,7 @@ reload_check_thread(thread_t * thread)
 }
 
 /* CHECK Child respawning thread */
-int
+static int
 check_respawn_thread(thread_t * thread)
 {
 	pid_t pid;
@@ -283,6 +293,10 @@ start_check_child(void)
 	openlog(PROG_CHECK, LOG_PID | ((__test_bit(LOG_CONSOLE_BIT, &debug)) ? LOG_CONS : 0)
 			  , (log_facility==LOG_DAEMON) ? LOG_LOCAL2 : log_facility);
 
+#ifdef _MEM_CHECK_
+        mem_log_init(PROG_CHECK);
+#endif
+
 	/* Child process part, write pidfile */
 	if (!pidfile_write(checkers_pidfile, getpid())) {
 		log_message(LOG_INFO, "Healthcheck child process: cannot write pidfile");
@@ -291,7 +305,7 @@ start_check_child(void)
 
 	/* Create the new master thread */
 	signal_handler_destroy();
-	thread_destroy_master(master);
+	thread_destroy_master(master);	/* This destroys any residual settings from the parent */
 	master = thread_make_master();
 
 	/* change to / dir */
