@@ -30,20 +30,24 @@
 #include "parser.h"
 #include "utils.h"
 #include "html.h"
+#ifndef _HAVE_SOCK_CLOEXEC_
+#include "old_socket.h"
+#include "string.h"
+#endif
 
-int http_connect_thread(thread_t *);
+static int http_connect_thread(thread_t *);
 
 /* Configuration stream handling */
-void
+static void
 free_url(void *data)
 {
 	url_t *url = data;
-	FREE(url->path);
-	FREE(url->digest);
+	FREE_PTR(url->path);
+	FREE_PTR(url->digest);
 	FREE(url);
 }
 
-void
+static void
 dump_url(void *data)
 {
 	url_t *url = data;
@@ -56,19 +60,34 @@ dump_url(void *data)
 		       url->status_code);
 }
 
-void
+static void
+free_http_request(request_t *req)
+{
+	if(!req)
+		return;
+	if (req->ssl)
+		SSL_free(req->ssl);
+	if (req->buffer)
+		FREE(req->buffer);
+	FREE(req);
+}
+
+static void
 free_http_get_check(void *data)
 {
 	http_checker_t *http_get_chk = CHECKER_DATA(data);
+	http_t *http = HTTP_ARG(http_get_chk);
+	request_t *req = HTTP_REQ(http);
 
-	free_list(http_get_chk->url);
-	FREE(http_get_chk->arg);
-	FREE(http_get_chk);
-	FREE(CHECKER_CO(data));
+	free_list(&http_get_chk->url);
+	free_http_request(req);
+	FREE(http);
+	FREE_PTR(http_get_chk);
+	FREE_PTR(CHECKER_CO(data));
 	FREE(data);
 }
 
-void
+static void
 dump_http_get_check(void *data)
 {
 	http_checker_t *http_get_chk = CHECKER_DATA(data);
@@ -77,7 +96,7 @@ dump_http_get_check(void *data)
 		log_message(LOG_INFO, "   Keepalive method = HTTP_GET");
 	else
 		log_message(LOG_INFO, "   Keepalive method = SSL_GET");
-	dump_conn_opts (CHECKER_GET_CO());
+	dump_conn_opts(CHECKER_CO(data));
 	log_message(LOG_INFO, "   Nb get retry = %d", http_get_chk->nb_get_retry);
 	log_message(LOG_INFO, "   Delay before retry = %lu",
 	       http_get_chk->delay_before_retry/TIMER_HZ);
@@ -99,7 +118,7 @@ alloc_http_get(char *proto)
 	return http_get_chk;
 }
 
-void
+static void
 http_get_handler(vector_t *strvec)
 {
 	http_checker_t *http_get_chk;
@@ -111,21 +130,21 @@ http_get_handler(vector_t *strvec)
 		      http_connect_thread, http_get_chk, CHECKER_NEW_CO());
 }
 
-void
+static void
 nb_get_retry_handler(vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
 	http_get_chk->nb_get_retry = CHECKER_VALUE_INT(strvec);
 }
 
-void
+static void
 delay_before_retry_handler(vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
 	http_get_chk->delay_before_retry = CHECKER_VALUE_INT(strvec) * TIMER_HZ;
 }
 
-void
+static void
 url_handler(vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
@@ -137,7 +156,7 @@ url_handler(vector_t *strvec)
 	list_add(http_get_chk->url, new);
 }
 
-void
+static void
 path_handler(vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
@@ -146,7 +165,7 @@ path_handler(vector_t *strvec)
 	url->path = CHECKER_VALUE_STRING(strvec);
 }
 
-void
+static void
 digest_handler(vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
@@ -155,7 +174,7 @@ digest_handler(vector_t *strvec)
 	url->digest = CHECKER_VALUE_STRING(strvec);
 }
 
-void
+static void
 status_code_handler(vector_t *strvec)
 {
 	http_checker_t *http_get_chk = CHECKER_GET();
@@ -164,53 +183,46 @@ status_code_handler(vector_t *strvec)
 	url->status_code = CHECKER_VALUE_INT(strvec);
 }
 
+static void
+install_http_ssl_check_keyword(const char *keyword)
+{
+	install_keyword(keyword, &http_get_handler);
+	install_sublevel();
+	install_connect_keywords();
+	install_keyword("warmup", &warmup_handler);
+	install_keyword("nb_get_retry", &nb_get_retry_handler);
+	install_keyword("delay_before_retry", &delay_before_retry_handler);
+	install_keyword("url", &url_handler);
+	install_sublevel();
+	install_keyword("path", &path_handler);
+	install_keyword("digest", &digest_handler);
+	install_keyword("status_code", &status_code_handler);
+	install_sublevel_end();
+	install_sublevel_end();
+}
+
 void
 install_http_check_keyword(void)
 {
-	install_keyword("HTTP_GET", &http_get_handler);
-	install_sublevel();
-	install_connect_keywords();
-	install_keyword("warmup", &warmup_handler);
-	install_keyword("nb_get_retry", &nb_get_retry_handler);
-	install_keyword("delay_before_retry", &delay_before_retry_handler);
-	install_keyword("url", &url_handler);
-	install_sublevel();
-	install_keyword("path", &path_handler);
-	install_keyword("digest", &digest_handler);
-	install_keyword("status_code", &status_code_handler);
-	install_sublevel_end();
-	install_sublevel_end();
+	install_http_ssl_check_keyword("HTTP_GET");
 }
 
-/* a little code duplication :/ */
 void
 install_ssl_check_keyword(void)
 {
-	install_keyword("SSL_GET", &http_get_handler);
-	install_sublevel();
-	install_connect_keywords();
-	install_keyword("warmup", &warmup_handler);
-	install_keyword("nb_get_retry", &nb_get_retry_handler);
-	install_keyword("delay_before_retry", &delay_before_retry_handler);
-	install_keyword("url", &url_handler);
-	install_sublevel();
-	install_keyword("path", &path_handler);
-	install_keyword("digest", &digest_handler);
-	install_keyword("status_code", &status_code_handler);
-	install_sublevel_end();
-	install_sublevel_end();
+	install_http_ssl_check_keyword("SSL_GET");
 }
 
-/* 
+/*
  * The global design of this checker is the following :
- * 
+ *
  * - All the actions are done asynchronously.
  * - All the actions handle timeout connection.
  * - All the actions handle error from low layer to upper
  *   layers.
- * 
+ *
  * The global synopsis of the inter-thread-call is :
- *     
+ *
  *     http_connect_thread (handle layer4 connect)
  *            v
  *     http_check_thread (handle SSL connect)
@@ -235,7 +247,7 @@ install_ssl_check_keyword(void)
  * method == 1 => register a new checker thread
  * method == 2 => register a retry on url checker thread
  */
-int
+static int
 epilog(thread_t * thread, int method, int t, int c)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -249,21 +261,42 @@ epilog(thread_t * thread, int method, int t, int c)
 		http->retry_it += c ? c : -http->retry_it;
 	}
 
+	if (method == 1 && http->url_it >= LIST_SIZE(http_get_check->url)) {
+		/* All the url have been successfully checked.
+		 * Check completed.
+		 * check if server is currently alive.
+		 */
+		if (!svr_checker_up(checker->id, checker->rs)) {
+			log_message(LOG_INFO, "Remote Web server %s succeed on service."
+					    , FMT_HTTP_RS(checker));
+			smtp_alert(checker->rs, NULL, NULL, "UP",
+				   "=> CHECK succeed on service <=");
+			update_svr_checker_state(UP, checker->id
+						   , checker->vs
+						   , checker->rs);
+		}
+
+		/* Reset it counters */
+		http->url_it = 0;
+		http->retry_it = 0;
+	}
 	/*
 	 * The get retry implementation mean that we retry performing
-	 * a GET on the same url until the remote web server return 
+	 * a GET on the same url until the remote web server return
 	 * html buffer. This is sometime needed with some applications
 	 * servers.
 	 */
-	if (http->retry_it > http_get_check->nb_get_retry-1) {
+	else if (method == 2 && http->retry_it > http_get_check->nb_get_retry) {
 		if (svr_checker_up(checker->id, checker->rs)) {
-			log_message(LOG_INFO, "Check on service %s failed after %d retry."
-			       , FMT_HTTP_RS(checker)
-			       , http->retry_it);
+			if (http_get_check->nb_get_retry)
+				log_message(LOG_INFO
+				   , "Check on service %s failed after %d retry."
+				   , FMT_HTTP_RS(checker)
+				   , http->retry_it - 1);
 			smtp_alert(checker->rs, NULL, NULL,
 				   "DOWN",
 				   "=> CHECK failed on service"
-				   " : MD5 digest mismatch <=");
+				   " : HTTP request failed <=");
 			update_svr_checker_state(DOWN, checker->id
 						     , checker->vs
 						     , checker->rs);
@@ -277,11 +310,7 @@ epilog(thread_t * thread, int method, int t, int c)
 	/* register next timer thread */
 	switch (method) {
 	case 1:
-		if (req)
-			delay = checker->vs->delay_loop;
-		else
-			delay =
-			    http_get_check->delay_before_retry;
+		delay = checker->vs->delay_loop;
 		break;
 	case 2:
 		if (http->url_it == 0 && http->retry_it == 0)
@@ -293,11 +322,7 @@ epilog(thread_t * thread, int method, int t, int c)
 
 	/* If req == NULL, fd is not created */
 	if (req) {
-		if (req->ssl)
-			SSL_free(req->ssl);
-		if (req->buffer)
-			FREE(req->buffer);
-		FREE(req);
+		free_http_request(req);
 		http->req = NULL;
 		close(thread->u.fd);
 	}
@@ -308,28 +333,24 @@ epilog(thread_t * thread, int method, int t, int c)
 }
 
 int
-timeout_epilog(thread_t * thread, char *smtp_msg, char *debug_msg)
+timeout_epilog(thread_t * thread, const char *debug_msg)
 {
 	checker_t *checker = THREAD_ARG(thread);
 
-
 	/* check if server is currently alive */
 	if (svr_checker_up(checker->id, checker->rs)) {
-		log_message(LOG_INFO, "Timeout %s server %s."
+		log_message(LOG_INFO, "%s server %s."
 				    , debug_msg
 				    , FMT_HTTP_RS(checker));
-		smtp_alert(checker->rs, NULL, NULL,
-			   "DOWN", smtp_msg);
-		update_svr_checker_state(DOWN, checker->id
-					     , checker->vs
-					     , checker->rs);
+		return epilog(thread, 2, 0, 1);
 	}
 
+	/* do not retry if server is already known as dead */
 	return epilog(thread, 1, 0, 0);
 }
 
 /* return the url pointer of the current url iterator  */
-url_t *
+static url_t *
 fetch_next_url(http_checker_t * http_get_check)
 {
 	http_t *http = HTTP_ARG(http_get_check);
@@ -350,51 +371,25 @@ http_handle_response(thread_t * thread, unsigned char digest[16]
 	char *digest_tmp;
 	url_t *fetched_url = fetch_next_url(http_get_check);
 	enum {
-		none,
-		on_status,
-		on_digest
-	} last_success = none; /* the source of last considered success */
+		NONE,
+		ON_SUCCESS,
+		ON_STATUS,
+		ON_DIGEST
+	} last_success = NONE; /* the source of last considered success */
 
 	/* First check if remote webserver returned data */
 	if (empty_buffer)
-		return timeout_epilog(thread, "=> CHECK failed on service"
-				      " : empty buffer received <=\n\n",
-				      "Read, no data received from ");
+		return timeout_epilog(thread, "Read, no data received from ");
 
 	/* Next check the HTTP status code */
 	if (fetched_url->status_code) {
-		if (req->status_code != fetched_url->status_code) {
-			/* check if server is currently alive */
-			if (svr_checker_up(checker->id, checker->rs)) {
-				log_message(LOG_INFO,
-				       "HTTP status code error to %s url(%s)"
-				       ", status_code [%d].",
-				       FMT_HTTP_RS(checker),
-				       fetched_url->path,
-				       req->status_code);
-				smtp_alert(checker->rs, NULL, NULL,
-					   "DOWN",
-					   "=> CHECK failed on service"
-					   " : HTTP status code mismatch <=");
-				update_svr_checker_state(DOWN, checker->id
-							     , checker->vs
-							     , checker->rs);
-			} else {
-				DBG("HTTP Status_code to %s url(%d) = [%d]."
-				    , FMT_HTTP_RS(checker)
-				    , http->url_it + 1
-				    , req->status_code);
-				/*
-				 * We set retry iterator to max value to not retry
-				 * when service is already know as die.
-				 */
-				http->retry_it = http_get_check->nb_get_retry;
-			}
-			return epilog(thread, 2, 0, 1);
-		} else {
-			last_success = on_status;
-		}
+		if (req->status_code != fetched_url->status_code)
+			return timeout_epilog(thread, "HTTP status code error to");
+
+		last_success = ON_STATUS;
 	}
+	else if (req->status_code >= 200 && req->status_code <= 299)
+		last_success = ON_SUCCESS;
 
 	/* Continue with MD5SUM */
 	if (fetched_url->digest) {
@@ -404,57 +399,34 @@ http_handle_response(thread_t * thread, unsigned char digest[16]
 			sprintf(digest_tmp + 2 * di, "%02x", digest[di]);
 
 		r = strcmp(fetched_url->digest, digest_tmp);
+		FREE(digest_tmp);
 
-		if (r) {
-			/* check if server is currently alive */
-			if (svr_checker_up(checker->id, checker->rs)) {
-				log_message(LOG_INFO,
-				       "MD5 digest error to %s url[%s]"
-				       ", MD5SUM [%s].",
-				       FMT_HTTP_RS(checker),
-				       fetched_url->path,
-				       digest_tmp);
-				smtp_alert(checker->rs, NULL, NULL,
-					   "DOWN",
-					   "=> CHECK failed on service"
-					   " : HTTP MD5SUM mismatch <=");
-				update_svr_checker_state(DOWN, checker->id
-							     , checker->vs
-							     , checker->rs);
-			} else {
-				DBG("MD5SUM to %s url(%d) = [%s]."
-				    , FMT_HTTP_RS(checker)
-				    , http->url_it + 1
-				    , digest_tmp);
-				/*
-				 * We set retry iterator to max value to not retry
-				 * when service is already know as die.
-				 */
-				http->retry_it = http_get_check->nb_get_retry;
-			}
-			FREE(digest_tmp);
-			return epilog(thread, 2, 0, 1);
-		} else {
-			last_success = on_digest;
-			FREE(digest_tmp);
-		}
+		if (r)
+			return timeout_epilog(thread, "MD5 digest error to");
+		last_success = ON_DIGEST;
 	}
 
 	if (!svr_checker_up(checker->id, checker->rs)) {
 		switch (last_success) {
-			case none:
+			case NONE:
 				break;
-			case on_status:
+			case ON_SUCCESS:
+				log_message(LOG_INFO,
+				       "HTTP success to %s url(%d)."
+				       , FMT_HTTP_RS(checker)
+				       , http->url_it + 1);
+				return epilog(thread, 1, 1, 0) + 1;
+			case ON_STATUS:
 				log_message(LOG_INFO,
 				       "HTTP status code success to %s url(%d)."
 				       , FMT_HTTP_RS(checker)
 				       , http->url_it + 1);
 				return epilog(thread, 1, 1, 0) + 1;
-			case on_digest:
-				if (!svr_checker_up(checker->id, checker->rs))
-					log_message(LOG_INFO, "MD5 digest success to %s url(%d)."
-						   , FMT_HTTP_RS(checker)
-						   , http->url_it + 1);
+			case ON_DIGEST:
+				log_message(LOG_INFO,
+					"MD5 digest success to %s url(%d)."
+					, FMT_HTTP_RS(checker)
+					, http->url_it + 1);
 				return epilog(thread, 1, 1, 0) + 1;
 		}
 	}
@@ -464,7 +436,7 @@ http_handle_response(thread_t * thread, unsigned char digest[16]
 
 /* Handle response stream performing MD5 updates */
 int
-http_process_response(request_t *req, int r)
+http_process_response(request_t *req, int r, int do_md5)
 {
 	req->len += r;
 	if (!req->extracted) {
@@ -472,12 +444,9 @@ http_process_response(request_t *req, int r)
 		     extract_html(req->buffer, req->len))) {
 			req->status_code = extract_status_code(req->buffer, req->len);
 			r = req->len - (req->extracted - req->buffer);
-			if (r) {
-				memmove(req->buffer, req->extracted, r);
-				MD5_Update(&req->context, req->buffer, r);
-				r = 0;
-			}
-			req->len = r;
+			if (r && do_md5)
+				MD5_Update(&req->context, req->extracted, r);
+			req->len = 0;
 		}
 	} else if (req->len) {
 		MD5_Update(&req->context, req->buffer,
@@ -489,13 +458,14 @@ http_process_response(request_t *req, int r)
 }
 
 /* Asynchronous HTTP stream reader */
-int
+static int
 http_read_thread(thread_t * thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	http_checker_t *http_get_check = CHECKER_ARG(checker);
 	http_t *http = HTTP_ARG(http_get_check);
 	request_t *req = HTTP_REQ(http);
+	url_t *url = list_element(http_get_check->url, http->url_it);
 	unsigned timeout = checker->co->connection_to;
 	unsigned char digest[16];
 	int r = 0;
@@ -503,8 +473,7 @@ http_read_thread(thread_t * thread)
 
 	/* Handle read timeout */
 	if (thread->type == THREAD_READ_TIMEOUT)
-		return timeout_epilog(thread, "=> HTTP CHECK failed on service"
-				      " : recevice data <=\n\n", "HTTP read");
+		return timeout_epilog(thread, "Timeout HTTP read");
 
 	/* Set descriptor non blocking */
 	val = fcntl(thread->u.fd, F_GETFL, 0);
@@ -530,23 +499,12 @@ http_read_thread(thread_t * thread)
 	if (r == -1 || r == 0) {	/* -1:error , 0:EOF */
 
 		/* All the HTTP stream has been parsed */
-		MD5_Final(digest, &req->context);
+		if (url->digest)
+			MD5_Final(digest, &req->context);
 
 		if (r == -1) {
 			/* We have encourred a real read error */
-			if (svr_checker_up(checker->id, checker->rs)) {
-				log_message(LOG_INFO, "Read error with server %s: %s"
-				       , FMT_HTTP_RS(checker)
-				       , strerror(errno));
-				smtp_alert(checker->rs, NULL, NULL,
-					   "DOWN",
-					   "=> HTTP CHECK failed on service"
-					   " : cannot receive data <=");
-				update_svr_checker_state(DOWN, checker->id
-							     , checker->vs
-							     , checker->rs);
-			}
-			return epilog(thread, 1, 0, 0);
+			return timeout_epilog(thread, "Read error with");
 		}
 
 		/* Handle response stream */
@@ -555,7 +513,7 @@ http_read_thread(thread_t * thread)
 	} else {
 
 		/* Handle response stream */
-		http_process_response(req, r);
+		http_process_response(req, r, (url->digest != NULL));
 
 		/*
 		 * Register next http stream reader.
@@ -572,26 +530,27 @@ http_read_thread(thread_t * thread)
  * Read get result from the remote web server.
  * Apply trigger check to this result.
  */
-int
+static int
 http_response_thread(thread_t * thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	http_checker_t *http_get_check = CHECKER_ARG(checker);
 	http_t *http = HTTP_ARG(http_get_check);
 	request_t *req = HTTP_REQ(http);
+	url_t *url = list_element(http_get_check->url, http->url_it);
 	unsigned timeout = checker->co->connection_to;
 
 	/* Handle read timeout */
 	if (thread->type == THREAD_READ_TIMEOUT)
-		return timeout_epilog(thread, "=> CHECK failed on service"
-				      " : recevice data <=\n\n", "WEB read");
+		return timeout_epilog(thread, "Timeout WEB read");
 
 	/* Allocate & clean the get buffer */
 	req->buffer = (char *) MALLOC(MAX_BUFFER_LENGTH);
 	req->extracted = NULL;
 	req->len = 0;
 	req->error = 0;
-	MD5_Init(&req->context);
+	if (url->digest)
+		MD5_Init(&req->context);
 
 	/* Register asynchronous http/ssl read thread */
 	if (http_get_check->proto == PROTO_SSL)
@@ -604,7 +563,7 @@ http_response_thread(thread_t * thread)
 }
 
 /* remote Web server is connected, send it the get url query.  */
-int
+static int
 http_request_thread(thread_t * thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -623,9 +582,7 @@ http_request_thread(thread_t * thread)
 
 	/* Handle read timeout */
 	if (thread->type == THREAD_WRITE_TIMEOUT)
-		return timeout_epilog(thread, "=> CHECK failed on service"
-				      " : read timeout <=\n\n",
-				      "Web read, timeout");
+		return timeout_epilog(thread, "Timeout WEB read");
 
 	/* Allocate & clean the GET string */
 	str_request = (char *) MALLOC(GET_BUFFER_LENGTH);
@@ -642,7 +599,7 @@ http_request_thread(thread_t * thread)
 
 		/* Allocate a buffer for the port string ( ":" [0-9][0-9][0-9][0-9][0-9] "\0" ) */
 		request_host_port = (char*) MALLOC(7);
-		snprintf(request_host_port, 7, ":%d", 
+		snprintf(request_host_port, 7, ":%d",
 			 ntohs(inet_sockaddrport(addr)));
 	}
 
@@ -680,20 +637,7 @@ http_request_thread(thread_t * thread)
 	FREE(str_request);
 
 	if (!ret) {
-		log_message(LOG_INFO, "Cannot send get request to %s."
-				    , FMT_HTTP_RS(checker));
-
-		/* check if server is currently alive */
-		if (svr_checker_up(checker->id, checker->rs)) {
-			smtp_alert(checker->rs, NULL, NULL,
-				   "DOWN",
-				   "=> CHECK failed on service"
-				   " : cannot send data <=");
-			update_svr_checker_state(DOWN, checker->id
-						     , checker->vs
-						     , checker->rs);
-		}
-		return epilog(thread, 1, 0, 0);
+		return timeout_epilog(thread, "Cannot send get request to");
 	}
 
 	/* Register read timeouted thread */
@@ -721,25 +665,11 @@ http_check_thread(thread_t * thread)
 	status = tcp_socket_state(thread->u.fd, thread, http_check_thread);
 	switch (status) {
 	case connect_error:
-		/* check if server is currently alive */
-		if (svr_checker_up(checker->id, checker->rs)) {
-			log_message(LOG_INFO, "Error connecting server %s."
-					 , FMT_HTTP_RS(checker));
-			smtp_alert(checker->rs, NULL, NULL,
-				   "DOWN",
-				   "=> CHECK failed on service"
-				   " : connection error <=");
-			update_svr_checker_state(DOWN, checker->id
-						 , checker->vs
-						 , checker->rs);
-		}
-		return epilog(thread, 1, 0, 0);
+		return timeout_epilog(thread, "Error connecting");
 		break;
 
 	case connect_timeout:
-		return timeout_epilog(thread, "==> CHECK failed on service"
-				      " : connection timeout <=\n\n",
-				      "connect, timeout");
+		return timeout_epilog(thread, "Timeout connecting");
 		break;
 
 	case connect_success:{
@@ -755,9 +685,7 @@ http_check_thread(thread_t * thread)
 				    thread->type != THREAD_READ_TIMEOUT)
 					ret = ssl_connect(thread, new_req);
 				else {
-					return timeout_epilog(thread, "==> CHECK failed on service"
-							      " : connection timeout <=\n\n",
-							      "connect, timeout");
+					return timeout_epilog(thread, "Timeout connecting");
 				}
 
 				if (ret == -1) {
@@ -802,23 +730,8 @@ http_check_thread(thread_t * thread)
 					ssl_printerr(SSL_get_error
 						     (req->ssl, ret));
 #endif
-				if ((http_get_check->proto == PROTO_SSL) &&
-				    (svr_checker_up(checker->id, checker->rs))) {
-					log_message(LOG_INFO, "SSL handshake/communication error"
-							 " connecting to server"
-							 " (openssl errno: %d) %s."
-						       , SSL_get_error (http->req->ssl, ret)
-						       , FMT_HTTP_RS(checker));
-					smtp_alert(checker->rs, NULL, NULL,
-						   "DOWN",
-						   "=> CHECK failed on service"
-						   " : SSL connection error <=");
-					update_svr_checker_state(DOWN, checker->id
-								 , checker->vs
-								 , checker->rs);
-				}
-
-				return epilog(thread, 1, 0, 0);
+				return timeout_epilog(thread, "SSL handshake/communication error"
+							 " connecting to");
 			}
 		}
 		break;
@@ -827,12 +740,11 @@ http_check_thread(thread_t * thread)
 	return 0;
 }
 
-int
+static int
 http_connect_thread(thread_t * thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	http_checker_t *http_get_check = CHECKER_ARG(checker);
-	http_t *http = HTTP_ARG(http_get_check);
 	conn_opts_t *co = checker->co;
 	url_t *fetched_url;
 	enum connect_result status;
@@ -848,29 +760,13 @@ http_connect_thread(thread_t * thread)
 		return 0;
 	}
 
-	/* Find eventual url end */
+	/* if there are no URLs in list, enable server w/o checking */
 	fetched_url = fetch_next_url(http_get_check);
-
-	if (!fetched_url) {
-		/* All the url have been successfully checked.
-		 * Check completed.
-		 * check if server is currently alive.
-		 */
-		if (!svr_checker_up(checker->id, checker->rs)) {
-			log_message(LOG_INFO, "Remote Web server %s succeed on service."
-					    , FMT_HTTP_RS(checker));
-			smtp_alert(checker->rs, NULL, NULL, "UP",
-				   "=> CHECK succeed on service <=");
-			update_svr_checker_state(UP, checker->id
-						   , checker->vs
-						   , checker->rs);
-		}
-		http->req = NULL;
-		return epilog(thread, 1, 0, 0) + 1;
-	}
+	if (!fetched_url)
+		return epilog(thread, 1, 1, 0) + 1;
 
 	/* Create the socket */
-	if ((fd = socket(co->dst.ss_family, SOCK_STREAM, IPPROTO_TCP)) == -1) {
+	if ((fd = socket(co->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC, IPPROTO_TCP)) == -1) {
 		log_message(LOG_INFO, "WEB connection fail to create socket. Rescheduling.");
 		thread_add_timer(thread->master, http_connect_thread, checker,
 				checker->vs->delay_loop);
@@ -878,6 +774,11 @@ http_connect_thread(thread_t * thread)
 		return 0;
 	}
 
+#ifndef _HAVE_SOCK_CLOEXEC_
+	if (set_sock_flags(fd, F_SETFD, FD_CLOEXEC))
+		log_message(LOG_INFO, "Unable to set CLOEXEC on http_connect socket - %s (%d)", strerror(errno), errno);
+#endif
+
 	status = tcp_bind_connect(fd, co);
 
 	/* handle tcp connection status & register check worker thread */
