@@ -31,9 +31,13 @@
 #include "utils.h"
 #include "parser.h"
 #include "daemon.h"
+#ifndef _HAVE_SOCK_CLOEXEC_
+#include "old_socket.h"
+#include "string.h"
+#endif
 
-int smtp_connect_thread(thread_t *);
-int smtp_final(thread_t *thread, int error, const char *format, ...)
+static int smtp_connect_thread(thread_t *);
+static int smtp_final(thread_t *thread, int error, const char *format, ...)
 	 __attribute__ ((format (printf, 3, 4)));
 
 /*
@@ -41,20 +45,20 @@ int smtp_final(thread_t *thread, int error, const char *format, ...)
  * the list elements in smtp_checker->host before we
  * free smtp_checker itself.
  */
-void
+static void
 smtp_free_host(void *data)
 {
-        FREE(data);
+	FREE(data);
 }
 
 /* Used as a callback from the checker api, queue_checker(),
  * to free up a checker entry and all its associated data.
  */
-void
+static void
 free_smtp_check(void *data)
 {
-  	smtp_checker_t *smtp_checker = CHECKER_DATA(data);
-	free_list(smtp_checker->host);
+	smtp_checker_t *smtp_checker = CHECKER_DATA(data);
+	free_list(&smtp_checker->host);
 	FREE(smtp_checker->helo_name);
 	FREE(smtp_checker->default_co);
 	FREE(smtp_checker);
@@ -62,20 +66,10 @@ free_smtp_check(void *data)
 }
 
 /*
- * Used as a callback from dump_list() to print out all
- * the list elements in smtp_checker->host.
- */
-void
-smtp_dump_host(void *data)
-{
-	dump_conn_opts (data);
-}
-
-/* 
  * Callback for whenever we've been requested to dump our
  * configuration.
  */
-void
+static void
 dump_smtp_check(void *data)
 {
 	smtp_checker_t *smtp_checker = CHECKER_DATA(data);
@@ -87,7 +81,7 @@ dump_smtp_check(void *data)
 }
 
 /* Allocates a default host structure */
-smtp_host_t *
+static smtp_host_t *
 smtp_alloc_host(void)
 {
 	smtp_host_t *new;
@@ -105,16 +99,16 @@ smtp_alloc_host(void)
 	return new;
 }
 
-/* 
+/*
  * Callback for whenever an SMTP_CHECK keyword is encountered
- * in the config file. 
+ * in the config file.
  */
-void
+static void
 smtp_check_handler(vector_t *strvec)
 {
 	smtp_checker_t *smtp_checker = (smtp_checker_t *)MALLOC(sizeof(smtp_checker_t));
 
-	/* 
+	/*
 	 * Set something sane for the default HELO banner
 	 * May be overridden by a "helo_name" keyword later.
 	 */
@@ -150,10 +144,11 @@ smtp_check_handler(vector_t *strvec)
 	 * be used instead of the default, but all the uninitialized options
 	 * of those hosts will be set to the default's values.
 	 */
-	smtp_checker->host = alloc_list(smtp_free_host, smtp_dump_host);
+	smtp_checker->host = alloc_list(smtp_free_host, dump_conn_opts);
 }
 
-void smtp_check_end_handler(void)
+static void
+smtp_check_end_handler(void)
 {
 	smtp_checker_t *smtp_checker = CHECKER_GET();
 
@@ -165,21 +160,21 @@ void smtp_check_end_handler(void)
 		list_add(smtp_checker->host, smtp_alloc_host());
 }
 
-/* 
+/*
  * Callback for whenever the "host" keyword is encountered
- * in the config file. 
+ * in the config file.
  */
-void
+static void
 smtp_host_handler(vector_t *strvec)
 {
-        smtp_checker_t *smtp_checker = CHECKER_GET();
+	smtp_checker_t *smtp_checker = CHECKER_GET();
 
-        /* add an empty host to the list, smtp_checker->host */
-        list_add(smtp_checker->host, smtp_alloc_host());
+	/* add an empty host to the list, smtp_checker->host */
+	list_add(smtp_checker->host, smtp_alloc_host());
 }
 
 /* "helo_name" keyword */
-void
+static void
 smtp_helo_name_handler(vector_t *strvec)
 {
 	smtp_checker_t *smtp_checker = CHECKER_GET();
@@ -187,18 +182,18 @@ smtp_helo_name_handler(vector_t *strvec)
 }
 
 /* "retry" keyword */
-void
+static void
 smtp_retry_handler(vector_t *strvec)
 {
-        smtp_checker_t *smtp_checker = CHECKER_GET();
+	smtp_checker_t *smtp_checker = CHECKER_GET();
 	smtp_checker->retry = CHECKER_VALUE_INT(strvec);
 }
 
 /* "delay_before_retry" keyword */
-void
+static void
 smtp_db_retry_handler(vector_t *strvec)
 {
-        smtp_checker_t *smtp_checker = CHECKER_GET();
+	smtp_checker_t *smtp_checker = CHECKER_GET();
 	smtp_checker->db_retry = CHECKER_VALUE_INT(strvec) * TIMER_HZ;
 }
 
@@ -206,11 +201,11 @@ smtp_db_retry_handler(vector_t *strvec)
 void
 install_smtp_check_keyword(void)
 {
-        /* 
-         * Notify the config log parser that we need to be notified via
+	/*
+	 * Notify the config log parser that we need to be notified via
 	 * callbacks when the following keywords are encountered in the
 	 * keepalive.conf file.
-         */
+	 */
 	install_keyword("SMTP_CHECK", &smtp_check_handler);
 	install_sublevel();
 	install_keyword("helo_name", &smtp_helo_name_handler);
@@ -237,11 +232,11 @@ install_smtp_check_keyword(void)
 }
 
 /*
- * Final handler. Determines if we need a retry or not. 
+ * Final handler. Determines if we need a retry or not.
  * Also has to make a decision if we need to bring the resulting
  * service down in case of error.
  */
-int
+static int
 smtp_final(thread_t *thread, int error, const char *format, ...)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -255,10 +250,10 @@ smtp_final(thread_t *thread, int error, const char *format, ...)
 
 	/* If we're here, an attempt HAS been made already for the current host */
 	smtp_checker->attempts++;
-	
+
 	if (error) {
 		/* Always syslog the error when the real server is up */
-                if (svr_checker_up(checker->id, checker->rs)) {
+		if (svr_checker_up(checker->id, checker->rs)) {
 			if (format != NULL) {
 				/* prepend format with the "SMTP_CHECK " string */
 				error_buff[0] = '\0';
@@ -290,7 +285,7 @@ smtp_final(thread_t *thread, int error, const char *format, ...)
 		 * be noted that smtp_alert makes a copy of the string arguments, so
 		 * we don't have to keep them statically allocated.
 		 */
-                if (svr_checker_up(checker->id, checker->rs)) {
+		if (svr_checker_up(checker->id, checker->rs)) {
 			if (format != NULL) {
 				snprintf(smtp_buff, 542, "=> CHECK failed on service : %s <=",
 					 error_buff + 11);
@@ -311,7 +306,7 @@ smtp_final(thread_t *thread, int error, const char *format, ...)
 		thread_add_timer(thread->master, smtp_connect_thread, checker, checker->vs->delay_loop);
 
 		return 0;
-	}	
+	}
 
 	/*
 	 * Ok this host was successful, increment to the next host in the list
@@ -326,14 +321,14 @@ smtp_final(thread_t *thread, int error, const char *format, ...)
 	return 0;
 }
 
-/* 
+/*
  * Zeros out the rx/tx buffer
  */
-void
+static void
 smtp_clear_buff(thread_t *thread)
 {
-        checker_t *checker = THREAD_ARG(thread);
-        smtp_checker_t *smtp_checker = CHECKER_ARG(checker);
+	checker_t *checker = THREAD_ARG(thread);
+	smtp_checker_t *smtp_checker = CHECKER_ARG(checker);
 	memset(smtp_checker->buff, 0, SMTP_BUFF_MAX);
 	smtp_checker->buff_ctr = 0;
 }
@@ -344,7 +339,7 @@ smtp_clear_buff(thread_t *thread)
  * single packet, but we don't care. We are only looking at the
  * SMTP response codes at the beginning anyway.
  */
-int
+static int
 smtp_get_line_cb(thread_t *thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -352,8 +347,8 @@ smtp_get_line_cb(thread_t *thread)
 	smtp_host_t *smtp_host = smtp_checker->host_ptr;
 	int f, r, x;
 
-        /* Handle read timeout */
-        if (thread->type == THREAD_READ_TIMEOUT) {
+	/* Handle read timeout */
+	if (thread->type == THREAD_READ_TIMEOUT) {
 		smtp_final(thread, 1, "Read timeout from server %s"
 				    , FMT_SMTP_RS(smtp_host));
 		return 0;
@@ -378,13 +373,13 @@ smtp_get_line_cb(thread_t *thread)
 	if (r == -1 && (errno == EAGAIN || errno == EINTR)) {
 		thread_add_read(thread->master, smtp_get_line_cb, checker,
 				thread->u.fd, smtp_host->connection_to);
-        	fcntl(thread->u.fd, F_SETFL, f);
+		fcntl(thread->u.fd, F_SETFL, f);
 		return 0;
 	} else if (r > 0)
 		smtp_checker->buff_ctr += r;
 
-        /* restore descriptor flags */
-        fcntl(thread->u.fd, F_SETFL, f);
+	/* restore descriptor flags */
+	fcntl(thread->u.fd, F_SETFL, f);
 
 	/* check if we have a newline, if so, callback */
 	for (x = 0; x < SMTP_BUFF_MAX; x++) {
@@ -421,7 +416,7 @@ smtp_get_line_cb(thread_t *thread)
 	return 0;
 }
 
-/* 
+/*
  * Ok a caller has asked us to asyncronously schedule a single line
  * to be received from the server. They have also passed us a call back
  * function that we'll call once we have the newline. If something bad
@@ -430,7 +425,7 @@ smtp_get_line_cb(thread_t *thread)
  * function smtp_get_line_cb is what does the dirty work since the
  * sceduler can only accept a single *thread argument.
  */
-void
+static void
 smtp_get_line(thread_t *thread, int (*callback) (thread_t *))
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -453,9 +448,9 @@ smtp_get_line(thread_t *thread, int (*callback) (thread_t *))
  * The scheduler function that puts the data out on the wire.
  * All our data will fit into one packet, so we only check if
  * the current write would block or not. If it wants to block,
- * we'll return to the scheduler and try again later. 
+ * we'll return to the scheduler and try again later.
  */
-int
+static int
 smtp_put_line_cb(thread_t *thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -464,29 +459,29 @@ smtp_put_line_cb(thread_t *thread)
 	int f, w;
 
 
-        /* Handle read timeout */
-        if (thread->type == THREAD_WRITE_TIMEOUT) {
+	/* Handle read timeout */
+	if (thread->type == THREAD_WRITE_TIMEOUT) {
 		smtp_final(thread, 1, "Write timeout to server %s"
 				     , FMT_SMTP_RS(smtp_host));
 		return 0;
 	}
 
 	/* Set descriptor non blocking */
-        f = fcntl(thread->u.fd, F_GETFL, 0);
-        fcntl(thread->u.fd, F_SETFL, f | O_NONBLOCK);
+	f = fcntl(thread->u.fd, F_GETFL, 0);
+	fcntl(thread->u.fd, F_SETFL, f | O_NONBLOCK);
 
-        /* write the data */
-        w = write(thread->u.fd, smtp_checker->buff, smtp_checker->buff_ctr);
+	/* write the data */
+	w = write(thread->u.fd, smtp_checker->buff, smtp_checker->buff_ctr);
 
 	if (w == -1 && (errno == EAGAIN || errno == EINTR)) {
 		thread_add_write(thread->master, smtp_put_line_cb, checker,
 				 thread->u.fd, smtp_host->connection_to);
-        	fcntl(thread->u.fd, F_SETFL, f);
+		fcntl(thread->u.fd, F_SETFL, f);
 		return 0;
 	}
 
-        /* restore descriptor flags */
-        fcntl(thread->u.fd, F_SETFL, f);
+	/* restore descriptor flags */
+	fcntl(thread->u.fd, F_SETFL, f);
 
 	DBG("SMTP_CHECK %s > %s"
 	    , FMT_SMTP_RS(smtp_host)
@@ -507,11 +502,11 @@ smtp_put_line_cb(thread_t *thread)
 	return 0;
 }
 
-/* 
+/*
  * This is the same as smtp_get_line() except that we're sending a
  * line of data instead of receiving one.
  */
-void
+static void
 smtp_put_line(thread_t *thread, int (*callback) (thread_t *))
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -535,17 +530,17 @@ smtp_put_line(thread_t *thread, int (*callback) (thread_t *))
  * buffer and return it as an integer. If it's not legible,
  * return -1.
  */
-int
+static int
 smtp_get_status(thread_t *thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	smtp_checker_t *smtp_checker = CHECKER_ARG(checker);
 	char *buff = smtp_checker->buff;
 
-	/* First make sure they're all digits */	
+	/* First make sure they're all digits */
 	if (isdigit(buff[0]) && isdigit(buff[1]) &&
 	    isdigit(buff[2])) {
-		/* Truncate the string and convert */	
+		/* Truncate the string and convert */
 		buff[3] = '\0';
 		return atoi(buff);
 	}
@@ -553,14 +548,14 @@ smtp_get_status(thread_t *thread)
 	return -1;
 }
 
-/* 
- * We have a connected socket and are ready to begin 
- * the conversation. This function schedules itself to 
- * be called via callbacks and tracking state in 
- * smtp_checker->state. Upon first calling, smtp_checker->state 
+/*
+ * We have a connected socket and are ready to begin
+ * the conversation. This function schedules itself to
+ * be called via callbacks and tracking state in
+ * smtp_checker->state. Upon first calling, smtp_checker->state
  * should be set to SMTP_START.
  */
-int
+static int
 smtp_engine_thread(thread_t *thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -643,12 +638,12 @@ smtp_engine_thread(thread_t *thread)
 	smtp_final(thread, 1, "Unknown smtp engine state encountered");
 	return 0;
 }
-		
-/* 
+
+/*
  * Second step in the process. Here we'll see if the connection
  * to the host we're checking was successful or not.
  */
-int
+static int
 smtp_check_thread(thread_t *thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -681,23 +676,23 @@ smtp_check_thread(thread_t *thread)
 			break;
 	}
 
-	/* we shouldn't be here */		
+	/* we shouldn't be here */
 	smtp_final(thread, 1, "Unknown connection error to server %s"
 			     , FMT_SMTP_RS(smtp_host));
 	return 0;
 }
 
-/* 
+/*
  * This is the main thread, where all the action starts.
  * When the check daemon comes up, it goes down the checkers_queue
  * and launches a thread for each checker that got registered.
  * This is the callback/event function for that initial thread.
  *
- * It should be noted that we ARE responsible for sceduling 
+ * It should be noted that we ARE responsible for sceduling
  * ourselves to run again. It doesn't have to be right here,
  * but eventually has to happen.
  */
-int
+static int
 smtp_connect_thread(thread_t *thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
@@ -712,22 +707,22 @@ smtp_connect_thread(thread_t *thread)
 	 * for sceduling many types of events. thread->arg in this
 	 * case points to a checker structure. The checker
 	 * structure holds data about the vs and rs configurations
-	 * as well as the delay loop, etc. Each real server 
-         * defined in the keepalived.conf will more than likely have
+	 * as well as the delay loop, etc. Each real server
+	 * defined in the keepalived.conf will more than likely have
 	 * a checker structure assigned to it. Each checker structure
-         * has a data element that is meant to hold per checker 
+	 * has a data element that is meant to hold per checker
 	 * configurations. So thread->arg(checker)->data points to
- 	 * a smtp_checker structure. In the smtp_checker structure
+	 * a smtp_checker structure. In the smtp_checker structure
 	 * we hold global configuration data for the smtp check.
 	 * Smtp_checker has a list of per host (smtp_host) configuration
 	 * data in smtp_checker->host.
 	 *
 	 * So this whole thing looks like this:
 	 * thread->arg(checker)->data(smtp_checker)->host(smtp_host)
-	 * 
+	 *
 	 * To make life simple, we'll break the structures out so
 	 * that "checker" always points to the current checker structure,
-	 * "smtp_checker" points to the current smtp_checker structure, 
+	 * "smtp_checker" points to the current smtp_checker structure,
 	 * and "smtp_host" points to the current smtp_host structure.
 	 */
 
@@ -743,7 +738,7 @@ smtp_connect_thread(thread_t *thread)
 	}
 
 	/*
-	 * Set the internal host pointer to the host that well be 
+	 * Set the internal host pointer to the host that well be
 	 * working on. If it's NULL, we've successfully tested all hosts.
 	 * We'll bring the service up (if it's not already), reset the host list,
 	 * and insert the delay loop. When we get scheduled again the host list
@@ -770,12 +765,16 @@ smtp_connect_thread(thread_t *thread)
 	smtp_host = smtp_checker->host_ptr;
 
 	/* Create the socket, failling here should be an oddity */
-	if ((sd = socket(smtp_host->dst.ss_family, SOCK_STREAM, IPPROTO_TCP)) == -1) {
+	if ((sd = socket(smtp_host->dst.ss_family, SOCK_STREAM | SOCK_CLOEXEC, IPPROTO_TCP)) == -1) {
 		log_message(LOG_INFO, "SMTP_CHECK connection failed to create socket. Rescheduling.");
 		thread_add_timer(thread->master, smtp_connect_thread, checker,
 				 checker->vs->delay_loop);
 		return 0;
 	}
+#ifndef _HAVE_SOCK_CLOEXEC_
+	if (set_sock_flags(sd, F_SETFD, FD_CLOEXEC))
+		log_message(LOG_INFO, "Unable to set CLOEXEC on smtp socket - %s (%d)", strerror(errno), errno);
+#endif
 
 	status = tcp_bind_connect(sd, smtp_host);
 
@@ -786,6 +785,6 @@ smtp_connect_thread(thread_t *thread)
 		thread_add_timer(thread->master, smtp_connect_thread, checker,
 			checker->vs->delay_loop);
 	}
- 
+
 	return 0;
 }
