@@ -63,18 +63,8 @@ xdaemon(int nochdir, int noclose, int exitflag)
 	}
 
 	/* File descriptor close. */
-	if (!noclose) {
-		int fd;
-
-		fd = open("/dev/null", O_RDWR, 0);
-		if (fd != -1) {
-			dup2(fd, STDIN_FILENO);
-			dup2(fd, STDOUT_FILENO);
-			dup2(fd, STDERR_FILENO);
-			if (fd > 2)
-				close(fd);
-		}
-	}
+	if (!noclose)
+		set_std_fd(true);
 
 	umask(0);
 	return 0;
