@@ -20,6 +20,8 @@
  * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
  */
 
+#include <sys/resource.h>
+
 #include "main.h"
 #include "config.h"
 #include "signals.h"
@@ -27,19 +29,20 @@
 #include "bitops.h"
 #include "logger.h"
 
+#define CHILD_WAIT_SECS	5
+
 /* global var */
-char *conf_file = NULL;		/* Configuration file */
-int log_facility = LOG_DAEMON;	/* Optional logging facilities */
-pid_t vrrp_child = -1;		/* VRRP child process ID */
-pid_t checkers_child = -1;	/* Healthcheckers child process ID */
-int daemon_mode = 0;		/* VRRP/CHECK subsystem selection */
-int linkwatch = 0;		/* Use linkwatch kernel netlink reflection */
-char *main_pidfile = KEEPALIVED_PID_FILE;	/* overrule default pidfile */
-char *checkers_pidfile = CHECKERS_PID_FILE;	/* overrule default pidfile */
-char *vrrp_pidfile = VRRP_PID_FILE;	/* overrule default pidfile */
+char *conf_file = CONF;					/* Configuration file */
+int log_facility = LOG_DAEMON;				/* Optional logging facilities */
+pid_t vrrp_child = -1;					/* VRRP child process ID */
+pid_t checkers_child = -1;				/* Healthcheckers child process ID */
+const char *main_pidfile = KEEPALIVED_PID_FILE;		/* overrule default pidfile */
+const char *checkers_pidfile = CHECKERS_PID_FILE;	/* overrule default pidfile */
+const char *vrrp_pidfile = VRRP_PID_FILE;		/* overrule default pidfile */
+unsigned long daemon_mode = 0;				/* VRRP/CHECK subsystem selection */
 #ifdef _WITH_SNMP_
-int snmp = 0;			/* Enable SNMP support */
-const char *snmp_socket = NULL;	/* Socket to use for SNMP agent */
+int snmp = 0;						/* Enable SNMP support */
+const char *snmp_socket = NULL;				/* Socket to use for SNMP agent */
 #endif
 
 /* Log facility table */
@@ -50,24 +53,29 @@ static struct {
 	{LOG_LOCAL4}, {LOG_LOCAL5}, {LOG_LOCAL6}, {LOG_LOCAL7}
 };
 
+/* Control producing core dumps */
+static bool set_core_dump_pattern = false;
+static bool create_core_dump = false;
+static const char *core_dump_pattern = "core";
+static char *orig_core_dump_pattern = NULL;
+
 /* Daemon stop sequence */
 static void
 stop_keepalived(void)
 {
-	log_message(LOG_INFO, "Stopping " VERSION_STRING);
 	/* Just cleanup memory & exit */
 	signal_handler_destroy();
 	thread_destroy_master(master);
 
 	pidfile_rm(main_pidfile);
 
-	if (daemon_mode & 1 || !daemon_mode)
+	if (__test_bit(DAEMON_VRRP, &daemon_mode))
 		pidfile_rm(vrrp_pidfile);
 
-	if (daemon_mode & 2 || !daemon_mode)
+	if (__test_bit(DAEMON_CHECKERS, &daemon_mode))
 		pidfile_rm(checkers_pidfile);
 
-#ifdef _DEBUG_
+#ifdef _MEM_CHECK_
 	keepalived_free_final("Parent process");
 #endif
 }
@@ -78,57 +86,183 @@ start_keepalived(void)
 {
 #ifdef _WITH_LVS_
 	/* start healthchecker child */
-	if (daemon_mode & 2 || !daemon_mode)
+	if (__test_bit(DAEMON_CHECKERS, &daemon_mode))
 		start_check_child();
 #endif
 #ifdef _WITH_VRRP_
 	/* start vrrp child */
-	if (daemon_mode & 1 || !daemon_mode)
+	if (__test_bit(DAEMON_VRRP, &daemon_mode))
 		start_vrrp_child();
 #endif
 }
 
-/* SIGHUP handler */
-void
-sighup(void *v, int sig)
+/* SIGHUP/USR1/USR2 handler */
+static void
+propogate_signal(void *v, int sig)
 {
 	/* Signal child process */
 	if (vrrp_child > 0)
-		kill(vrrp_child, SIGHUP);
-	if (checkers_child > 0)
-		kill(checkers_child, SIGHUP);
+		kill(vrrp_child, sig);
+	if (checkers_child > 0 && sig == SIGHUP)
+		kill(checkers_child, sig);
 }
 
 /* Terminate handler */
-void
+static void
 sigend(void *v, int sig)
 {
 	int status;
+	int ret;
+	int wait_count = 0;
+	sigset_t old_set, child_wait;
+	struct timespec timeout = {
+		.tv_sec = CHILD_WAIT_SECS,
+		.tv_nsec = 0
+	};
+	struct timeval start_time, now;
 
 	/* register the terminate thread */
 	thread_add_terminate_event(master);
 
+	log_message(LOG_INFO, "Stopping");
+	sigprocmask(0, NULL, &old_set);
+	if (!sigismember(&old_set, SIGCHLD)) {
+		sigemptyset(&child_wait);
+		sigaddset(&child_wait, SIGCHLD);
+		sigprocmask(SIG_BLOCK, &child_wait, NULL);
+	}
+
 	if (vrrp_child > 0) {
 		kill(vrrp_child, SIGTERM);
-		waitpid(vrrp_child, &status, WNOHANG);
+		wait_count++;
 	}
 	if (checkers_child > 0) {
 		kill(checkers_child, SIGTERM);
-		waitpid(checkers_child, &status, WNOHANG);
+		wait_count++;
 	}
+
+	gettimeofday(&start_time, NULL);
+	while (wait_count) {
+		ret = sigtimedwait(&child_wait, NULL, &timeout);
+		if (ret == -1) {
+			if (errno == EINTR)
+				continue;
+			if (errno == EAGAIN)
+				break;
+		}
+
+		if (vrrp_child > 0 && vrrp_child == waitpid(vrrp_child, &status, WNOHANG)) {
+			report_child_status(status, vrrp_child, PROG_VRRP);
+			wait_count--;
+		}
+
+		if (checkers_child > 0 && checkers_child == waitpid(checkers_child, &status, WNOHANG)) {
+			report_child_status(status, checkers_child, PROG_CHECK);
+			wait_count--;
+		}
+		if (wait_count) {
+			gettimeofday(&now, NULL);
+			if (now.tv_usec < start_time.tv_usec) {
+				timeout.tv_nsec = (start_time.tv_usec - now.tv_usec) * 1000;
+				timeout.tv_sec = CHILD_WAIT_SECS - (now.tv_sec - start_time.tv_sec);
+			} else if (now.tv_usec == start_time.tv_usec) {
+				timeout.tv_nsec = 0;
+				timeout.tv_sec = CHILD_WAIT_SECS - (now.tv_sec - start_time.tv_sec);
+			} else {
+				timeout.tv_nsec = (1000000L + start_time.tv_usec - now.tv_usec) * 1000;
+				timeout.tv_sec = CHILD_WAIT_SECS - (now.tv_sec - start_time.tv_sec + 1);
+			}
+
+			timeout.tv_nsec = (start_time.tv_usec - now.tv_usec) * 1000;
+			timeout.tv_sec = CHILD_WAIT_SECS - (now.tv_sec - start_time.tv_sec);
+			if (timeout.tv_nsec < 0) {
+				timeout.tv_nsec += 1000000000L;
+				timeout.tv_sec--;
+			}
+		}
+	}
+
+	if (!sigismember(&old_set, SIGCHLD))
+		sigprocmask(SIG_UNBLOCK, &child_wait, NULL);
 }
 
 /* Initialize signal handler */
-void
+static void
 signal_init(void)
 {
 	signal_handler_init();
-	signal_set(SIGHUP, sighup, NULL);
+	signal_set(SIGHUP, propogate_signal, NULL);
+	signal_set(SIGUSR1, propogate_signal, NULL);
+	signal_set(SIGUSR2, propogate_signal, NULL);
 	signal_set(SIGINT, sigend, NULL);
 	signal_set(SIGTERM, sigend, NULL);
 	signal_ignore(SIGPIPE);
 }
 
+/* To create a core file when abrt is running (a RedHat distribution),
+ * and keepalived isn't installed from an RPM package, edit the file
+ * “/etc/abrt/abrt.conf”, and change the value of the field
+ * “ProcessUnpackaged” to “yes”. */
+static void
+update_core_dump_pattern(const char *pattern_str)
+{
+	int fd;
+	bool initialising = (orig_core_dump_pattern == NULL);
+
+	/* CORENAME_MAX_SIZE in kernel source defines the maximum string length,
+	 * see core_pattern[CORENAME_MAX_SIZE] in fs/coredump.c. Currently,
+	 * (Linux 4.6) defineds it to be 128, but the definition is not exposed
+	 * to user-space. */
+#define	CORENAME_MAX_SIZE	128
+
+	if (initialising)
+		orig_core_dump_pattern = MALLOC(CORENAME_MAX_SIZE);
+
+	fd = open ("/proc/sys/kernel/core_pattern", O_RDWR);
+
+	if (fd == -1 ||
+	    ( initialising && read(fd, orig_core_dump_pattern, CORENAME_MAX_SIZE - 1) == -1) ||
+	    write(fd, pattern_str, strlen(pattern_str)) == -1) {
+		log_message(LOG_INFO, "Unable to read/write core_pattern");
+
+		if (fd != -1)
+			close(fd);
+
+		FREE(orig_core_dump_pattern);
+		orig_core_dump_pattern = NULL;
+
+		return;
+	}
+
+	close(fd);
+
+	if (!initialising) {
+		FREE(orig_core_dump_pattern);
+		orig_core_dump_pattern = NULL;
+	}
+}
+
+static void
+core_dump_init(void)
+{
+	struct rlimit rlim;
+
+	if (set_core_dump_pattern) {
+		/* If we set the core_pattern here, we will attempt to restore it when we
+		 * exit. This will be fine if it is a child of ours that core dumps, 
+		 * but if we ourself core dump, then the core_pattern will not be restored */
+		update_core_dump_pattern(core_dump_pattern);
+	}
+
+	if (create_core_dump) {
+		rlim.rlim_cur = RLIM_INFINITY;
+		rlim.rlim_max = RLIM_INFINITY;
+
+		if (setrlimit(RLIMIT_CORE, &rlim) == -1)
+			log_message(LOG_INFO, "Failed to set core file size");
+	}
+}
+		
 /* Usage function */
 static void
 usage(const char *prog)
@@ -152,6 +286,11 @@ usage(const char *prog)
 #ifdef _WITH_SNMP_
 	fprintf(stderr, "  -x, --snmp                   Enable SNMP subsystem\n");
 	fprintf(stderr, "  -A, --snmp-agent-socket=FILE Use the specified socket for master agent\n");
+#endif
+	fprintf(stderr, "  -m, --core-dump              Produce core dump if terminate abnormally\n");
+	fprintf(stderr, "  -M, --core-dump-pattern=PATN Also set /proc/sys/kernel/core_pattern to PATN (default 'core')\n");
+#ifdef _MEM_CHECK_LOG_
+	fprintf(stderr, "  -L, --mem-check-log          Log malloc/frees to syslog\n");
 #endif
 	fprintf(stderr, "  -v, --version                Display the version number\n");
 	fprintf(stderr, "  -h, --help                   Display this help message\n");
@@ -183,19 +322,29 @@ parse_cmdline(int argc, char **argv)
 		{"snmp",              no_argument,       0, 'x'},
 		{"snmp-agent-socket", required_argument, 0, 'A'},
  #endif
+		{"core-dump",         no_argument,       0, 'm'},
+		{"core-dump-pattern", optional_argument, 0, 'M'},
+#ifdef _MEM_CHECK_LOG_
+		{"mem-check-log",     no_argument,       0, 'L'},
+#endif
 		{"version",           no_argument,       0, 'v'},
 		{"help",              no_argument,       0, 'h'},
 		{0, 0, 0, 0}
 	};
 
+	while ((c = getopt_long(argc, argv, "vhlndVIDRS:f:PCp:c:r:mML"
 #ifdef _WITH_SNMP_
-	while ((c = getopt_long(argc, argv, "vhlndVIDRS:f:PCp:c:r:xA:", long_options, NULL)) != EOF) {
-#else
-	while ((c = getopt_long(argc, argv, "vhlndVIDRS:f:PCp:c:r:", long_options, NULL)) != EOF) {
+					    "xA:"
 #endif
+									, long_options, NULL)) != EOF) {
 		switch (c) {
 		case 'v':
-			fprintf(stderr, VERSION_STRING);
+			fprintf(stderr, "%s", VERSION_STRING);
+#ifdef GIT_COMMIT
+			fprintf(stderr, ", git commit %s", GIT_COMMIT);
+#endif
+			fprintf(stderr, "\n\n%s\n\n", COPYRIGHT_STRING);
+			fprintf(stderr, "Build options: %s\n", BUILD_OPTIONS);
 			exit(0);
 			break;
 		case 'h':
@@ -233,10 +382,12 @@ parse_cmdline(int argc, char **argv)
 			conf_file = optarg;
 			break;
 		case 'P':
-			daemon_mode |= 1;
+			daemon_mode = 0;
+			__set_bit(DAEMON_VRRP, &daemon_mode);
 			break;
 		case 'C':
-			daemon_mode |= 2;
+			daemon_mode = 0;
+			__set_bit(DAEMON_CHECKERS, &daemon_mode);
 			break;
 		case 'p':
 			main_pidfile = optarg;
@@ -254,6 +405,18 @@ parse_cmdline(int argc, char **argv)
 		case 'A':
 			snmp_socket = optarg;
 			break;
+#endif
+		case 'M':
+			set_core_dump_pattern = true;
+			if (optarg && optarg[0])
+				core_dump_pattern = optarg;
+		case 'm':
+			create_core_dump = true;
+			break;
+#ifdef _MEM_CHECK_LOG_
+		case 'L':
+			__set_bit(MEM_CHECK_LOG_BIT, &debug);
+			break;
 #endif
 		default:
 			exit(0);
@@ -273,10 +436,15 @@ parse_cmdline(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
+	int report_stopped = true;
+
 	/* Init debugging level */
-	mem_allocated = 0;
 	debug = 0;
 
+	/* Initialise daemon_mode */
+	__set_bit(DAEMON_VRRP, &daemon_mode);
+	__set_bit(DAEMON_CHECKERS, &daemon_mode);
+
 	/*
 	 * Parse command line and set debug level.
 	 * bits 0..7 reserved by main.c
@@ -285,11 +453,23 @@ main(int argc, char **argv)
 
 	openlog(PROG, LOG_PID | ((__test_bit(LOG_CONSOLE_BIT, &debug)) ? LOG_CONS : 0)
 		    , log_facility);
-	log_message(LOG_INFO, "Starting " VERSION_STRING);
+#ifdef GIT_COMMIT
+	log_message(LOG_INFO, "Starting %s, git commit %s", VERSION_STRING, GIT_COMMIT);
+#else
+	log_message(LOG_INFO, "Starting %s", VERSION_STRING);
+#endif
+
+#ifdef _MEM_CHECK_
+	mem_log_init(PROG);
+#endif
+
+	/* Handle any core file requirements */
+	core_dump_init();
 
 	/* Check if keepalived is already running */
 	if (keepalived_running(daemon_mode)) {
 		log_message(LOG_INFO, "daemon is already running");
+		report_stopped = false;
 		goto end;
 	}
 
@@ -300,6 +480,16 @@ main(int argc, char **argv)
 	if (!__test_bit(DONT_FORK_BIT, &debug))
 		xdaemon(0, 0, 0);
 
+	/* Check we can read the configuration file(s).
+ 	   NOTE: the working directory will be / if we
+ 	   forked, but will be the current working directory
+ 	   when keepalived was run if we haven't forked.
+ 	   This means that if any config file names are not
+ 	   absolute file names, the behaviour will be different
+ 	   depending on whether we forked or not. */
+	if (!check_conf_file(conf_file))
+		goto end;
+
 	/* write the father's pidfile */
 	if (!pidfile_write(main_pidfile, getpid()))
 		goto end;
@@ -328,6 +518,18 @@ main(int argc, char **argv)
 	 * finally return from system
 	 */
 end:
+	if (report_stopped) {
+#ifdef GIT_COMMIT
+		log_message(LOG_INFO, "Stopped %s, git commit %s", VERSION_STRING, GIT_COMMIT);
+#else
+		log_message(LOG_INFO, "Stopped %s", VERSION_STRING);
+#endif
+	}
+
+	/* Restore original core_pattern if necessary */
+	if (orig_core_dump_pattern)
+		update_core_dump_pattern(orig_core_dump_pattern);
+
 	closelog();
 	exit(0);
 }
