@@ -25,13 +25,12 @@
 #include <fcntl.h>
 #include "logger.h"
 #include "pidfile.h"
-extern char *main_pidfile;
-extern char *checkers_pidfile;
-extern char *vrrp_pidfile;
+#include "main.h"
+#include "bitops.h"
 
 /* Create the runnnig daemon pidfile */
 int
-pidfile_write(char *pid_file, int pid)
+pidfile_write(const char *pid_file, int pid)
 {
 	FILE *pidfile = NULL;
 	int pidfd = creat(pid_file, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
@@ -49,14 +48,14 @@ pidfile_write(char *pid_file, int pid)
 
 /* Remove the running daemon pidfile */
 void
-pidfile_rm(char *pid_file)
+pidfile_rm(const char *pid_file)
 {
 	unlink(pid_file);
 }
 
 /* return the daemon running state */
-int
-process_running(char *pid_file)
+static int
+process_running(const char *pid_file)
 {
 	FILE *pidfile = fopen(pid_file, "r");
 	pid_t pid;
@@ -67,11 +66,16 @@ process_running(char *pid_file)
 		return 0;
 
 	ret = fscanf(pidfile, "%d", &pid);
-	if (ret == EOF && ferror(pidfile) != 0) {
-		log_message(LOG_INFO, "Error opening pid file %s", pid_file);
+	if (ret != 1) {
+		log_message(LOG_INFO, "Error reading pid file %s", pid_file);
+		pid = 0;
 	}
 	fclose(pidfile);
 
+	/* What should we return - we don't know if it is running or not. */
+	if (!pid)
+		return 1;
+
 	/* If no process is attached to pidfile, remove it */
 	if (kill(pid, 0)) {
 		log_message(LOG_INFO, "Remove a zombie pid file %s", pid_file);
@@ -84,16 +88,13 @@ process_running(char *pid_file)
 
 /* Return parent process daemon state */
 int
-keepalived_running(int mode)
+keepalived_running(unsigned long mode)
 {
 	if (process_running(main_pidfile))
 		return 1;
-	else if (mode & 1 || mode & 2)
-		return process_running((mode & 1) ? vrrp_pidfile :
-				       checkers_pidfile);
-
-	if (process_running(vrrp_pidfile) ||
-	    process_running(checkers_pidfile))
+	if (__test_bit(DAEMON_VRRP, &mode) && process_running(vrrp_pidfile))
+		return 1;
+	if (__test_bit(DAEMON_CHECKERS, &mode) && process_running(checkers_pidfile))
 		return 1;
 	return 0;
 }
