@@ -25,6 +25,8 @@
 #include "config.h"
 #include "global_data.h"
 
+#include <net-snmp/agent/agent_sysORTable.h>
+
 static int
 snmp_keepalived_log(int major, int minor, void *serverarg, void *clientarg)
 {
@@ -94,6 +96,7 @@ snmp_header_list_table(struct variable *vp, oid *name, size_t *length,
 #define SNMP_MAIL_EMAILADDRESS 7
 #define SNMP_TRAPS 8
 #define SNMP_LINKBEAT 9
+#define SNMP_LVSFLUSH 10
 
 static u_char*
 snmp_scalar(struct variable *vp, oid *name, size_t *length,
@@ -104,10 +107,10 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 
 	if (header_generic(vp, name, length, exact, var_len, write_method))
 		return NULL;
-	
+
 	switch (vp->magic) {
 	case SNMP_KEEPALIVEDVERSION:
-		*var_len = sizeof(version) - 2;
+		*var_len = sizeof(version) - 1;
 		return (u_char *)version;
 	case SNMP_ROUTERID:
 		if (!global_data->router_id) return NULL;
@@ -140,6 +143,9 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 	case SNMP_LINKBEAT:
 		long_ret = global_data->linkbeat_use_polling?2:1;
 		return (u_char *)&long_ret;
+	case SNMP_LVSFLUSH:
+		long_ret = global_data->lvs_flush?1:2;
+		return (u_char *)&long_ret;
 	default:
 		break;
 	}
@@ -162,8 +168,8 @@ snmp_mail(struct variable *vp, oid *name, size_t *length,
 		return (u_char *)m;
 	default:
 		break;
-        }
-        return NULL;
+	}
+	return NULL;
 }
 
 static const char global_name[] = "Keepalived";
@@ -184,6 +190,8 @@ static struct variable8 global_vars[] = {
 	{SNMP_TRAPS, ASN_INTEGER, RONLY, snmp_scalar, 1, {4}},
 	/* linkBeat */
 	{SNMP_LINKBEAT, ASN_INTEGER, RONLY, snmp_scalar, 1, {5}},
+	/* lvsFlush */
+	{SNMP_LVSFLUSH, ASN_INTEGER, RONLY, snmp_scalar, 1, {6}},
 };
 
 static int
@@ -213,11 +221,17 @@ void snmp_register_mib(oid *myoid, int len, const char *name,
 		log_message(LOG_WARNING, "Unable to register %s MIB", name);
 
 	snprintf(name_buf, sizeof(name_buf), "The MIB module for %s", name);
-	register_sysORTable(myoid, OID_LENGTH(myoid) - 1, name_buf);
+	register_sysORTable(myoid, len, name_buf);
 }
 
 void
-snmp_agent_init(const char *snmp_socket)
+snmp_unregister_mib(oid *myoid, int len)
+{
+	unregister_sysORTable(myoid, len);
+}
+
+void
+snmp_agent_init(const char *snmp_socket, bool base_mib)
 {
 	log_message(LOG_INFO, "Starting SNMP subagent");
 	netsnmp_enable_subagent();
@@ -254,17 +268,18 @@ snmp_agent_init(const char *snmp_socket)
 			   NETSNMP_DS_AGENT_AGENTX_PING_INTERVAL, 120);
 
 	init_agent(global_name);
-	snmp_register_mib(global_oid, OID_LENGTH(global_oid), global_name,
-			  (struct variable *)global_vars,
-			  sizeof(struct variable8),
-			  sizeof(global_vars)/sizeof(struct variable8));
+	if (base_mib)
+		snmp_register_mib(global_oid, OID_LENGTH(global_oid), global_name,
+				  (struct variable *)global_vars,
+				  sizeof(struct variable8),
+				  sizeof(global_vars)/sizeof(struct variable8));
 	init_snmp(global_name);
 }
 
 void
-snmp_agent_close(oid *myoid, int len, char *name)
+snmp_agent_close(bool base_mib)
 {
-	unregister_sysORTable(myoid, len);
-	unregister_sysORTable(global_oid, OID_LENGTH(global_oid));
+	if (base_mib)
+		snmp_unregister_mib(global_oid, OID_LENGTH(global_oid));
 	snmp_shutdown(global_name);
 }
