@@ -37,9 +37,6 @@
   #elif _KRNL_2_6_
     #include "../libipvs-2.6/ip_vs.h"
   #endif
-  #define SCHED_MAX_LENGTH IP_VS_SCHEDNAME_MAXLEN
-#else
-  #define SCHED_MAX_LENGTH   1
 #endif
 
 /* local includes */
@@ -52,7 +49,7 @@ typedef unsigned int checker_id_t;
 
 /* Daemon dynamic data structure definition */
 #define MAX_TIMEOUT_LENGTH		5
-#define KEEPALIVED_DEFAULT_DELAY	(60 * TIMER_HZ) 
+#define KEEPALIVED_DEFAULT_DELAY	(60 * TIMER_HZ)
 
 /* SSL specific data */
 typedef struct _ssl_data {
@@ -71,6 +68,8 @@ typedef struct _real_server {
 	struct sockaddr_storage		addr;
 	int				weight;
 	int				iweight;	/* Initial weight */
+	int				pweight;	/* previous weight
+							 * used for reloading */
 #ifdef _KRNL_2_6_
 	uint32_t			u_threshold;   /* Upper connection limit. */
 	uint32_t			l_threshold;   /* Lower connection limit. */
@@ -84,7 +83,7 @@ typedef struct _real_server {
 	list				failed_checkers;/* List of failed checkers */
 	int				set;		/* in the IPVS table */
 	int				reloaded;	/* active state was copied from old config while reloading */
-#if defined(_WITH_SNMP_) && defined(_KRNL_2_6_) && defined(_WITH_LVS_)
+#if defined(_WITH_SNMP_CHECKER_) && defined(_KRNL_2_6_) && defined(_WITH_LVS_)
 	/* Statistics */
 	uint32_t			activeconns;	/* active connections */
 	uint32_t			inactconns;	/* inactive connections */
@@ -116,15 +115,18 @@ typedef struct _virtual_server {
 	struct sockaddr_storage		addr;
 	real_server_t			*s_svr;
 	uint32_t			vfwmark;
+	uint16_t			af;
 	uint16_t			service_type;
 	long				delay_loop;
 	int				ha_suspend;
 	int				ops;
-	char				sched[SCHED_MAX_LENGTH];
+#ifdef _WITH_LVS_
+	char				sched[IP_VS_SCHEDNAME_MAXLEN];
 	char				timeout_persistence[MAX_TIMEOUT_LENGTH];
+	char				pe_name[IP_VS_PENAME_MAXLEN];
 	unsigned			loadbalancing_kind;
-	uint32_t			nat_mask;
 	uint32_t			granularity_persistence;
+#endif
 	char				*virtualhost;
 	list				rs;
 	int				alive;
@@ -137,7 +139,7 @@ typedef struct _virtual_server {
 	long unsigned			hysteresis;	/* up/down events "lag" WRT quorum. */
 	unsigned			quorum_state;	/* Reflects result of the last transition done. */
 	int				reloaded;	/* quorum_state was copied from old config while reloading */
-#if defined(_WITH_SNMP_) && defined(_KRNL_2_6_) && defined(_WITH_LVS_)
+#if defined(_WITH_SNMP_CHECKER_) && defined(_KRNL_2_6_) && defined(_WITH_LVS_)
 	/* Statistics */
 	time_t				lastupdated;
 	struct ip_vs_stats_user		stats;
@@ -217,9 +219,9 @@ static inline int inaddr_equal(sa_family_t family, void *addr1, void *addr2)
 
 #define VS_ISEQ(X,Y)	(sockstorage_equal(&(X)->addr,&(Y)->addr)			&&\
 			 (X)->vfwmark                 == (Y)->vfwmark			&&\
+			 (X)->af                      == (Y)->af                        &&\
 			 (X)->service_type            == (Y)->service_type		&&\
 			 (X)->loadbalancing_kind      == (Y)->loadbalancing_kind	&&\
-			 (X)->nat_mask                == (Y)->nat_mask			&&\
 			 (X)->granularity_persistence == (Y)->granularity_persistence	&&\
 			 (  (!(X)->quorum_up && !(Y)->quorum_up) || \
 			    ((X)->quorum_up && (Y)->quorum_up && !strcmp ((X)->quorum_up, (Y)->quorum_up)) \
@@ -227,15 +229,14 @@ static inline int inaddr_equal(sa_family_t family, void *addr1, void *addr2)
 			 !strcmp((X)->sched, (Y)->sched)				&&\
 			 !strcmp((X)->timeout_persistence, (Y)->timeout_persistence)	&&\
 			 (((X)->vsgname && (Y)->vsgname &&				\
-			   !strcmp((X)->vsgname, (Y)->vsgname)) || 			\
+			   !strcmp((X)->vsgname, (Y)->vsgname)) ||			\
 			  (!(X)->vsgname && !(Y)->vsgname)))
 
 #define VSGE_ISEQ(X,Y)	(sockstorage_equal(&(X)->addr,&(Y)->addr) &&	\
 			 (X)->range     == (Y)->range &&		\
 			 (X)->vfwmark   == (Y)->vfwmark)
 
-#define RS_ISEQ(X,Y)	(sockstorage_equal(&(X)->addr,&(Y)->addr) &&	\
-			 (X)->iweight   == (Y)->iweight)
+#define RS_ISEQ(X,Y)	(sockstorage_equal(&(X)->addr,&(Y)->addr))
 
 /* Global vars exported */
 extern check_data_t *check_data;
@@ -249,9 +250,6 @@ extern void alloc_vsg_entry(vector_t *);
 extern void alloc_vs(char *, char *);
 extern void alloc_rs(char *, char *);
 extern void alloc_ssvr(char *, char *);
-extern void alloc_group(char *);
-extern void alloc_rsgroup(char *, char *);
-extern void set_rsgroup(char *);
 extern check_data_t *alloc_check_data(void);
 extern void free_check_data(check_data_t *);
 extern void dump_check_data(check_data_t *);
