@@ -39,16 +39,24 @@
 #include "check_daemon.h"
 #include "global_data.h"
 
+/* State flags */
+enum daemon_bits {
+	DAEMON_VRRP,
+	DAEMON_CHECKERS
+};
+
 /* Global vars exported */
-extern char *conf_file;		/* Configuration file */
-extern int log_facility;	/* Optional logging facilities */
-extern pid_t vrrp_child;	/* VRRP child process ID */
-extern pid_t checkers_child;	/* Healthcheckers child process ID */
-extern int daemon_mode;		/* VRRP/CHECK subsystem selection */
-extern int linkwatch;		/* Use linkwatch kernel netlink reflection */
+extern unsigned long daemon_mode;	/* Which child processes are run */
+extern char *conf_file;			/* Configuration file */
+extern int log_facility;		/* Optional logging facilities */
+extern pid_t vrrp_child;		/* VRRP child process ID */
+extern pid_t checkers_child;		/* Healthcheckers child process ID */
+extern const char *main_pidfile;	/* overrule default pidfile */
+extern const char *checkers_pidfile;	/* overrule default pidfile */
+extern const char *vrrp_pidfile;	/* overrule default pidfile */
 #ifdef _WITH_SNMP_
-extern int snmp;		/* Enable SNMP support */
-extern const char *snmp_socket; /* Socket to use for SNMP agent */
+extern int snmp;			/* Enable SNMP support */
+extern const char *snmp_socket;		/* Socket to use for SNMP agent */
 #endif
 
 #endif
