@@ -11,7 +11,7 @@
  *              but WITHOUT ANY WARRANTY; without even the implied warranty of
  *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *              See the GNU General Public License for more details.
- *              
+ *
  *              This program is free software; you can redistribute it and/or
  *              modify it under the terms of the GNU General Public License
  *              as published by the Free Software Foundation; either version
@@ -57,8 +57,8 @@ typedef struct _ip_address {
 	char			*label;			/* Alias name, e.g. eth0:1 */
 	int			set;			/* TRUE if addr is set */
 	bool			iptable_rule_set;	/* TRUE if iptable drop rule
-							 * set to addr
-							 */
+							 * set to addr */
+	bool			garp_gna_pending;	/* Is a gratuitous ARP/NA message still to be sent */
 } ip_address_t;
 
 #define IPADDRESS_DEL 0
@@ -88,15 +88,18 @@ typedef struct _ip_address {
 
 #define IP_ISEQ(X,Y)    (((X) && (Y)) ? ((IP_FAMILY(X) == IP_FAMILY(Y)) ? (IP_IS6(X) ? IP6_ISEQ(X, Y) : IP4_ISEQ(X, Y)) : 0) : (((!(X) && (Y))||((X) && !(Y))) ? 0 : 1))
 
+struct ipt_handle;	// AAGH - TODO
+
 /* prototypes */
+extern char *ipaddresstos(char *, ip_address_t *);
+extern int netlink_ipaddress(ip_address_t *, int);
 extern void netlink_iplist(list, int);
-extern void handle_iptable_rule_to_iplist(list, int, char *);
+extern void handle_iptable_rule_to_iplist(struct ipt_handle *, list, int, char *, bool force);
 extern void free_ipaddress(void *);
-extern char *ipaddresstos(ip_address_t *);
 extern void dump_ipaddress(void *);
-extern ip_address_t *parse_ipaddress(ip_address_t *, char *);
+extern ip_address_t *parse_ipaddress(ip_address_t *, char *, int);
 extern void alloc_ipaddress(list, vector_t *, interface_t *);
-extern void clear_diff_address(list, list);
+extern void clear_diff_address(struct ipt_handle *, list, list);
 extern void clear_diff_saddresses(void);
 
 #endif
