@@ -28,11 +28,16 @@
 #include "logger.h"
 #include "memory.h"
 #include "utils.h"
+#include "bitops.h"
+#include "vrrp_scheduler.h"
 #include "vrrp_arp.h"
+#ifndef _HAVE_SOCK_CLOEXEC_
+#include "old_socket.h"
+#endif
 
-/* global vars */
-char *garp_buffer;
-int garp_fd;
+/* static vars */
+static char *garp_buffer;
+static int garp_fd;
 
 /* Send the gratuitous ARP message */
 static int send_arp(ip_address_t *ipaddress)
@@ -47,6 +52,10 @@ static int send_arp(ip_address_t *ipaddress)
 	sll.sll_halen = ETHERNET_HW_LEN;
 	sll.sll_ifindex = IF_INDEX(ipaddress->ifp);
 
+	if (__test_bit(LOG_DETAIL_BIT, &debug))
+		log_message(LOG_INFO, "Sending gratuitous ARP on %s for %s",
+			    IF_NAME(ipaddress->ifp), inet_ntop2(ipaddress->u.sin.sin_addr.s_addr));
+
 	/* Send packet */
 	len = sendto(garp_fd, garp_buffer, sizeof(arphdr_t) + ETHER_HDR_LEN
 		     , 0, (struct sockaddr *)&sll, sizeof(sll));
@@ -57,7 +66,7 @@ static int send_arp(ip_address_t *ipaddress)
 }
 
 /* Build a gratuitous ARP message over a specific interface */
-int send_gratuitous_arp(ip_address_t *ipaddress)
+int send_gratuitous_arp_immediate(interface_t *ifp, ip_address_t *ipaddress)
 {
 	struct ether_header *eth = (struct ether_header *) garp_buffer;
 	arphdr_t *arph		 = (arphdr_t *) (garp_buffer + ETHER_HDR_LEN);
@@ -83,11 +92,51 @@ int send_gratuitous_arp(ip_address_t *ipaddress)
 	/* Send the ARP message */
 	len = send_arp(ipaddress);
 
+	/* If we have to delay between sending garps, note the next time we can */
+	if (ifp->garp_delay && ifp->garp_delay->have_garp_interval)
+		ifp->garp_delay->garp_next_time = timer_add_now(ifp->garp_delay->garp_interval);
+
 	/* Cleanup room for next round */
 	memset(garp_buffer, 0, sizeof(arphdr_t) + ETHER_HDR_LEN);
 	return len;
 }
 
+static void queue_garp(vrrp_t *vrrp, interface_t *ifp, ip_address_t *ipaddress)
+{
+	timeval_t next_time = timer_add_now(ifp->garp_delay->garp_interval);
+
+	vrrp->garp_pending = true;
+	ipaddress->garp_gna_pending = true;
+
+	/* Do we need to reschedule the garp thread? */
+	if (!garp_thread || timer_cmp(next_time, garp_next_time) < 0) {
+		if (garp_thread)
+			thread_cancel(garp_thread);
+
+		garp_next_time = next_time;
+
+		garp_thread = thread_add_timer(master, vrrp_arp_thread, NULL, -timer_long(timer_sub_now(garp_next_time)));
+	}
+}
+
+void send_gratuitous_arp(vrrp_t *vrrp, ip_address_t *ipaddress)
+{
+	interface_t *ifp = IF_BASE_IFP(ipaddress->ifp);
+
+	set_time_now();
+
+	/* Do we need to delay sending the garp? */
+	if (ifp->garp_delay &&
+	    ifp->garp_delay->have_garp_interval &&
+	    ifp->garp_delay->garp_next_time.tv_sec) {
+		if (timer_cmp(time_now, ifp->garp_delay->garp_next_time) < 0) {
+			queue_garp(vrrp, ifp, ipaddress);
+			return;
+		}
+	}
+
+	send_gratuitous_arp_immediate(ifp, ipaddress);
+}
 
 /*
  *	Gratuitous ARP init/close
@@ -98,12 +147,19 @@ void gratuitous_arp_init(void)
 	garp_buffer = (char *)MALLOC(sizeof(arphdr_t) + ETHER_HDR_LEN);
 
 	/* Create the socket descriptor */
-	garp_fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_RARP));
+	garp_fd = socket(PF_PACKET, SOCK_RAW | SOCK_CLOEXEC, htons(ETH_P_RARP));
 
 	if (garp_fd > 0)
 		log_message(LOG_INFO, "Registering gratuitous ARP shared channel");
-	else
+	else {
 		log_message(LOG_INFO, "Error while registering gratuitous ARP shared channel");
+		return;
+	}
+
+#ifndef _HAVE_SOCK_CLOEXEC_
+	if (set_sock_flags(garp_fd, F_SETFD, FD_CLOEXEC))
+		log_message(LOG_INFO, "Unable to set CLOEXEC on gratuitous ARP socket");
+#endif
 }
 void gratuitous_arp_close(void)
 {
