@@ -27,7 +27,11 @@
 #include "vrrp_ndisc.h"
 #include "vrrp_netlink.h"
 #include "vrrp_ipaddress.h"
+#include "vrrp_iptables.h"
+#ifdef _HAVE_FIB_ROUTING_
+#include "vrrp_iprule.h"
 #include "vrrp_iproute.h"
+#endif
 #include "vrrp_parser.h"
 #include "vrrp_data.h"
 #include "vrrp.h"
@@ -37,6 +41,7 @@
 #include "daemon.h"
 #include "logger.h"
 #include "signals.h"
+#include "process.h"
 #include "bitops.h"
 #ifdef _WITH_LVS_
   #include "ipvswrapper.h"
@@ -44,34 +49,61 @@
 #ifdef _WITH_SNMP_
   #include "vrrp_snmp.h"
 #endif
+#ifdef _HAVE_LIBIPSET_
+  #include "vrrp_ipset.h"
+#endif
 #include "list.h"
 #include "main.h"
 #include "memory.h"
 #include "parser.h"
 
-extern char *vrrp_pidfile;
+/* Forward declarations */
+static int print_vrrp_data(thread_t * thread);
+static int print_vrrp_stats(thread_t * thread);
+static int reload_vrrp_thread(thread_t * thread);
+
+
 
 /* Daemon stop sequence */
 static void
 stop_vrrp(void)
 {
-	signal_handler_destroy();
+	/* Ensure any interfaces are in backup mode,
+	 * sending a priority 0 vrrp message
+	 */
+	restore_vrrp_interfaces();
 
-	if (!__test_bit(DONT_RELEASE_VRRP_BIT, &debug))
-		shutdown_vrrp_instances();
+#ifdef _HAVE_LIBIPTC_
+	iptables_fini();
+#endif
 
 	/* Clear static entries */
+#ifdef _HAVE_FIB_ROUTING_
+	netlink_rulelist(vrrp_data->static_rules, IPRULE_DEL, false);
 	netlink_rtlist(vrrp_data->static_routes, IPROUTE_DEL);
+#endif
 	netlink_iplist(vrrp_data->static_addresses, IPADDRESS_DEL);
 
 #ifdef _WITH_SNMP_
-	if (snmp)
+	if (global_data->enable_snmp_keepalived || global_data->enable_snmp_rfcv2 || global_data->enable_snmp_rfcv3)
 		vrrp_snmp_agent_close();
 #endif
 
 	/* Stop daemon */
 	pidfile_rm(vrrp_pidfile);
 
+	/* Clean data */
+	vrrp_dispatcher_release(vrrp_data);
+
+	/* This is not nice, but it significantly increases the chances
+	 * of an IGMP leave group being sent for some reason.
+	 * Since we are about to exit, it doesn't affect anything else
+	 * running. */
+	sleep ( 1 );
+
+	if (!__test_bit(DONT_RELEASE_VRRP_BIT, &debug))
+		shutdown_vrrp_instances();
+
 #ifdef _WITH_LVS_
 	if (vrrp_ipvs_needed()) {
 		/* Clean ipvs related */
@@ -79,18 +111,20 @@ stop_vrrp(void)
 	}
 #endif
 
-	/* Clean data */
-	free_global_data(global_data);
-	vrrp_dispatcher_release(vrrp_data);
-	free_vrrp_data(vrrp_data);
-	free_vrrp_buffer();
-	free_interface_queue();
+	/* We mustn't receive a SIGCHLD after master is destroyed */
+	signal_handler_destroy();
+
 	kernel_netlink_close();
 	thread_destroy_master(master);
 	gratuitous_arp_close();
 	ndisc_close();
 
-#ifdef _DEBUG_
+	free_global_data(global_data);
+	free_vrrp_data(vrrp_data);
+	free_vrrp_buffer();
+	free_interface_queue();
+
+#ifdef _MEM_CHECK_
 	keepalived_free_final("VRRP Child process");
 #endif
 
@@ -98,7 +132,10 @@ stop_vrrp(void)
 	 * Reached when terminate signal catched.
 	 * finally return to parent process.
 	 */
+	log_message(LOG_INFO, "Stopped");
+
 	closelog();
+
 	exit(0);
 }
 
@@ -111,15 +148,15 @@ start_vrrp(void)
 	kernel_netlink_init();
 	gratuitous_arp_init();
 	ndisc_init();
-#ifdef _WITH_SNMP_
-	if (!reload && snmp)
-		vrrp_snmp_agent_init(snmp_socket);
+
+	global_data = alloc_global_data();
+
+#ifdef _HAVE_LIBIPTC_
+	iptables_init();
 #endif
 
 	/* Parse configuration file */
-	global_data = alloc_global_data();
 	vrrp_data = alloc_vrrp_data();
-	alloc_vrrp_buffer();
 	init_data(conf_file, vrrp_init_keywords);
 	if (!vrrp_data) {
 		stop_vrrp();
@@ -127,6 +164,22 @@ start_vrrp(void)
 	}
 	init_global_data(global_data);
 
+	/* Set the process priority and non swappable if configured */
+	if (global_data->vrrp_process_priority)
+		set_process_priority(global_data->vrrp_process_priority);
+
+	if (global_data->vrrp_no_swap)
+		set_process_dont_swap(4096);	/* guess a stack size to reserve */
+
+#ifdef _WITH_SNMP_
+	if (!reload && (global_data->enable_snmp_keepalived || global_data->enable_snmp_rfcv2 || global_data->enable_snmp_rfcv3)) {
+		vrrp_snmp_agent_init(global_data->snmp_socket);
+#ifdef _WITH_SNMP_RFC_
+		vrrp_start_time = timer_now();
+#endif
+	}
+#endif
+
 #ifdef _WITH_LVS_
 	if (vrrp_ipvs_needed()) {
 		/* Initialize ipvs related */
@@ -134,15 +187,36 @@ start_vrrp(void)
 			stop_vrrp();
 			return;
 		}
+
+#ifdef _HAVE_IPVS_SYNCD_
+		/* If we are managing the sync daemon, then stop any
+		 * instances of it that may have been running if
+		 * we terminated abnormally */
+		ipvs_syncd_cmd(IPVS_STOPDAEMON, NULL, IPVS_MASTER, 0, true);
+		ipvs_syncd_cmd(IPVS_STOPDAEMON, NULL, IPVS_BACKUP, 0, true);
+#endif
 	}
 #endif
 
 	if (reload) {
 		clear_diff_saddresses();
+#ifdef _HAVE_FIB_ROUTING_
+		clear_diff_srules();
 		clear_diff_sroutes();
+#endif
 		clear_diff_vrrp();
 		clear_diff_script();
 	}
+	else {
+		/* Clear leftover static entries */
+		netlink_iplist(vrrp_data->static_addresses, IPADDRESS_DEL);
+#ifdef _HAVE_FIB_ROUTING_
+		netlink_rtlist(vrrp_data->static_routes, IPROUTE_DEL);
+		netlink_error_ignore = ENOENT;
+		netlink_rulelist(vrrp_data->static_rules, IPRULE_DEL, true);
+		netlink_error_ignore = 0;
+#endif
+	}
 
 	/* Complete VRRP initialization */
 	if (!vrrp_complete_init()) {
@@ -152,17 +226,31 @@ start_vrrp(void)
 		return;
 	}
 
+#ifdef _HAVE_LIBIPTC_
+	iptables_startup();
+#endif
+
 	/* Post initializations */
+#ifdef _DEBUG_
 	log_message(LOG_INFO, "Configuration is using : %lu Bytes", mem_allocated);
+#endif
 
 	/* Set static entries */
 	netlink_iplist(vrrp_data->static_addresses, IPADDRESS_ADD);
+#ifdef _HAVE_FIB_ROUTING_
 	netlink_rtlist(vrrp_data->static_routes, IPROUTE_ADD);
+	netlink_rulelist(vrrp_data->static_rules, IPRULE_ADD, false);
+#endif
 
 	/* Dump configuration */
 	if (__test_bit(DUMP_CONF_BIT, &debug)) {
+		list ifl;
+
 		dump_global_data(global_data);
 		dump_vrrp_data(vrrp_data);
+		ifl = get_if_list();
+		if (!LIST_ISEMPTY(ifl))
+			dump_list(ifl);
 	}
 
 	/* Initialize linkbeat */
@@ -173,15 +261,13 @@ start_vrrp(void)
 			 VRRP_DISPATCHER);
 }
 
-/* Reload handler */
-int reload_vrrp_thread(thread_t * thread);
-void
+static void
 sighup_vrrp(void *v, int sig)
 {
 	thread_add_event(master, reload_vrrp_thread, NULL, 0);
 }
-int print_vrrp_data(thread_t * thread);
-void
+
+static void
 sigusr1_vrrp(void *v, int sig)
 {
 	log_message(LOG_INFO, "Printing VRRP data for process(%d) on signal",
@@ -189,8 +275,7 @@ sigusr1_vrrp(void *v, int sig)
 	thread_add_event(master, print_vrrp_data, NULL, 0);
 }
 
-int print_vrrp_stats(thread_t * thread);
-void
+static void
 sigusr2_vrrp(void *v, int sig)
 {
 	log_message(LOG_INFO, "Printing VRRP stats for process(%d) on signal",
@@ -199,7 +284,7 @@ sigusr2_vrrp(void *v, int sig)
 }
 
 /* Terminate handler */
-void
+static void
 sigend_vrrp(void *v, int sig)
 {
 	if (master)
@@ -207,7 +292,7 @@ sigend_vrrp(void *v, int sig)
 }
 
 /* VRRP Child signal handling */
-void
+static void
 vrrp_signal_init(void)
 {
 	signal_handler_init();
@@ -220,23 +305,25 @@ vrrp_signal_init(void)
 }
 
 /* Reload thread */
-int
+static int
 reload_vrrp_thread(thread_t * thread)
 {
 	/* set the reloading flag */
 	SET_RELOAD;
 
-	/* Signal handling */
-	signal_reset();
-	signal_handler_destroy();
-
 	/* Destroy master thread */
 	vrrp_dispatcher_release(vrrp_data);
 	kernel_netlink_close();
-	thread_destroy_master(master);
-	master = thread_make_master();
+	thread_cleanup_master(master);
+#ifdef _HAVE_IPVS_SYNCD_
+	/* TODO - Note: this didn't work if we found ipvs_syndc on vrrp before on old_vrrp */
+	if (global_data->lvs_syncd_if)
+		ipvs_syncd_cmd(IPVS_STOPDAEMON, NULL,
+		       (global_data->lvs_syncd_vrrp->state == VRRP_STATE_MAST) ? IPVS_MASTER:
+										 IPVS_BACKUP,
+		       global_data->lvs_syncd_syncid, false);
+#endif
 	free_global_data(global_data);
-	free_interface_queue();
 	free_vrrp_buffer();
 	gratuitous_arp_close();
 	ndisc_close();
@@ -251,28 +338,38 @@ reload_vrrp_thread(thread_t * thread)
 	/* Save previous conf data */
 	old_vrrp_data = vrrp_data;
 	vrrp_data = NULL;
+	reset_interface_queue();
 
 	/* Reload the conf */
+#ifdef _DEBUG_
 	mem_allocated = 0;
-	vrrp_signal_init();
-	signal_set(SIGCHLD, thread_child_handler, master);
+#endif
 	start_vrrp();
 
+#ifdef _HAVE_IPVS_SYNCD_
+	if (global_data->lvs_syncd_if)
+		ipvs_syncd_cmd(IPVS_STARTDAEMON, NULL,
+			       (global_data->lvs_syncd_vrrp->state == VRRP_STATE_MAST) ? IPVS_MASTER:
+											 IPVS_BACKUP,
+			       global_data->lvs_syncd_syncid, false);
+#endif
+
 	/* free backup data */
 	free_vrrp_data(old_vrrp_data);
+	free_old_interface_queue();
 	UNSET_RELOAD;
 
 	return 0;
 }
 
-int
+static int
 print_vrrp_data(thread_t * thread)
 {
 	vrrp_print_data();
 	return 0;
 }
 
-int
+static int
 print_vrrp_stats(thread_t * thread)
 {
 	vrrp_print_stats();
@@ -281,7 +378,7 @@ print_vrrp_stats(thread_t * thread)
 
 
 /* VRRP Child respawning thread */
-int
+static int
 vrrp_respawn_thread(thread_t * thread)
 {
 	pid_t pid;
@@ -333,10 +430,16 @@ start_vrrp_child(void)
 		return 0;
 	}
 
+	signal_handler_destroy();
+
 	/* Opening local VRRP syslog channel */
 	openlog(PROG_VRRP, LOG_PID | ((__test_bit(LOG_CONSOLE_BIT, &debug)) ? LOG_CONS : 0)
 			 , (log_facility==LOG_DAEMON) ? LOG_LOCAL1 : log_facility);
 
+#ifdef _MEM_CHECK_
+	mem_log_init(PROG_VRRP);
+#endif
+
 	/* Child process part, write pidfile */
 	if (!pidfile_write(vrrp_pidfile, getpid())) {
 		/* Fatal error */
@@ -345,8 +448,7 @@ start_vrrp_child(void)
 	}
 
 	/* Create the new master thread */
-	signal_handler_destroy();
-	thread_destroy_master(master);
+	thread_destroy_master(master);	/* This destroys any residual settings from the parent */
 	master = thread_make_master();
 
 	/* change to / dir */
