@@ -44,6 +44,9 @@ typedef uint8_t u8;
 #ifdef _KRNL_2_4_
 #include <linux/ethtool.h>
 #endif
+#ifndef _HAVE_SOCK_CLOEXEC_
+#include "old_socket.h"
+#endif
 
 /* local include */
 #include "scheduler.h"
@@ -56,10 +59,16 @@ typedef uint8_t u8;
 #include "utils.h"
 #include "logger.h"
 
-/* Global vars */
+/* Local vars */
 static list if_queue;
 static struct ifreq ifr;
 
+static list old_if_queue;
+static list old_garp_delay;
+
+/* Global vars */
+list garp_delay;
+
 /* Helper functions */
 /* Return interface from interface index */
 interface_t *
@@ -85,7 +94,22 @@ base_if_get_by_ifindex(const int ifindex)
 {
 	interface_t *ifp = if_get_by_ifindex(ifindex);
 
+#ifdef _HAVE_VRRP_VMAC_
 	return (ifp && ifp->vmac) ? if_get_by_ifindex(ifp->base_ifindex) : ifp;
+#else
+	return ifp;
+#endif
+}
+
+/* Return base interface from interface index incase of VMAC */
+interface_t *
+base_if_get_by_ifp(interface_t *ifp)
+{
+#ifdef _HAVE_VRRP_VMAC_
+	return (ifp && ifp->vmac) ? if_get_by_ifindex(ifp->base_ifindex) : ifp;
+#else
+	return ifp;
+#endif
 }
 
 interface_t *
@@ -105,6 +129,24 @@ if_get_by_ifname(const char *ifname)
 	return NULL;
 }
 
+/* Return the interface list itself */
+list
+get_if_list(void)
+{
+	return if_queue;
+}
+
+void
+reset_interface_queue(void)
+{
+	old_if_queue = if_queue;
+	old_garp_delay = garp_delay;
+
+	if_queue = NULL;
+	garp_delay = NULL;
+}
+
+#ifdef _HAVE_VRRP_VMAC_
 /*
  * Reflect base interface flags on VMAC interfaces.
  * VMAC interfaces should never update it own flags, only be reflected
@@ -125,6 +167,7 @@ if_vmac_reflect_flags(const int ifindex, const unsigned long flags)
 			ifp->flags = flags;
 	}
 }
+#endif
 
 /* MII Transceiver Registers poller functions */
 static int
@@ -196,16 +239,22 @@ if_mii_status(const int fd)
 		return LINK_DOWN;
 }
 
-int
+static int
 if_mii_probe(const char *ifname)
 {
 	uint16_t *data = (uint16_t *) (&ifr.ifr_data);
 	int phy_id;
-	int fd = socket(AF_INET, SOCK_DGRAM, 0);
+	int fd = socket(AF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	int status = 0;
 
 	if (fd < 0)
 		return -1;
+
+#ifndef _HAVE_SOCK_CLOEXEC_
+	if (set_sock_flags(fd, F_SETFD, FD_CLOEXEC))
+		log_message(LOG_INFO, "Unable to set CLOEXEC on mii_probe socket - %s (%d)", strerror(errno), errno);
+#endif
+
 	memset(&ifr, 0, sizeof (struct ifreq));
 	strncpy(ifr.ifr_name, ifname, sizeof (ifr.ifr_name));
 	if (ioctl(fd, SIOCGMIIPHY, &ifr) < 0) {
@@ -247,14 +296,20 @@ if_ethtool_status(const int fd)
 		return -1;
 }
 
-int
+static int
 if_ethtool_probe(const char *ifname)
 {
-	int fd = socket(AF_INET, SOCK_DGRAM, 0);
+	int fd = socket(AF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	int status = 0;
 
 	if (fd < 0)
 		return -1;
+
+#ifndef _HAVE_SOCK_CLOEXEC_
+	if (set_sock_flags(fd, F_SETFD, FD_CLOEXEC))
+		log_message(LOG_INFO, "Unable to set CLOEXEC on ethtool_probe socket - %s (%d)", strerror(errno), errno);
+#endif
+
 	memset(&ifr, 0, sizeof (struct ifreq));
 	strncpy(ifr.ifr_name, ifname, sizeof (ifr.ifr_name));
 
@@ -263,13 +318,19 @@ if_ethtool_probe(const char *ifname)
 	return status;
 }
 
-void
+static void
 if_ioctl_flags(interface_t * ifp)
 {
-	int fd = socket(AF_INET, SOCK_DGRAM, 0);
+	int fd = socket(AF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 
 	if (fd < 0)
 		return;
+
+#ifndef _HAVE_SOCK_CLOEXEC_
+	if (set_sock_flags(fd, F_SETFD, FD_CLOEXEC))
+		log_message(LOG_INFO, "Unable to set CLOEXEC on ioctl_flags socket - %s (%d)", strerror(errno), errno);
+#endif
+
 	memset(&ifr, 0, sizeof (struct ifreq));
 	strncpy(ifr.ifr_name, ifp->ifname, sizeof (ifr.ifr_name));
 	if (ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) {
@@ -287,20 +348,70 @@ free_if(void *data)
 	FREE(data);
 }
 
+/* garp_delay facility function */
 void
+alloc_garp_delay(void)
+{
+	if (!LIST_EXISTS(garp_delay))
+		garp_delay = alloc_list(NULL, NULL);
+
+	list_add(garp_delay, MALLOC(sizeof(garp_delay_t)));
+}
+	
+void
+set_default_garp_delay(void)
+{
+	garp_delay_t default_delay;
+	element e;
+	interface_t *ifp;
+	garp_delay_t *delay;
+
+	if (global_data->vrrp_garp_interval) {
+		default_delay.garp_interval.tv_sec = global_data->vrrp_garp_interval / 1000000;
+		default_delay.garp_interval.tv_usec = global_data->vrrp_garp_interval % 1000000;
+		default_delay.have_garp_interval = true;
+	}
+	if (global_data->vrrp_gna_interval) {
+		default_delay.gna_interval.tv_sec = global_data->vrrp_gna_interval / 1000000;
+		default_delay.gna_interval.tv_usec = global_data->vrrp_gna_interval % 1000000;
+		default_delay.have_gna_interval = true;
+	}
+
+	/* Allocate a delay structure to each physical inteface that doesn't have one */
+	for (e = LIST_HEAD(if_queue); e; ELEMENT_NEXT(e)) {
+		ifp = ELEMENT_DATA(e);
+		if (!ifp->garp_delay
+#ifdef _HAVE_VRRP_VMAC_
+				     && !ifp->vmac)
+#else
+)
+#endif
+		{
+			alloc_garp_delay();
+			delay = LIST_TAIL_DATA(garp_delay);
+			*delay = default_delay;
+			ifp->garp_delay = delay;
+		}
+	}
+}
+
+static void
 dump_if(void *data)
 {
 	interface_t *ifp = data;
-	char addr_str[41];
+#ifdef _HAVE_VRRP_VMAC_
+	interface_t *ifp_u;
+#endif
+	char addr_str[INET6_ADDRSTRLEN];
 
 	log_message(LOG_INFO, "------< NIC >------");
 	log_message(LOG_INFO, " Name = %s", ifp->ifname);
 	log_message(LOG_INFO, " index = %d", ifp->ifindex);
 	log_message(LOG_INFO, " IPv4 address = %s", inet_ntop2(ifp->sin_addr.s_addr));
-	inet_ntop(AF_INET6, &ifp->sin6_addr, addr_str, 41);
+	inet_ntop(AF_INET6, &ifp->sin6_addr, addr_str, sizeof(addr_str));
 	log_message(LOG_INFO, " IPv6 address = %s", addr_str);
 
-	/* FIXME: Harcoded for ethernet */
+	/* FIXME: Hardcoded for ethernet */
 	if (ifp->hw_type == ARPHRD_ETHER)
 		log_message(LOG_INFO, " MAC = %.2x:%.2x:%.2x:%.2x:%.2x:%.2x",
 		       ifp->hw_addr[0], ifp->hw_addr[1], ifp->hw_addr[2]
@@ -329,6 +440,11 @@ dump_if(void *data)
 		break;
 	}
 
+#ifdef _HAVE_VRRP_VMAC_
+	if (ifp->vmac && (ifp_u = if_get_by_ifindex(ifp->base_ifindex)))
+		log_message(LOG_INFO, " VMAC underlying interface = %s", ifp_u->ifname);
+#endif
+
 	/* MII channel supported ? */
 	if (IF_MII_SUPPORTED(ifp))
 		log_message(LOG_INFO, " NIC support MII regs");
@@ -336,6 +452,20 @@ dump_if(void *data)
 		log_message(LOG_INFO, " NIC support EHTTOOL GLINK interface");
 	else
 		log_message(LOG_INFO, " Enabling NIC ioctl refresh polling");
+
+	if (ifp->garp_delay) {
+		if (ifp->garp_delay->have_garp_interval)
+			log_message(LOG_INFO, " Gratuitous ARP interval %ldms",
+				    ifp->garp_delay->garp_interval.tv_sec * 100 +
+				     ifp->garp_delay->garp_interval.tv_usec / (TIMER_HZ / 100));
+
+		if (ifp->garp_delay->have_gna_interval)
+			log_message(LOG_INFO, " Gratuitous NA interval %ldms",
+				    ifp->garp_delay->gna_interval.tv_sec * 100 +
+				     ifp->garp_delay->gna_interval.tv_usec / (TIMER_HZ / 100));
+		if (ifp->garp_delay->aggregation_group)
+			log_message(LOG_INFO, " Gratuitous ARP aggregation group %d", ifp->garp_delay->aggregation_group);
+	}
 }
 
 static void
@@ -416,9 +546,15 @@ if_linkbeat(const interface_t * ifp)
 void
 free_interface_queue(void)
 {
-	if (!LIST_ISEMPTY(if_queue))
-		free_list(if_queue);
-	if_queue = NULL;
+	free_list(&if_queue);
+	free_list(&garp_delay);
+}
+
+void
+free_old_interface_queue(void)
+{
+	free_list(&old_if_queue);
+	free_list(&old_garp_delay);
 }
 
 void
@@ -459,7 +595,6 @@ if_join_vrrp_group(sa_family_t family, int *sd, interface_t *ifp, int proto)
 	if (family == AF_INET) {
 		memset(&imr, 0, sizeof(imr));
 		imr.imr_multiaddr = ((struct sockaddr_in *) &global_data->vrrp_mcast_group4)->sin_addr;
-		imr.imr_address.s_addr = IF_ADDR(ifp);
 		imr.imr_ifindex = IF_INDEX(ifp);
 
 		/* -> Need to handle multicast convergance after takeover.
@@ -476,11 +611,11 @@ if_join_vrrp_group(sa_family_t family, int *sd, interface_t *ifp, int proto)
 	}
 
 	if (ret < 0) {
-		log_message(LOG_INFO, "cant do IP%s_ADD_MEMBERSHIP errno=%s (%d)",
-			    (family == AF_INET) ? "" : "V6", strerror(errno), errno);
+		log_message(LOG_INFO, "(%s): cant do IP%s_ADD_MEMBERSHIP errno=%s (%d)",
+			    ifp->ifname, (family == AF_INET) ? "" : "V6", strerror(errno), errno);
 		close(*sd);
 		*sd = -1;
-        }
+	}
 
 	return *sd;
 }
@@ -488,7 +623,7 @@ if_join_vrrp_group(sa_family_t family, int *sd, interface_t *ifp, int proto)
 int
 if_leave_vrrp_group(sa_family_t family, int sd, interface_t *ifp)
 {
-	struct ip_mreq imr;
+	struct ip_mreqn imr;
 	struct ipv6_mreq imr6;
 	int ret = 0;
 
@@ -500,9 +635,9 @@ if_leave_vrrp_group(sa_family_t family, int sd, interface_t *ifp)
 	if (family == AF_INET) {
 		memset(&imr, 0, sizeof(imr));
 		imr.imr_multiaddr = ((struct sockaddr_in *) &global_data->vrrp_mcast_group4)->sin_addr;
-		imr.imr_interface.s_addr = IF_ADDR(ifp);
+		imr.imr_ifindex = IF_INDEX(ifp);
 		ret = setsockopt(sd, IPPROTO_IP, IP_DROP_MEMBERSHIP,
-				 (char *) &imr, sizeof(struct ip_mreq));
+				 (char *) &imr, sizeof(imr));
 	} else {
 		memset(&imr6, 0, sizeof(imr6));
 		imr6.ipv6mr_multiaddr = ((struct sockaddr_in6 *) &global_data->vrrp_mcast_group6)->sin6_addr;
@@ -512,14 +647,11 @@ if_leave_vrrp_group(sa_family_t family, int sd, interface_t *ifp)
 	}
 
 	if (ret < 0) {
-		log_message(LOG_INFO, "cant do IP%s_DROP_MEMBERSHIP errno=%s (%d)",
-			    (family == AF_INET) ? "" : "V6", strerror(errno), errno);
-		close(sd);
+		log_message(LOG_INFO, "(%s): cant do IP%s_DROP_MEMBERSHIP errno=%s (%d)",
+			    ifp->ifname, (family == AF_INET) ? "" : "V6", strerror(errno), errno);
 		return -1;
 	}
 
-	/* Finally close the desc */
-	close(sd);
 	return 0;
 }
 
@@ -591,6 +723,38 @@ if_setsockopt_ipv6_checksum(int *sd)
 }
 
 
+int
+if_setsockopt_mcast_all(sa_family_t family, int *sd)
+{
+#ifndef IP_MULTICAST_ALL
+	/* IP_MULTICAST_ALL sockopt is available since Linux 2.6.31.
+	 * It seems reasonable to just skip the calls to if_setsockopt_mcast_all
+	 * if there is no support for that feature in header files */
+	return -1;
+#else
+	int ret;
+	unsigned char no = 0;
+
+	if (*sd < 0)
+		return -1;
+
+	if (family == AF_INET6)
+		return *sd;
+
+	/* Don't accept multicast packets we haven't requested */
+	ret = setsockopt(*sd, IPPROTO_IP, IP_MULTICAST_ALL, &no, sizeof(no));
+
+	if (ret < 0) {
+		log_message(LOG_INFO, "cant set IP_MULTICAST_ALL IP option. errno=%d (%m)",
+			    errno);
+		close(*sd);
+		*sd = -1;
+	}
+
+	return *sd;
+#endif
+}
+
 int
 if_setsockopt_mcast_loop(sa_family_t family, int *sd)
 {
@@ -644,15 +808,24 @@ if_setsockopt_mcast_if(sa_family_t family, int *sd, interface_t *ifp)
 	int ret;
 	unsigned int ifindex;
 
-	/* Not applicable for IPv4 */
-	if (*sd < 0 || family == AF_INET)
+	if (*sd < 0)
 		return -1;
 
 	/* Set interface for sending outbound datagrams */
 	ifindex = IF_INDEX(ifp);
-	ret = setsockopt(*sd, IPPROTO_IPV6, IPV6_MULTICAST_IF, &ifindex, sizeof(ifindex));
+	if ( family == AF_INET)
+	{
+		struct ip_mreqn imr;
+
+		memset(&imr, 0, sizeof(imr));
+		imr.imr_ifindex = IF_INDEX(ifp);
+		ret = setsockopt(*sd, IPPROTO_IP, IP_MULTICAST_IF, &imr, sizeof(imr));
+	}
+	else
+		ret = setsockopt(*sd, IPPROTO_IPV6, IPV6_MULTICAST_IF, &ifindex, sizeof(ifindex));
+
 	if (ret < 0) {
-		log_message(LOG_INFO, "cant set IPV6_MULTICAST_IF IP option. errno=%d (%m)", errno);
+		log_message(LOG_INFO, "cant set IP%s_MULTICAST_IF IP option. errno=%d (%m)", (family == AF_INET) ? "" : "V6", errno);
 		close(*sd);
 		*sd = -1;
 	}
@@ -680,25 +853,6 @@ if_setsockopt_priority(int *sd)
 	return *sd;
 }
 
-int
-if_setsockopt_sndbuf(int *sd, int val)
-{
-	int ret;
-
-	if (*sd < 0)
-		return -1;
-
-	/* sndbuf option */
-        ret = setsockopt(*sd, SOL_SOCKET, SO_SNDBUF, &val, sizeof(val));
-        if (ret < 0) {
-		log_message(LOG_INFO, "cant set SO_SNDBUF IP option. errno=%d (%m)", errno);
-		close(*sd);
-		*sd = -1;
-        }
-
-        return *sd;
-}
-
 int
 if_setsockopt_rcvbuf(int *sd, int val)
 {
