@@ -23,7 +23,8 @@
  */
 
 #include "vrrp_ipsecah.h"
-#include <openssl/md5.h>
+
+#define	BLOCK_SIZE	64
 
 /* hmac_md5 computation according to the RFCs 2085 & 2104 */
 void
@@ -31,9 +32,9 @@ hmac_md5(unsigned char *buffer, int buffer_len, unsigned char *key, int key_len,
 	 unsigned char *digest)
 {
 	MD5_CTX context;
-	unsigned char k_ipad[65];	/* inner padding - key XORd with ipad */
-	unsigned char k_opad[65];	/* outer padding - key XORd with opad */
-	unsigned char tk[16];
+	unsigned char k_ipad[BLOCK_SIZE+1];	/* inner padding - key XORd with ipad */
+	unsigned char k_opad[BLOCK_SIZE+1];	/* outer padding - key XORd with opad */
+	unsigned char tk[MD5_DIGEST_LENGTH];
 	int i;
 
 	/* Initialize data */
@@ -42,7 +43,7 @@ hmac_md5(unsigned char *buffer, int buffer_len, unsigned char *key, int key_len,
 	memset(tk, 0, sizeof (tk));
 
 	/* If the key is longer than 64 bytes => set it to key=MD5(key) */
-	if (key_len > 64) {
+	if (key_len > BLOCK_SIZE) {
 		MD5_CTX tctx;
 
 		/* Compute the MD5 digest */
@@ -51,7 +52,7 @@ hmac_md5(unsigned char *buffer, int buffer_len, unsigned char *key, int key_len,
 		MD5_Final(tk, &tctx);
 
 		key = tk;
-		key_len = 16;
+		key_len = MD5_DIGEST_LENGTH;
 	}
 
 	/* The global HMAC_MD5 algo looks like (rfc2085.2.2) :
@@ -67,20 +68,20 @@ hmac_md5(unsigned char *buffer, int buffer_len, unsigned char *key, int key_len,
 	memcpy(k_opad, key, key_len);
 
 	/* XOR key with ipad and opad values */
-	for (i = 0; i < 64; i++) {
+	for (i = 0; i < BLOCK_SIZE; i++) {
 		k_ipad[i] ^= 0x36;
 		k_opad[i] ^= 0x5c;
 	}
 
 	/* Compute inner MD5 */
 	MD5_Init(&context);				/* Init context for 1st pass */
-	MD5_Update(&context, k_ipad, 64);		/* start with inner pad */
+	MD5_Update(&context, k_ipad, BLOCK_SIZE);		/* start with inner pad */
 	MD5_Update(&context, buffer, buffer_len);	/* next with buffer datagram */
 	MD5_Final(digest, &context);			/* Finish 1st pass */
 
 	/* Compute outer MD5 */
-	MD5_Init(&context);			/* Init context for 2nd pass */
-	MD5_Update(&context, k_opad, 64);	/* start with inner pad */
-	MD5_Update(&context, digest, 16);	/* next result of 1st pass */
-	MD5_Final(digest, &context);		/* Finish 2nd pass */
+	MD5_Init(&context);				/* Init context for 2nd pass */
+	MD5_Update(&context, k_opad, BLOCK_SIZE);		/* start with inner pad */
+	MD5_Update(&context, digest, MD5_DIGEST_LENGTH); /* next result of 1st pass */
+	MD5_Final(digest, &context);			/* Finish 2nd pass */
 }
