@@ -33,50 +33,115 @@
 #include <errno.h>
 #include <time.h>
 #include <sys/uio.h>
+#include <stdarg.h>
 
 /* local include */
 #include "check_api.h"
 #include "vrrp_netlink.h"
-#include "vrrp_if.h"
+#ifdef _HAVE_VRRP_VMAC_
+#include "vrrp_vmac.h"
+#endif
 #include "logger.h"
 #include "memory.h"
 #include "scheduler.h"
 #include "utils.h"
+#include "bitops.h"
+#ifndef _HAVE_SOCK_NONBLOCK
+#include "old_socket.h"
+#endif
 
 /* Global vars */
-nl_handle_t nl_kernel;	/* Kernel reflection channel */
 nl_handle_t nl_cmd;	/* Command channel */
+int netlink_error_ignore; /* If we get this error, ignore it */
+
+/* Static vars */
+static nl_handle_t nl_kernel;	/* Kernel reflection channel */
 
 /* Create a socket to netlink interface_t */
-int
-netlink_socket(nl_handle_t *nl, unsigned long groups)
+static int
+netlink_socket(nl_handle_t *nl, int flags, int group, ...)
 {
-	socklen_t addr_len;
 	int ret;
+	va_list gp;
 
 	memset(nl, 0, sizeof (*nl));
 
-	nl->fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
+#ifdef _HAVE_LIBNL3_
+	/* We need to keep libnl3 in step with our netlink socket creation.  */
+	nl->sk = nl_socket_alloc();
+	if ( nl->sk == NULL ) {
+		log_message(LOG_INFO, "Netlink: Cannot allocate netlink socket" );
+		return -1;
+	}
+
+	ret = nl_connect(nl->sk, NETLINK_ROUTE);
+	if (ret != 0) {
+		log_message(LOG_INFO, "Netlink: Cannot open netlink socket : (%d)", ret);
+		return -1;
+	}
+
+	/* Unfortunately we can't call nl_socket_add_memberships() with variadic arguments
+	 * from a variadic argument list passed to us
+	 */
+	va_start(gp, group);
+	while (group != 0) {
+		if (group < 0) {
+			va_end(gp);
+			return -1;
+		}
+
+		if ((ret = nl_socket_add_membership(nl->sk, group))) {
+			log_message(LOG_INFO, "Netlink: Cannot add socket membership 0x%x : (%d)", group, ret);
+			return -1;
+		}
+
+		group = va_arg(gp,int);
+	}
+	va_end(gp);
+
+	if (flags & SOCK_NONBLOCK) {
+		if ((ret = nl_socket_set_nonblocking(nl->sk))) {
+			log_message(LOG_INFO, "Netlink: Cannot set netlink socket non-blocking : (%d)", ret);
+			return -1;
+		}
+	}
+
+	if ((ret = nl_socket_set_buffer_size(nl->sk, IF_DEFAULT_BUFSIZE, 0))) {
+		log_message(LOG_INFO, "Netlink: Cannot set netlink buffer size : (%d)", ret);
+		return -1;
+	}
+
+	nl->nl_pid = nl_socket_get_local_port(nl->sk);
+
+	nl->fd = nl_socket_get_fd(nl->sk);
+
+	/* Set CLOEXEC */
+	fcntl(nl->fd, F_SETFD, fcntl(nl->fd, F_GETFD) | FD_CLOEXEC);
+#else
+	socklen_t addr_len;
+	struct sockaddr_nl snl;
+#ifndef _HAVE_SOCK_NONBLOCK_
+	int sock_flags = flags;
+	flags &= ~SOCK_NONBLOCK;
+#endif
+
+	nl->fd = socket(AF_NETLINK, SOCK_RAW | SOCK_CLOEXEC | flags, NETLINK_ROUTE);
 	if (nl->fd < 0) {
 		log_message(LOG_INFO, "Netlink: Cannot open netlink socket : (%s)",
 		       strerror(errno));
 		return -1;
 	}
 
-	ret = fcntl(nl->fd, F_SETFL, O_NONBLOCK);
-	if (ret < 0) {
-		log_message(LOG_INFO,
-		       "Netlink: Cannot set netlink socket flags : (%s)",
-		       strerror(errno));
-		close(nl->fd);
+#ifndef _HAVE_SOCK_NONBLOCK_
+	if ((sock_flags & SOCK_NONBLOCK) &&
+	    set_sock_flags(nl->fd, F_SETFL, O_NONBLOCK))
 		return -1;
-	}
+#endif
 
-	memset(&nl->snl, 0, sizeof (nl->snl));
-	nl->snl.nl_family = AF_NETLINK;
-	nl->snl.nl_groups = groups;
+	memset(&snl, 0, sizeof (snl));
+	snl.nl_family = AF_NETLINK;
 
-	ret = bind(nl->fd, (struct sockaddr *) &nl->snl, sizeof (nl->snl));
+	ret = bind(nl->fd, (struct sockaddr *) &snl, sizeof (snl));
 	if (ret < 0) {
 		log_message(LOG_INFO, "Netlink: Cannot bind netlink socket : (%s)",
 		       strerror(errno));
@@ -84,26 +149,51 @@ netlink_socket(nl_handle_t *nl, unsigned long groups)
 		return -1;
 	}
 
-	addr_len = sizeof (nl->snl);
-	ret = getsockname(nl->fd, (struct sockaddr *) &nl->snl, &addr_len);
-	if (ret < 0 || addr_len != sizeof (nl->snl)) {
+	/* Join the requested groups */
+	va_start(gp, group);
+	while (group != 0) {
+		if (group < 0) {
+			va_end(gp);
+			return -1;
+		}
+
+		ret = setsockopt(nl->fd, SOL_NETLINK, NETLINK_ADD_MEMBERSHIP, &group, sizeof(group));
+		if (ret < 0) {
+			log_message(LOG_INFO, "Netlink: Cannot add membership on netlink socket : (%s)",
+			       strerror(errno));
+			va_end(gp);
+			return -1;
+		}
+
+		group = va_arg(gp,int);
+	}
+	va_end(gp);
+
+	addr_len = sizeof (snl);
+	ret = getsockname(nl->fd, (struct sockaddr *) &snl, &addr_len);
+	if (ret < 0 || addr_len != sizeof (snl)) {
 		log_message(LOG_INFO, "Netlink: Cannot getsockname : (%s)",
 		       strerror(errno));
 		close(nl->fd);
 		return -1;
 	}
 
-	if (nl->snl.nl_family != AF_NETLINK) {
+	if (snl.nl_family != AF_NETLINK) {
 		log_message(LOG_INFO, "Netlink: Wrong address family %d",
-		       nl->snl.nl_family);
+		       snl.nl_family);
 		close(nl->fd);
 		return -1;
 	}
 
-	nl->seq = time(NULL);
+	/* Save the port id for checking message source later */
+	nl->nl_pid = snl.nl_pid;
 
 	/* Set default rcvbuf size */
 	if_setsockopt_rcvbuf(&nl->fd, IF_DEFAULT_BUFSIZE);
+#endif
+
+	nl->seq = time(NULL);
+
 	if (nl->fd < 0)
 		return -1;
 
@@ -111,17 +201,21 @@ netlink_socket(nl_handle_t *nl, unsigned long groups)
 }
 
 /* Close a netlink socket */
-int
+static int
 netlink_close(nl_handle_t *nl)
 {
 	/* First of all release pending thread */
 	thread_cancel(nl->thread);
+#ifdef _HAVE_LIBNL3_
+	nl_socket_free(nl->sk);
+#else
 	close(nl->fd);
+#endif
 	return 0;
 }
 
 /* Set netlink socket channel as blocking */
-int
+static int
 netlink_set_block(nl_handle_t *nl, int *flags)
 {
 	if ((*flags = fcntl(nl->fd, F_GETFL, 0)) < 0) {
@@ -139,15 +233,25 @@ netlink_set_block(nl_handle_t *nl, int *flags)
 }
 
 /* Set netlink socket channel as non-blocking */
-int
+static int
 netlink_set_nonblock(nl_handle_t *nl, int *flags)
 {
+#ifdef _HAVE_LIBNL3_
+	int ret;
+
+	if ((ret = nl_socket_set_nonblocking(nl->sk)) < 0 ) {
+		log_message(LOG_INFO, "Netlink: Cannot set nonblocking : (%s)",
+			strerror(ret));
+		return -1;
+	}
+#else
 	*flags |= O_NONBLOCK;
 	if (fcntl(nl->fd, F_SETFL, *flags) < 0) {
 		log_message(LOG_INFO, "Netlink: Cannot F_SETFL socket : (%s)",
 		       strerror(errno));
 		return -1;
 	}
+#endif
 	return 0;
 }
 
@@ -155,15 +259,15 @@ netlink_set_nonblock(nl_handle_t *nl, int *flags)
 int
 addattr32(struct nlmsghdr *n, int maxlen, int type, uint32_t data)
 {
-	int len = RTA_LENGTH(4);
+	int len = RTA_LENGTH(sizeof(data));
 	struct rtattr *rta;
-	if (NLMSG_ALIGN(n->nlmsg_len) + len > maxlen)
+	if (n->nlmsg_len + NLMSG_ALIGN(len) > maxlen)
 		return -1;
-	rta = (struct rtattr*)(((char*)n) + NLMSG_ALIGN(n->nlmsg_len));
+	rta = (struct rtattr*)(((char*)n) + n->nlmsg_len);
 	rta->rta_type = type;
 	rta->rta_len = len;
-	memcpy(RTA_DATA(rta), &data, 4);
-	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + len;
+	memcpy(RTA_DATA(rta), &data, sizeof(data));
+	n->nlmsg_len += NLMSG_ALIGN(len);
 	return 0;
 }
 
@@ -173,14 +277,14 @@ addattr_l(struct nlmsghdr *n, int maxlen, int type, void *data, int alen)
 	int len = RTA_LENGTH(alen);
 	struct rtattr *rta;
 
-	if (NLMSG_ALIGN(n->nlmsg_len) + len > maxlen)
+	if (n->nlmsg_len + NLMSG_ALIGN(len) > maxlen)
 		return -1;
 
-	rta = (struct rtattr *) (((char *) n) + NLMSG_ALIGN(n->nlmsg_len));
+	rta = (struct rtattr *) (((char *) n) + n->nlmsg_len);
 	rta->rta_type = type;
 	rta->rta_len = len;
 	memcpy(RTA_DATA(rta), data, alen);
-	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + len;
+	n->nlmsg_len += NLMSG_ALIGN(len);
 
 	return 0;
 }
@@ -212,18 +316,26 @@ parse_rtattr(struct rtattr **tb, int max, struct rtattr *rta, int len)
 	}
 }
 
-char *
+#ifdef _HAVE_VRRP_VMAC_
+static void
+parse_rtattr_nested(struct rtattr **tb, int max, struct rtattr *rta)
+{
+        parse_rtattr(tb, max, RTA_DATA(rta), RTA_PAYLOAD(rta));
+}
+#endif
+
+const char *
 netlink_scope_n2a(int scope)
 {
-	if (scope == 0)
+	if (scope == RT_SCOPE_UNIVERSE)
 		return "global";
-	if (scope == 255)
+	if (scope == RT_SCOPE_NOWHERE)
 		return "nowhere";
-	if (scope == 254)
+	if (scope == RT_SCOPE_HOST)
 		return "host";
-	if (scope == 253)
+	if (scope == RT_SCOPE_LINK)
 		return "link";
-	if (scope == 200)
+	if (scope == RT_SCOPE_SITE)
 		return "site";
 	return "unknown";
 }
@@ -232,283 +344,18 @@ int
 netlink_scope_a2n(char *scope)
 {
 	if (!strcmp(scope, "global"))
-		return 0;
+		return RT_SCOPE_UNIVERSE;
 	if (!strcmp(scope, "nowhere"))
-		return 255;
+		return RT_SCOPE_NOWHERE;
 	if (!strcmp(scope, "host"))
-		return 254;
+		return RT_SCOPE_HOST;
 	if (!strcmp(scope, "link"))
-		return 253;
+		return RT_SCOPE_LINK;
 	if (!strcmp(scope, "site"))
-		return 200;
+		return RT_SCOPE_SITE;
 	return -1;
 }
 
-/* Our netlink parser */
-static int
-netlink_parse_info(int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
-		   nl_handle_t *nl, struct nlmsghdr *n)
-{
-	int status;
-	int ret = 0;
-	int error;
-
-	while (1) {
-		char buf[4096];
-		struct iovec iov = { buf, sizeof buf };
-		struct sockaddr_nl snl;
-		struct msghdr msg =
-		    { (void *) &snl, sizeof snl, &iov, 1, NULL, 0, 0 };
-		struct nlmsghdr *h;
-
-		status = recvmsg(nl->fd, &msg, 0);
-
-		if (status < 0) {
-			if (errno == EINTR)
-				continue;
-			if (errno == EWOULDBLOCK || errno == EAGAIN)
-				break;
-			log_message(LOG_INFO, "Netlink: Received message overrun (%m)");
-			continue;
-		}
-
-		if (status == 0) {
-			log_message(LOG_INFO, "Netlink: EOF");
-			return -1;
-		}
-
-		if (msg.msg_namelen != sizeof snl) {
-			log_message(LOG_INFO,
-			       "Netlink: Sender address length error: length %d",
-			       msg.msg_namelen);
-			return -1;
-		}
-
-		for (h = (struct nlmsghdr *) buf; NLMSG_OK(h, status);
-		     h = NLMSG_NEXT(h, status)) {
-			/* Finish of reading. */
-			if (h->nlmsg_type == NLMSG_DONE)
-				return ret;
-
-			/* Error handling. */
-			if (h->nlmsg_type == NLMSG_ERROR) {
-				struct nlmsgerr *err = (struct nlmsgerr *) NLMSG_DATA(h);
-
-				/*
-				 * If error == 0 then this is a netlink ACK.
-				 * return if not related to multipart message.
-				 */
-				if (err->error == 0) {
-					if (!(h->nlmsg_flags & NLM_F_MULTI))
-						return 0;
-					continue;
-				}
-
-				if (h->nlmsg_len < NLMSG_LENGTH(sizeof (struct nlmsgerr))) {
-					log_message(LOG_INFO,
-					       "Netlink: error: message truncated");
-					return -1;
-				}
-
-				if (n && (err->error == -EEXIST) &&
-				    ((n->nlmsg_type == RTM_NEWROUTE) ||
-				     (n->nlmsg_type == RTM_NEWADDR)))
-					return 0;
-
-				log_message(LOG_INFO,
-				       "Netlink: error: %s, type=(%u), seq=%u, pid=%d",
-				       strerror(-err->error),
-				       err->msg.nlmsg_type,
-				       err->msg.nlmsg_seq, err->msg.nlmsg_pid);
-
-				return -1;
-			}
-
-			/* Skip unsolicited messages from cmd channel */
-			if (nl != &nl_cmd && h->nlmsg_pid == nl_cmd.snl.nl_pid)
-				continue;
-
-			error = (*filter) (&snl, h);
-			if (error < 0) {
-				log_message(LOG_INFO, "Netlink: filter function error");
-				ret = error;
-			}
-		}
-
-		/* After error care. */
-		if (msg.msg_flags & MSG_TRUNC) {
-			log_message(LOG_INFO, "Netlink: error: message truncated");
-			continue;
-		}
-		if (status) {
-			log_message(LOG_INFO, "Netlink: error: data remnant size %d",
-			       status);
-			return -1;
-		}
-	}
-
-	return ret;
-}
-
-/* Out talk filter */
-static int
-netlink_talk_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
-{
-	log_message(LOG_INFO, "Netlink: ignoring message type 0x%04x",
-	       h->nlmsg_type);
-	return 0;
-}
-
-/* send message to netlink kernel socket, then receive response */
-int
-netlink_talk(nl_handle_t *nl, struct nlmsghdr *n)
-{
-	int status;
-	int ret, flags;
-	struct sockaddr_nl snl;
-	struct iovec iov = { (void *) n, n->nlmsg_len };
-	struct msghdr msg = { (void *) &snl, sizeof snl, &iov, 1, NULL, 0, 0 };
-
-	memset(&snl, 0, sizeof snl);
-	snl.nl_family = AF_NETLINK;
-
-	n->nlmsg_seq = ++nl->seq;
-
-	/* Request Netlink acknowledgement */
-	n->nlmsg_flags |= NLM_F_ACK;
-
-	/* Send message to netlink interface. */
-	status = sendmsg(nl->fd, &msg, 0);
-	if (status < 0) {
-		log_message(LOG_INFO, "Netlink: sendmsg() error: %s",
-		       strerror(errno));
-		return -1;
-	}
-
-	/* Set blocking flag */
-	ret = netlink_set_block(nl, &flags);
-	if (ret < 0)
-		log_message(LOG_INFO, "Netlink: Warning, couldn't set "
-		       "blocking flag to netlink socket...");
-
-	status = netlink_parse_info(netlink_talk_filter, nl, n);
-
-	/* Restore previous flags */
-	if (ret == 0)
-		netlink_set_nonblock(nl, &flags);
-	return status;
-}
-
-/* Fetch a specific type information from netlink kernel */
-static int
-netlink_request(nl_handle_t *nl, int family, int type)
-{
-	int status;
-	struct sockaddr_nl snl;
-	struct {
-		struct nlmsghdr nlh;
-		struct rtgenmsg g;
-	} req;
-
-	/* Cleanup the room */
-	memset(&snl, 0, sizeof (snl));
-	snl.nl_family = AF_NETLINK;
-
-	req.nlh.nlmsg_len = sizeof (req);
-	req.nlh.nlmsg_type = type;
-	req.nlh.nlmsg_flags = NLM_F_ROOT | NLM_F_MATCH | NLM_F_REQUEST;
-	req.nlh.nlmsg_pid = 0;
-	req.nlh.nlmsg_seq = ++nl->seq;
-	req.g.rtgen_family = family;
-
-	status = sendto(nl->fd, (void *) &req, sizeof (req)
-			, 0, (struct sockaddr *) &snl, sizeof (snl));
-	if (status < 0) {
-		log_message(LOG_INFO, "Netlink: sendto() failed: %s",
-		       strerror(errno));
-		return -1;
-	}
-	return 0;
-}
-
-/* Netlink interface link lookup filter */
-static int
-netlink_if_link_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
-{
-	struct ifinfomsg *ifi;
-	struct rtattr *tb[IFLA_MAX + 1];
-	interface_t *ifp;
-	int i, len;
-	char *name;
-
-	ifi = NLMSG_DATA(h);
-
-	if (h->nlmsg_type != RTM_NEWLINK)
-		return 0;
-
-	len = h->nlmsg_len - NLMSG_LENGTH(sizeof (struct ifinfomsg));
-	if (len < 0)
-		return -1;
-
-	/* Interface name lookup */
-	memset(tb, 0, sizeof (tb));
-	parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifi), len);
-	if (tb[IFLA_IFNAME] == NULL)
-		return -1;
-	name = (char *) RTA_DATA(tb[IFLA_IFNAME]);
-
-	/* Return if loopback */
-	if (ifi->ifi_type == ARPHRD_LOOPBACK)
-		return 0;
-
-	/* Skip it if already exist */
-	ifp = if_get_by_ifname(name);
-	if (ifp) {
-		if (!ifp->vmac) {
-			if_vmac_reflect_flags(ifi->ifi_index, ifi->ifi_flags);
-			ifp->flags = ifi->ifi_flags;
-		}
-		return 0;
-	}
-
-	/* Fill the interface structure */
-	ifp = (interface_t *) MALLOC(sizeof(interface_t));
-	memcpy(ifp->ifname, name, strlen(name));
-	ifp->ifindex = ifi->ifi_index;
-	ifp->mtu = *(int *) RTA_DATA(tb[IFLA_MTU]);
-	ifp->hw_type = ifi->ifi_type;
-
-	if (!ifp->vmac) {
-		if_vmac_reflect_flags(ifi->ifi_index, ifi->ifi_flags);
-		ifp->flags = ifi->ifi_flags;
-		ifp->base_ifindex = ifi->ifi_index;
-	}
-
-	if (tb[IFLA_ADDRESS]) {
-		int hw_addr_len = RTA_PAYLOAD(tb[IFLA_ADDRESS]);
-
-		if (hw_addr_len > IF_HWADDR_MAX)
-			log_message(LOG_ERR, "MAC address for %s is too large: %d",
-			       name, hw_addr_len);
-		else {
-			ifp->hw_addr_len = hw_addr_len;
-			memcpy(ifp->hw_addr, RTA_DATA(tb[IFLA_ADDRESS]),
-			       hw_addr_len);
-			for (i = 0; i < hw_addr_len; i++)
-				if (ifp->hw_addr[i] != 0)
-					break;
-			if (i == hw_addr_len)
-				ifp->hw_addr_len = 0;
-			else
-				ifp->hw_addr_len = hw_addr_len;
-		}
-	}
-
-	/* Queue this new interface_t */
-	if_add_queue(ifp);
-	return 0;
-}
-
 /*
  * Netlink interface address lookup filter
  * We need to handle multiple primary address and
@@ -571,23 +418,382 @@ netlink_if_address_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 	return 0;
 }
 
+/* Our netlink parser */
+static int
+netlink_parse_info(int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
+		   nl_handle_t *nl, struct nlmsghdr *n)
+{
+	int status;
+	int ret = 0;
+	int error;
+
+	while (1) {
+		char buf[4096];
+		struct iovec iov = {
+			.iov_base = buf,
+			.iov_len = sizeof buf
+		};
+		struct sockaddr_nl snl;
+		struct msghdr msg = {
+			.msg_name = &snl,
+			.msg_namelen = sizeof(snl),
+			.msg_iov = &iov,
+			.msg_iovlen = 1,
+			.msg_control = NULL,
+			.msg_controllen = 0,
+			.msg_flags = 0
+		};
+		struct nlmsghdr *h;
+
+		status = recvmsg(nl->fd, &msg, 0);
+
+		if (status < 0) {
+			if (errno == EINTR)
+				continue;
+			if (errno == EWOULDBLOCK || errno == EAGAIN)
+				break;
+			log_message(LOG_INFO, "Netlink: Received message overrun (%m)");
+			continue;
+		}
+
+		if (status == 0) {
+			log_message(LOG_INFO, "Netlink: EOF");
+			return -1;
+		}
+
+		if (msg.msg_namelen != sizeof snl) {
+			log_message(LOG_INFO,
+			       "Netlink: Sender address length error: length %d",
+			       msg.msg_namelen);
+			return -1;
+		}
+
+		for (h = (struct nlmsghdr *) buf; NLMSG_OK(h, status);
+		     h = NLMSG_NEXT(h, status)) {
+			/* Finish of reading. */
+			if (h->nlmsg_type == NLMSG_DONE)
+				return ret;
+
+			/* Error handling. */
+			if (h->nlmsg_type == NLMSG_ERROR) {
+				struct nlmsgerr *err = (struct nlmsgerr *) NLMSG_DATA(h);
+
+				/*
+				 * If error == 0 then this is a netlink ACK.
+				 * return if not related to multipart message.
+				 */
+				if (err->error == 0) {
+					if (!(h->nlmsg_flags & NLM_F_MULTI))
+						return 0;
+					continue;
+				}
+
+				if (h->nlmsg_len < NLMSG_LENGTH(sizeof (struct nlmsgerr))) {
+					log_message(LOG_INFO,
+					       "Netlink: error: message truncated");
+					return -1;
+				}
+
+				if (n && (err->error == -EEXIST) &&
+				    ((n->nlmsg_type == RTM_NEWROUTE) ||
+				     (n->nlmsg_type == RTM_NEWADDR)))
+					return 0;
+
+				/* If have more than one IPv4 address in the same CIDR
+				 * and the "primary" address is removed, unless promote_secondaries
+				 * is configured on the interface, all the "secondary" addresses
+				 * in the same CIDR are deleted */
+				if (n && err->error == -EADDRNOTAVAIL &&
+				    n->nlmsg_type == RTM_DELADDR) {
+					netlink_if_address_filter(NULL, n);
+					if (!(h->nlmsg_flags & NLM_F_MULTI))
+						return 0;
+					continue;
+				}
+				if (netlink_error_ignore != -err->error)
+					log_message(LOG_INFO,
+					       "Netlink: error: %s, type=(%u), seq=%u, pid=%d",
+					       strerror(-err->error),
+					       err->msg.nlmsg_type,
+					       err->msg.nlmsg_seq, err->msg.nlmsg_pid);
+
+				return -1;
+			}
+
+			/* Skip unsolicited messages from cmd channel */
+			if (nl != &nl_cmd && h->nlmsg_pid == nl_cmd.nl_pid)
+				continue;
+
+			error = (*filter) (&snl, h);
+			if (error < 0) {
+				log_message(LOG_INFO, "Netlink: filter function error");
+				ret = error;
+			}
+		}
+
+		/* After error care. */
+		if (msg.msg_flags & MSG_TRUNC) {
+			log_message(LOG_INFO, "Netlink: error: message truncated");
+			continue;
+		}
+		if (status) {
+			log_message(LOG_INFO, "Netlink: error: data remnant size %d",
+			       status);
+			return -1;
+		}
+	}
+
+	return ret;
+}
+
+/* Out talk filter */
+static int
+netlink_talk_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
+{
+	log_message(LOG_INFO, "Netlink: ignoring message type 0x%04x",
+	       h->nlmsg_type);
+	return 0;
+}
+
+/* send message to netlink kernel socket, then receive response */
+int
+netlink_talk(nl_handle_t *nl, struct nlmsghdr *n)
+{
+	int status;
+	int ret, flags;
+	struct sockaddr_nl snl;
+	struct iovec iov = {
+		.iov_base = n,
+		.iov_len = n->nlmsg_len
+	};
+	struct msghdr msg = {
+		.msg_name = &snl,
+		.msg_namelen = sizeof(snl),
+		.msg_iov = &iov,
+		.msg_iovlen = 1,
+		.msg_control = NULL,
+		.msg_controllen = 0,
+		.msg_flags = 0
+	};
+
+	memset(&snl, 0, sizeof snl);
+	snl.nl_family = AF_NETLINK;
+
+	n->nlmsg_seq = ++nl->seq;
+
+	/* Request Netlink acknowledgement */
+	n->nlmsg_flags |= NLM_F_ACK;
+
+	/* Send message to netlink interface. */
+	status = sendmsg(nl->fd, &msg, 0);
+	if (status < 0) {
+		log_message(LOG_INFO, "Netlink: sendmsg() error: %s",
+		       strerror(errno));
+		return -1;
+	}
+
+	/* Set blocking flag */
+	ret = netlink_set_block(nl, &flags);
+	if (ret < 0)
+		log_message(LOG_INFO, "Netlink: Warning, couldn't set "
+		       "blocking flag to netlink socket...");
+
+	status = netlink_parse_info(netlink_talk_filter, nl, n);
+
+	/* Restore previous flags */
+	if (ret == 0)
+		netlink_set_nonblock(nl, &flags);
+	return status;
+}
+
+/* Fetch a specific type information from netlink kernel */
+static int
+netlink_request(nl_handle_t *nl, int family, int type)
+{
+	int status;
+	struct sockaddr_nl snl;
+	struct {
+		struct nlmsghdr nlh;
+		struct rtgenmsg g;
+	} req;
+
+	/* Cleanup the room */
+	memset(&snl, 0, sizeof (snl));
+	snl.nl_family = AF_NETLINK;
+
+	req.nlh.nlmsg_len = sizeof (req);
+	req.nlh.nlmsg_type = type;
+	req.nlh.nlmsg_flags = NLM_F_ROOT | NLM_F_MATCH | NLM_F_REQUEST;
+	req.nlh.nlmsg_pid = 0;
+	req.nlh.nlmsg_seq = ++nl->seq;
+	req.g.rtgen_family = family;
+
+	status = sendto(nl->fd, (void *) &req, sizeof (req)
+			, 0, (struct sockaddr *) &snl, sizeof (snl));
+	if (status < 0) {
+		log_message(LOG_INFO, "Netlink: sendto() failed: %s",
+		       strerror(errno));
+		return -1;
+	}
+	return 0;
+}
+
+static int
+netlink_if_link_populate(interface_t *ifp, struct rtattr *tb[], struct ifinfomsg *ifi)
+{
+	char *name;
+	int i;
+#ifdef _HAVE_VRRP_VMAC_
+	struct rtattr* linkinfo[IFLA_INFO_MAX+1];
+	struct rtattr* linkattr[IFLA_MACVLAN_MAX+1];
+	interface_t *ifp_base;
+#endif
+
+	name = (char *) RTA_DATA(tb[IFLA_IFNAME]);
+	/* Fill the interface structure */
+	memcpy(ifp->ifname, name, strlen(name));
+	ifp->ifindex = ifi->ifi_index;
+	ifp->mtu = *(int *) RTA_DATA(tb[IFLA_MTU]);
+	ifp->hw_type = ifi->ifi_type;
+
+	if (tb[IFLA_ADDRESS]) {
+		int hw_addr_len = RTA_PAYLOAD(tb[IFLA_ADDRESS]);
+
+		if (hw_addr_len > IF_HWADDR_MAX) {
+			log_message(LOG_ERR, "MAC address for %s is too large: %d",
+				name, hw_addr_len);
+			return -1;
+		}
+		else {
+			ifp->hw_addr_len = hw_addr_len;
+			memcpy(ifp->hw_addr, RTA_DATA(tb[IFLA_ADDRESS]),
+				hw_addr_len);
+			for (i = 0; i < hw_addr_len; i++)
+				if (ifp->hw_addr[i] != 0)
+					break;
+			if (i == hw_addr_len)
+				ifp->hw_addr_len = 0;
+			else
+				ifp->hw_addr_len = hw_addr_len;
+		}
+	}
+
+#ifdef _HAVE_VRRP_VMAC_
+	/* See if this interface is a MACVLAN of ours */
+	if (tb[IFLA_LINKINFO] && tb[IFLA_LINK]){
+		/* If appears that the value of *(int*)RTA_DATA(tb[IFLA_LINKINFO]) is 0x1000c
+		 *   for macvlan.  0x10000 for nested data, or'ed with 0x0c for macvlan;
+		 *   other values are 0x09 for vlan, 0x0b for bridge, 0x08 for tun, -1 for no
+		 *   underlying interface.
+		 *
+		 * I can't find where in the kernel these values are set or defined, so use
+		 * the string as below.
+		 */
+		parse_rtattr_nested(linkinfo, IFLA_INFO_MAX, tb[IFLA_LINKINFO]);
+
+		if (linkinfo[IFLA_INFO_KIND] &&
+		    RTA_PAYLOAD(linkinfo[IFLA_INFO_KIND]) >= strlen(macvlan_ll_kind) &&
+		    !strncmp(macvlan_ll_kind, RTA_DATA(linkinfo[IFLA_INFO_KIND]), strlen(macvlan_ll_kind)) &&
+		    linkinfo[IFLA_INFO_DATA]) {
+			parse_rtattr_nested(linkattr, IFLA_MACVLAN_MAX, linkinfo[IFLA_INFO_DATA]);
+
+			if (linkattr[IFLA_MACVLAN_MODE] &&
+			    *(int*)RTA_DATA(linkattr[IFLA_MACVLAN_MODE]) == MACVLAN_MODE_PRIVATE) {
+				ifp->base_ifindex = *(int*)RTA_DATA(tb[IFLA_LINK]);
+				ifp->vmac = true;
+			}
+		}
+	}
+
+	if (!ifp->vmac)
+#endif
+	{
+#ifdef _HAVE_VRRP_VMAC_
+		if_vmac_reflect_flags(ifi->ifi_index, ifi->ifi_flags);
+#endif
+		ifp->flags = ifi->ifi_flags;
+#ifdef _HAVE_VRRP_VMAC_
+		ifp->base_ifindex = ifi->ifi_index;
+#endif
+	}
+#ifdef _HAVE_VRRP_VMAC_
+	else {
+		if ((ifp_base = if_get_by_ifindex(ifp->base_ifindex)))
+			ifp->flags = ifp_base->flags;
+	}
+#endif
+
+	return 1;
+}
+
+/* Netlink interface link lookup filter */
+static int
+netlink_if_link_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
+{
+	struct ifinfomsg *ifi;
+	struct rtattr *tb[IFLA_MAX + 1];
+	interface_t *ifp;
+	int len, status;
+	char *name;
+
+	ifi = NLMSG_DATA(h);
+
+	if (h->nlmsg_type != RTM_NEWLINK)
+		return 0;
+
+	len = h->nlmsg_len - NLMSG_LENGTH(sizeof (struct ifinfomsg));
+	if (len < 0)
+		return -1;
+
+	/* Interface name lookup */
+	memset(tb, 0, sizeof (tb));
+	parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifi), len);
+	if (tb[IFLA_IFNAME] == NULL)
+		return -1;
+	name = (char *) RTA_DATA(tb[IFLA_IFNAME]);
+
+	/* Return if loopback */
+	if (ifi->ifi_type == ARPHRD_LOOPBACK)
+		return 0;
+
+	/* Skip it if already exist */
+	ifp = if_get_by_ifname(name);
+	if (ifp) {
+#ifdef _HAVE_VRRP_VMAC_
+		if (!ifp->vmac)
+#endif
+		{
+#ifdef _HAVE_VRRP_VMAC_
+			if_vmac_reflect_flags(ifi->ifi_index, ifi->ifi_flags);
+#endif
+			ifp->flags = ifi->ifi_flags;
+		}
+		return 0;
+	}
+
+	/* Fill the interface structure */
+	ifp = (interface_t *) MALLOC(sizeof(interface_t));
+
+	status = netlink_if_link_populate(ifp, tb, ifi);
+	if (status < 0) {
+		FREE(ifp);
+		return -1;
+	}
+	/* Queue this new interface_t */
+	if_add_queue(ifp);
+	return 0;
+}
+
 /* Interfaces lookup bootstrap function */
 int
 netlink_interface_lookup(void)
 {
 	nl_handle_t nlh;
 	int status = 0;
-	int ret, flags;
 
-	if (netlink_socket(&nlh, 0) < 0)
+	if (netlink_socket(&nlh, 0, 0) < 0)
 		return -1;
 
-	/* Set blocking flag */
-	ret = netlink_set_block(&nlh, &flags);
-	if (ret < 0)
-		log_message(LOG_INFO, "Netlink: Warning, couldn't set "
-		       "blocking flag to netlink socket...");
-
 	/* Interface lookup */
 	if (netlink_request(&nlh, AF_PACKET, RTM_GETLINK) < 0) {
 		status = -1;
@@ -606,17 +812,10 @@ netlink_address_lookup(void)
 {
 	nl_handle_t nlh;
 	int status = 0;
-	int ret, flags;
 
-	if (netlink_socket(&nlh, 0) < 0)
+	if (netlink_socket(&nlh, 0, 0) < 0)
 		return -1;
 
-	/* Set blocking flag */
-	ret = netlink_set_block(&nlh, &flags);
-	if (ret < 0)
-		log_message(LOG_INFO, "Netlink: Warning, couldn't set "
-		       "blocking flag to netlink socket...");
-
 	/* IPv4 Address lookup */
 	if (netlink_request(&nlh, AF_INET, RTM_GETADDR) < 0) {
 		status = -1;
@@ -643,7 +842,7 @@ netlink_reflect_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 	struct ifinfomsg *ifi;
 	struct rtattr *tb[IFLA_MAX + 1];
 	interface_t *ifp;
-	int len;
+	int len, status;
 
 	ifi = NLMSG_DATA(h);
 	if (!(h->nlmsg_type == RTM_NEWLINK || h->nlmsg_type == RTM_DELLINK))
@@ -663,18 +862,45 @@ netlink_reflect_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 	if (ifi->ifi_type == ARPHRD_LOOPBACK)
 		return 0;
 
-	/* find the interface_t */
+	/* find the interface_t. If the interface doesn't exist in the interface
+	 * list and this is a new interface add it to the interface list.
+	 * If an interface with the same name exists overwrite the older
+	 * structure and fill it with the new interface information.
+	 */
 	ifp = if_get_by_ifindex(ifi->ifi_index);
-	if (!ifp)
-		return -1;
+	if (!ifp) {
+		if (h->nlmsg_type == RTM_NEWLINK) {
+			char *name;
+			name = (char *) RTA_DATA(tb[IFLA_IFNAME]);
+			ifp = if_get_by_ifname(name);
+			if (!ifp) {
+				ifp = (interface_t *) MALLOC(sizeof(interface_t));
+				if_add_queue(ifp);
+			} else {
+				memset(ifp, 0, sizeof(interface_t));
+			}
+			status = netlink_if_link_populate(ifp, tb, ifi);
+			if (status < 0)
+				return -1;
+		} else {
+			if (__test_bit(LOG_DETAIL_BIT, &debug))
+				log_message(LOG_INFO, "Unknown interface %s deleted", (char *)tb[IFLA_IFNAME]);
+			return 0;
+		}
+	}
 
 	/*
 	 * Update flags.
 	 * VMAC interfaces should never update it own flags, only be reflected
 	 * by the base interface flags.
 	 */
-	if (!ifp->vmac) {
+#ifdef _HAVE_VRRP_VMAC_
+	if (!ifp->vmac)
+#endif
+	{
+#ifdef _HAVE_VRRP_VMAC_
 		if_vmac_reflect_flags(ifi->ifi_index, ifi->ifi_flags);
+#endif
 		ifp->flags = ifi->ifi_flags;
 	}
 
@@ -703,7 +929,7 @@ netlink_broadcast_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 	return 0;
 }
 
-int
+static int
 kernel_netlink(thread_t * thread)
 {
 	nl_handle_t *nl = THREAD_ARG(thread);
@@ -718,8 +944,6 @@ kernel_netlink(thread_t * thread)
 void
 kernel_netlink_init(void)
 {
-	unsigned long groups;
-
 	/* Start with a netlink address lookup */
 	netlink_address_lookup();
 
@@ -728,8 +952,7 @@ kernel_netlink_init(void)
 	 * subscribtion. We subscribe to LINK and ADDR
 	 * netlink broadcast messages.
 	 */
-	groups = RTMGRP_LINK | RTMGRP_IPV4_IFADDR | RTMGRP_IPV6_IFADDR;
-	netlink_socket(&nl_kernel, groups);
+	netlink_socket(&nl_kernel, SOCK_NONBLOCK, RTNLGRP_LINK, RTNLGRP_IPV4_IFADDR, RTNLGRP_IPV6_IFADDR, 0);
 
 	if (nl_kernel.fd > 0) {
 		log_message(LOG_INFO, "Registering Kernel netlink reflector");
@@ -739,7 +962,7 @@ kernel_netlink_init(void)
 		log_message(LOG_INFO, "Error while registering Kernel netlink reflector channel");
 
 	/* Prepare netlink command channel. */
-	netlink_socket(&nl_cmd, 0);
+	netlink_socket(&nl_cmd, SOCK_NONBLOCK, 0);
 	if (nl_cmd.fd > 0)
 		log_message(LOG_INFO, "Registering Kernel netlink command channel");
 	else
