@@ -113,9 +113,9 @@ script_open(char *script)
 }
 
 static int
-notify_script_exec(char* script, char *type, int state_num, char* name, int prio)
+notify_script_exec(char* script, const char *type, int state_num, char* name, int prio)
 {
-	char *state = "{UNKNOWN}";
+	const char *state = "{UNKNOWN}";
 	char *command_line = NULL;
 	int size = 0;
 
@@ -128,7 +128,7 @@ notify_script_exec(char* script, char *type, int state_num, char* name, int prio
 	 * Thus, the length of the buffer will be:
 	 *
 	 *     ( strlen(script) + 3 ) + ( strlen(type) + 1 ) + ( strlen(name) + 1 ) +
-	 *      ( strlen(state) + 2 ) + ( strlen(prio) + 1 ) + 1 
+	 *      ( strlen(state) + 2 ) + ( strlen(prio) + 1 ) + 1
 	 *
 	 * Note that the prio will be indicated as zero for a group.
 	 *
