@@ -1,14 +1,14 @@
-/* 
+/*
  * Soft:        Keepalived is a failover program for the LVS project
  *              <www.linuxvirtualserver.org>. It monitor & manipulate
  *              a loadbalanced server pool using multi-layer checks.
- * 
+ *
  * Part:        Configuration file parser/reader. Place into the dynamic
  *              data structure representation the conf file representing
  *              the loadbalanced server pool.
- *  
+ *
  * Author:      Alexandre Cassen, <acassen@linux-vs.org>
- *              
+ *
  *              This program is distributed in the hope that it will be useful,
  *              but WITHOUT ANY WARRANTY; without even the implied warranty of
  *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -27,10 +27,15 @@
 #include "vrrp_sync.h"
 #include "vrrp_index.h"
 #include "vrrp_if.h"
+#ifdef _HAVE_VRRP_VMAC_
 #include "vrrp_vmac.h"
+#endif
 #include "vrrp.h"
 #include "global_data.h"
 #include "global_parser.h"
+#ifdef _WITH_LVS_
+#include "check_parser.h"
+#endif
 #include "logger.h"
 #include "parser.h"
 #include "memory.h"
@@ -43,6 +48,7 @@ static_addresses_handler(vector_t *strvec)
 	alloc_value_block(strvec, alloc_saddress);
 }
 
+#ifdef _HAVE_FIB_ROUTING_
 /* Static routes handler */
 static void
 static_routes_handler(vector_t *strvec)
@@ -50,17 +56,58 @@ static_routes_handler(vector_t *strvec)
 	alloc_value_block(strvec, alloc_sroute);
 }
 
+/* Static rules handler */
+static void
+static_rules_handler(vector_t *strvec)
+{
+	alloc_value_block(strvec, alloc_srule);
+}
+#endif
+
 /* VRRP handlers */
 static void
 vrrp_sync_group_handler(vector_t *strvec)
 {
-	alloc_vrrp_sync_group(vector_slot(strvec, 1));
+	list l;
+	element e;
+	vrrp_sgroup_t *sg;
+	char* gname;
+
+	if (vector_count(strvec) != 2) {
+		log_message(LOG_INFO, "vrrp_sync_group must have a name - skipping");
+		skip_block();
+		return;
+	}
+
+	gname = vector_slot(strvec, 1);
+
+	/* check group doesn't already exist */
+	if (!LIST_ISEMPTY(vrrp_data->vrrp_sync_group)) {
+		l = vrrp_data->vrrp_sync_group;
+		for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
+			sg = ELEMENT_DATA(e);
+			if (!strcmp(gname,sg->gname)) {
+				log_message(LOG_INFO, "vrrp sync group %s already defined", gname);
+				skip_block();
+				return;
+			}
+		}
+	}
+
+	alloc_vrrp_sync_group(gname);
 }
 static void
 vrrp_group_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
-	vgroup->iname = read_value_block();
+
+	if (vgroup->iname) {
+		log_message(LOG_INFO, "Group list already specified for sync group %s", vgroup->gname);
+		skip_block();
+		return;
+	}
+
+	vgroup->iname = read_value_block(strvec);
 }
 static void
 vrrp_gnotify_backup_handler(vector_t *strvec)
@@ -105,48 +152,62 @@ vrrp_gglobal_tracking_handler(vector_t *strvec)
 static void
 vrrp_handler(vector_t *strvec)
 {
-	alloc_vrrp(vector_slot(strvec, 1));
+	list l;
+	element e;
+	vrrp_t *vrrp;
+	char *iname;
+
+	if (vector_count(strvec) != 2) {
+		log_message(LOG_INFO, "vrrp_instance must have a name");
+		skip_block();
+		return;
+	}
+
+	iname = vector_slot(strvec,1);
+
+	/* Make sure the vrrp instance doesn't already exist */
+	if (!LIST_ISEMPTY(vrrp_data->vrrp)) {
+		l = vrrp_data->vrrp;
+		for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
+			vrrp = ELEMENT_DATA(e);
+			if (!strcmp(iname,vrrp->iname)) {
+				log_message(LOG_INFO, "vrrp instance %s already defined", iname );
+				skip_block();
+				return;
+			}
+		}
+	}
+
+	alloc_vrrp(iname);
 }
+#ifdef _HAVE_VRRP_VMAC_
 static void
 vrrp_vmac_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	interface_t *ifp = vrrp->ifp;
-	struct sockaddr_storage *saddr = &vrrp->saddr;
+	interface_t *ifp;
 
 	__set_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags);
-	if (!vrrp->saddr.ss_family) {
-		if (!ifp) {
-			log_message(LOG_INFO, "Please define interface keyword before use_vmac keyword");
-			return;
-		} else {
-			if (vrrp->family == AF_INET) {
-				inet_ip4tosockaddr(&ifp->sin_addr, saddr);
-			} else if (vrrp->family == AF_INET6) {
-				inet_ip6tosockaddr(&ifp->sin6_addr, saddr);
-				/* IPv6 use-case: Binding to link-local address requires an interface */
-				inet_ip6scopeid(IF_INDEX(ifp), saddr);
-			}
+
+	if (vector_size(strvec) >= 2) {
+		strncpy(vrrp->vmac_ifname, vector_slot(strvec, 1), IFNAMSIZ - 1);
+
+		/* Check if the interface exists and is a macvlan we can use */
+		if ((ifp = if_get_by_ifname(vrrp->vmac_ifname)) &&
+		    !ifp->vmac) {
+			log_message(LOG_INFO, "(%s): interface %s already exists and is not a private macvlan; ignoring vmac if_name", vrrp->iname, vrrp->vmac_ifname);
+			vrrp->vmac_ifname[0] = '\0';
 		}
 	}
-	if (vector_size(strvec) == 2) {
-		strncpy(vrrp->vmac_ifname, vector_slot(strvec, 1),
-			IFNAMSIZ - 1);
-	} else if (vrrp->vrid) {
-		snprintf(vrrp->vmac_ifname, IFNAMSIZ, "vrrp.%d", vrrp->vrid);
-	} else {
-		return;
-	}
-
-	netlink_link_add_vmac(vrrp);
 }
 static void
 vrrp_vmac_xmit_base_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags))
-		__set_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags);
+
+	__set_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags);
 }
+#endif
 static void
 vrrp_unicast_peer_handler(vector_t *strvec)
 {
@@ -156,13 +217,14 @@ static void
 vrrp_native_ipv6_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+
+	if (vrrp->family == AF_INET) {
+		log_message(LOG_INFO,"(%s): Cannot specify native_ipv6 with IPv4 addresses", vrrp->iname);
+		return;
+	}
+
 	vrrp->family = AF_INET6;
-
-	if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags))
-		log_message(LOG_INFO, "You should declare native_ipv6 before use_vmac!");
-
-	if (vrrp->auth_type != VRRP_AUTH_NONE)
-		vrrp->auth_type = VRRP_AUTH_NONE;
+	vrrp->version = VRRP_VERSION_3;
 }
 static void
 vrrp_state_handler(vector_t *strvec)
@@ -175,6 +237,8 @@ vrrp_state_handler(vector_t *strvec)
 		vrrp->wantstate = VRRP_STATE_MAST;
 		vrrp->init_state = VRRP_STATE_MAST;
 	}
+	else if (strcmp(str, "BACKUP"))
+		log_message(LOG_INFO,"(%s): unknown state '%s', defaulting to BACKUP", vrrp->iname, str);
 
 	/* set eventual sync group */
 	if (vgroup)
@@ -192,9 +256,6 @@ vrrp_int_handler(vector_t *strvec)
 				    , name, vrrp->iname);
 		return;
 	}
-
-	if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags))
-		netlink_link_add_vmac(vrrp);
 }
 static void
 vrrp_track_int_handler(vector_t *strvec)
@@ -216,7 +277,6 @@ static void
 vrrp_srcip_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	interface_t *ifp = vrrp->ifp;
 	struct sockaddr_storage *saddr = &vrrp->saddr;
 	int ret;
 
@@ -228,21 +288,13 @@ vrrp_srcip_handler(vector_t *strvec)
 		return;
 	}
 
-	if (saddr->ss_family != vrrp->family) {
+	if (vrrp->family == AF_UNSPEC)
+		vrrp->family = saddr->ss_family;
+	else if (saddr->ss_family != vrrp->family) {
 		log_message(LOG_ERR, "Configuration error: VRRP instance[%s] and unicast src address"
 				     "[%s] MUST be of the same family !!! Skipping..."
 				   , vrrp->iname, FMT_STR_VSLOT(strvec, 1));
-		memset(saddr, 0, sizeof(struct sockaddr_storage));
-	}
-
-	/* IPv6 use-case: Binding to link-local address requires an interface.
-	 * Just specify scope_id for all address types */
-	if (saddr->ss_family == AF_INET6) {
-		if (!ifp) {
-			log_message(LOG_INFO, "Please define interface keyword before mcast_src_ip keyword");
-			return;
-		}
-		inet_ip6scopeid(IF_INDEX(ifp), saddr);
+		saddr->ss_family = AF_UNSPEC;
 	}
 }
 static void
@@ -255,44 +307,38 @@ vrrp_vrid_handler(vector_t *strvec)
 		log_message(LOG_INFO, "VRRP Error : VRID not valid !");
 		log_message(LOG_INFO,
 		       "             must be between 1 & 255. reconfigure !");
-	} else {
-		alloc_vrrp_bucket(vrrp);
-		if (__test_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags)) {
-			if (strlen(vrrp->vmac_ifname) == 0)
-				snprintf(vrrp->vmac_ifname, IFNAMSIZ, "vrrp.%d", vrrp->vrid);
-			netlink_link_add_vmac(vrrp);
-		}
+
+		vrrp->vrid = 0;
+		return;
 	}
+
+	alloc_vrrp_bucket(vrrp);
 }
 static void
 vrrp_prio_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->effective_priority = vrrp->base_priority = atoi(vector_slot(strvec, 1));
+	vrrp->base_priority = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_PRIORITY(vrrp->base_priority)) {
-		log_message(LOG_INFO, "VRRP Error : Priority not valid !");
-		log_message(LOG_INFO,
-		       "             must be between 1 & 255. reconfigure !");
-		log_message(LOG_INFO,
-			    "             Using default value : %d\n", VRRP_PRIO_DFL);
-		vrrp->effective_priority = vrrp->base_priority = VRRP_PRIO_DFL;
+		log_message(LOG_INFO, "(%s): Priority not valid! must be between 1 & 255. Reconfigure !", vrrp->iname);
+		log_message(LOG_INFO, "%*sUsing default value : %d", (int)strlen(vrrp->iname) + 4, "", VRRP_PRIO_DFL);
+
+		vrrp->base_priority = VRRP_PRIO_DFL;
 	}
+	vrrp->effective_priority = vrrp->base_priority;
 }
 static void
 vrrp_adv_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->adver_int = atof(vector_slot(strvec, 1)) * 100; /* multiply with 100 to get decimal value */
+	int adver_int = atof(vector_slot(strvec, 1)) * TIMER_HZ;
 
-	/* Simple check. Note that using VRRPv2 with 0.01s advert interval will not report an error */
-	if (VRRP_IS_BAD_ADVERT_INT(vrrp->adver_int)) {
-		log_message(LOG_INFO, "VRRP Error : Advert interval not valid !");
-		log_message(LOG_INFO, "             must be >=1sec for VRRPv2 or >=0.01sec for VRRPv3.\n");
-		log_message(LOG_INFO, "             Using default value : 1sec");
-		vrrp->adver_int = 100;
-	}
-	vrrp->adver_int *= TIMER_HZ / 100.0;
+	/* Simple check - just positive */
+	if (adver_int <= 0)
+		log_message(LOG_INFO, "(%s): Advert interval (%s) not valid! Must be > 0 - ignoring", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
+	else
+		vrrp->adver_int = adver_int;
 }
 static void
 vrrp_debug_handler(vector_t *strvec)
@@ -301,12 +347,45 @@ vrrp_debug_handler(vector_t *strvec)
 	vrrp->debug = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_DEBUG_INT(vrrp->debug)) {
-		log_message(LOG_INFO, "VRRP Error : Debug interval not valid !");
-		log_message(LOG_INFO, "             must be between 0-4");
+		log_message(LOG_INFO, "(%s): Debug value not valid! must be between 0-4", vrrp->iname);
 		vrrp->debug = 0;
 	}
 }
 static void
+vrrp_skip_check_adv_addr_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	int res;
+
+	if (vector_size(strvec) >= 2) {
+		res = check_true_false(vector_slot(strvec, 1));
+		if (res >= 0)
+			vrrp->skip_check_adv_addr = res;
+		else
+			log_message(LOG_INFO, "(%s): invalid skip_check_adv_addr %s specified", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
+	} else {
+		/* Defaults to true */
+		vrrp->skip_check_adv_addr = true;
+	}
+}
+static void
+vrrp_strict_mode_handler(vector_t *strvec)
+{
+	int res;
+
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vector_size(strvec) >= 2) {
+		res = check_true_false(vector_slot(strvec, 1));
+		if (res >= 0)
+			vrrp->strict_mode = res;
+		else
+			log_message(LOG_INFO, "(%s): invalid strict_mode %s specified", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
+	} else {
+		/* Defaults to true */
+		vrrp->strict_mode = true;
+	}
+}
+static void
 vrrp_nopreempt_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
@@ -325,9 +404,7 @@ vrrp_preempt_delay_handler(vector_t *strvec)
 	vrrp->preempt_delay = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_PREEMPT_DELAY(vrrp->preempt_delay)) {
-		log_message(LOG_INFO, "VRRP Error : Preempt_delay not valid !");
-		log_message(LOG_INFO, "             must be between 0-%d",
-		       TIMER_MAX_SEC);
+		log_message(LOG_INFO, "(%s): Preempt_delay not valid! must be between 0-%d", vrrp->iname, TIMER_MAX_SEC);
 		vrrp->preempt_delay = 0;
 	}
 	vrrp->preempt_delay *= TIMER_HZ;
@@ -378,7 +455,17 @@ static void
 vrrp_lvs_syncd_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->lvs_syncd_if = set_value(strvec);
+
+	log_message(LOG_INFO, "(%s): Specifying lvs_sync_daemon_interface against a vrrp is deprecated.", vrrp->iname);  /* Deprecated after v1.2.19 */
+	log_message(LOG_INFO, "      %*sPlease use global lvs_sync_daemon_interface and lvs_sync_daemon_vrrp", (int)strlen(vrrp->iname) - 2, "");
+
+	if (global_data->lvs_syncd_if) {
+		log_message(LOG_INFO, "(%s): lvs_sync_daemon_interface has already been specified as %s - ignoring", vrrp->iname, global_data->lvs_syncd_if);
+		return;
+	}
+
+	global_data->lvs_syncd_if = set_value(strvec);
+	global_data->lvs_syncd_vrrp = vrrp;
 }
 static void
 vrrp_garp_delay_handler(vector_t *strvec)
@@ -410,16 +497,54 @@ vrrp_garp_refresh_rep_handler(vector_t *strvec)
 	if (vrrp->garp_refresh_rep < 1)
 		vrrp->garp_refresh_rep = 1;
 }
+
+static void
+vrrp_garp_lower_prio_delay_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp->garp_lower_prio_delay = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+}
+static void
+vrrp_garp_lower_prio_rep_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp->garp_lower_prio_rep = atoi(vector_slot(strvec, 1));
+	/* Allow 0 GARP messages to be sent */
+	if ( vrrp->garp_lower_prio_rep < 0 )
+		vrrp->garp_lower_prio_rep = 0;
+}
+static void
+vrrp_lower_prio_no_advert_handler(vector_t *strvec)
+{
+	int res;
+
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vector_size(strvec) >= 2) {
+		res = check_true_false(vector_slot(strvec, 1));
+		if (res >= 0)
+			vrrp->lower_prio_no_advert = res;
+		else
+			log_message(LOG_INFO, "(%s): invalid lower_prio_no_advert %s specified", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
+	} else {
+		/* Defaults to true */
+		vrrp->lower_prio_no_advert = true;
+	}
+}
+
+
+#if defined _WITH_VRRP_AUTH_
 static void
 vrrp_auth_type_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	char *str = vector_slot(strvec, 1);
 
-	if (!strcmp(str, "AH") && vrrp->family == AF_INET)
+	if (!strcmp(str, "AH"))
 		vrrp->auth_type = VRRP_AUTH_AH;
-	else if (!strcmp(str, "PASS") && vrrp->family == AF_INET)
+	else if (!strcmp(str, "PASS"))
 		vrrp->auth_type = VRRP_AUTH_PASS;
+	else
+		log_message(LOG_INFO, "(%s): unknown authentication type '%s'", vrrp->iname, str);
 }
 static void
 vrrp_auth_pass_handler(vector_t *strvec)
@@ -438,6 +563,7 @@ vrrp_auth_pass_handler(vector_t *strvec)
 	memset(vrrp->auth_data, 0, max_size);
 	memcpy(vrrp->auth_data, str, str_len);
 }
+#endif
 static void
 vrrp_vip_handler(vector_t *strvec)
 {
@@ -445,10 +571,11 @@ vrrp_vip_handler(vector_t *strvec)
 	char *buf;
 	char *str = NULL;
 	vector_t *vec = NULL;
-	int nbvip = 0;
+	int address_family;
 
 	buf = (char *) MALLOC(MAXBUF);
 	while (read_line(buf, MAXBUF)) {
+		address_family = AF_UNSPEC;
 		vec = alloc_strvec(buf);
 		if (vec) {
 			str = vector_slot(vec, 0);
@@ -458,17 +585,18 @@ vrrp_vip_handler(vector_t *strvec)
 			}
 
 			if (vector_size(vec)) {
-				nbvip++;
-				if (nbvip > VRRP_MAX_VIP) {
-					log_message(LOG_INFO,
-					       "VRRP_Instance(%s) "
-					       "trunc to the first %d VIPs.",
-					       vrrp->iname, VRRP_MAX_VIP);
-					log_message(LOG_INFO,
-					       "  => Declare others VIPs into"
-					       " the excluded vip block");
-				} else
-					alloc_vrrp_vip(vec);
+				alloc_vrrp_vip(vec);
+				if (!LIST_ISEMPTY(vrrp->vip))
+					address_family = IP_FAMILY((ip_address_t*)LIST_TAIL_DATA(vrrp->vip));
+			}
+
+			if (address_family != AF_UNSPEC) {
+				if (vrrp->family == AF_UNSPEC)
+					vrrp->family = address_family;
+				else if (address_family != vrrp->family) {
+					log_message(LOG_INFO, "(%s): address family must match VRRP instance [%s] - ignoring", vrrp->iname, str);
+					free_list_element(vrrp->vip, vrrp->vip->tail);
+				}
 			}
 
 			free_strvec(vec);
@@ -482,12 +610,19 @@ vrrp_evip_handler(vector_t *strvec)
 {
 	alloc_value_block(strvec, alloc_vrrp_evip);
 }
+#ifdef _HAVE_FIB_ROUTING_
 static void
 vrrp_vroutes_handler(vector_t *strvec)
 {
 	alloc_value_block(strvec, alloc_vrrp_vroute);
 }
 static void
+vrrp_vrules_handler(vector_t *strvec)
+{
+	alloc_value_block(strvec, alloc_vrrp_vrule);
+}
+#endif
+static void
 vrrp_script_handler(vector_t *strvec)
 {
 	alloc_vrrp_script(vector_slot(strvec, 1));
@@ -544,10 +679,17 @@ vrrp_version_handler(vector_t *strvec)
 
 	uint8_t version = atoi(vector_slot(strvec, 1));
 	if (VRRP_IS_BAD_VERSION(version)) {
-		log_message(LOG_INFO, "VRRP Error : Version not valid !\n");
-		log_message(LOG_INFO, "             must be between either 2 or 3. reconfigure !\n");
+		log_message(LOG_INFO, "VRRP Error : Version not valid !");
+		log_message(LOG_INFO, "             must be between either 2 or 3. reconfigure !");
 		return;
 	}
+
+	if ((vrrp->version && vrrp->version != version) ||
+	    (version == VRRP_VERSION_2 && vrrp->family == AF_INET6)) {
+		log_message(LOG_INFO, "(%s): vrrp_version conflicts with configured or deduced version; ignoring.", vrrp->iname);
+		return;
+	}
+
 	vrrp->version = version;
 }
 
@@ -559,18 +701,113 @@ vrrp_accept_handler(vector_t *strvec)
 	vrrp->accept = true;
 }
 
-vector_t *
-vrrp_init_keywords(void)
+static void
+garp_group_handler(vector_t *strvec)
 {
-	/* global definitions mapping */
-	global_init_keywords();
+	alloc_garp_delay();
+}
+static void
+garp_group_garp_interval_handler(vector_t *strvec)
+{
+	garp_delay_t *delay = LIST_TAIL_DATA(garp_delay);
 
+	delay->garp_interval.tv_usec = atof(vector_slot(strvec, 1)) * 1000000;
+	delay->garp_interval.tv_sec = delay->garp_interval.tv_usec / 1000000;
+	delay->garp_interval.tv_usec %= 1000000;
+	delay->have_garp_interval = true;
+
+	if (delay->garp_interval.tv_sec >= 1)
+		log_message(LOG_INFO, "The garp_interval is very large - %s seconds", FMT_STR_VSLOT(strvec,1));
+}
+static void
+garp_group_gna_interval_handler(vector_t *strvec)
+{
+	garp_delay_t *delay = LIST_TAIL_DATA(garp_delay);
+
+	delay->gna_interval.tv_usec = atof(vector_slot(strvec, 1)) * 1000000;
+	delay->gna_interval.tv_sec = delay->gna_interval.tv_usec / 1000000;
+	delay->gna_interval.tv_usec %= 1000000;
+	delay->have_gna_interval = true;
+
+	if (delay->gna_interval.tv_sec >= 1)
+		log_message(LOG_INFO, "The gna_interval is very large - %s seconds", FMT_STR_VSLOT(strvec,1));
+}
+static void
+garp_group_interface_handler(vector_t *strvec)
+{
+	interface_t *ifp = if_get_by_ifname(vector_slot(strvec, 1));
+	if (!ifp) {
+		log_message(LOG_INFO, "Unknown interface %s specified for garp_group - ignoring", FMT_STR_VSLOT(strvec, 1));
+		return;
+	}
+
+	if (ifp->garp_delay) {
+		log_message(LOG_INFO, "garp_group already specified for %s - ignoring", FMT_STR_VSLOT(strvec, 1));
+		return;
+	}
+
+#ifdef _HAVE_VRRP_VMAC_
+	if (ifp->vmac) {
+		log_message(LOG_INFO, "Cannot specify garp_delay on a vmac (%s) - ignoring", ifp->ifname);
+		return;
+	}
+#endif
+	ifp->garp_delay = LIST_TAIL_DATA(garp_delay);
+}
+static void
+garp_group_interfaces_handler(vector_t *strvec)
+{
+	garp_delay_t *delay = LIST_TAIL_DATA(garp_delay);
+	interface_t *ifp;
+        vector_t *interface_vec = read_value_block(strvec);
+	int i;
+	garp_delay_t *gd;
+	element e;
+
+	/* First set the next aggregation group number */
+	delay->aggregation_group = 1;
+	for (e = LIST_HEAD(garp_delay); e; ELEMENT_NEXT(e)) {
+		gd = ELEMENT_DATA(e);
+		if (gd->aggregation_group && gd != delay)
+			delay->aggregation_group++;
+	}
+
+        for (i = 0; i < vector_size(interface_vec); i++) {
+		ifp = if_get_by_ifname(vector_slot(interface_vec, i));
+		if (!ifp) {
+			log_message(LOG_INFO, "Unknown interface %s specified for garp_group - ignoring", FMT_STR_VSLOT(interface_vec, i));
+			continue;
+		}
+
+		if (ifp->garp_delay) {
+			log_message(LOG_INFO, "garp_group already specified for %s - ignoring", FMT_STR_VSLOT(strvec, 1));
+			continue;
+		}
+
+#ifdef _HAVE_VRRP_VMAC_
+		if (ifp->vmac) {
+			log_message(LOG_INFO, "Cannot specify garp_delay on a vmac (%s) - ignoring", ifp->ifname);
+			continue;
+		}
+#endif
+		ifp->garp_delay = delay;
+	}
+
+        free_strvec(interface_vec);
+}
+
+void
+init_vrrp_keywords(bool active)
+{
 	/* Static routes mapping */
-	install_keyword_root("static_ipaddress", &static_addresses_handler);
-	install_keyword_root("static_routes", &static_routes_handler);
+	install_keyword_root("static_ipaddress", &static_addresses_handler, active);
+#ifdef _HAVE_FIB_ROUTING_
+	install_keyword_root("static_routes", &static_routes_handler, active);
+	install_keyword_root("static_rules", &static_rules_handler, active);
+#endif
 
 	/* VRRP Instance mapping */
-	install_keyword_root("vrrp_sync_group", &vrrp_sync_group_handler);
+	install_keyword_root("vrrp_sync_group", &vrrp_sync_group_handler, active);
 	install_keyword("group", &vrrp_group_handler);
 	install_keyword("notify_backup", &vrrp_gnotify_backup_handler);
 	install_keyword("notify_master", &vrrp_gnotify_master_handler);
@@ -578,9 +815,18 @@ vrrp_init_keywords(void)
 	install_keyword("notify", &vrrp_gnotify_handler);
 	install_keyword("smtp_alert", &vrrp_gsmtp_handler);
 	install_keyword("global_tracking", &vrrp_gglobal_tracking_handler);
-	install_keyword_root("vrrp_instance", &vrrp_handler);
+
+	install_keyword_root("garp_group", &garp_group_handler, active);
+	install_keyword("garp_interval", &garp_group_garp_interval_handler);
+	install_keyword("gna_interval", &garp_group_gna_interval_handler);
+	install_keyword("interface", &garp_group_interface_handler);
+	install_keyword("interfaces", &garp_group_interfaces_handler);
+
+	install_keyword_root("vrrp_instance", &vrrp_handler, active);
+#ifdef _HAVE_VRRP_VMAC_
 	install_keyword("use_vmac", &vrrp_vmac_handler);
 	install_keyword("vmac_xmit_base", &vrrp_vmac_xmit_base_handler);
+#endif
 	install_keyword("unicast_peer", &vrrp_unicast_peer_handler);
 	install_keyword("native_ipv6", &vrrp_native_ipv6_handler);
 	install_keyword("state", &vrrp_state_handler);
@@ -596,8 +842,13 @@ vrrp_init_keywords(void)
 	install_keyword("advert_int", &vrrp_adv_handler);
 	install_keyword("virtual_ipaddress", &vrrp_vip_handler);
 	install_keyword("virtual_ipaddress_excluded", &vrrp_evip_handler);
+#ifdef _HAVE_FIB_ROUTING_
 	install_keyword("virtual_routes", &vrrp_vroutes_handler);
+	install_keyword("virtual_rules", &vrrp_vrules_handler);
+#endif
 	install_keyword("accept", &vrrp_accept_handler);
+	install_keyword("skip_check_adv_addr", &vrrp_skip_check_adv_addr_handler);
+	install_keyword("strict_mode", &vrrp_strict_mode_handler);
 	install_keyword("preempt", &vrrp_preempt_handler);
 	install_keyword("nopreempt", &vrrp_nopreempt_handler);
 	install_keyword("preempt_delay", &vrrp_preempt_delay_handler);
@@ -613,18 +864,35 @@ vrrp_init_keywords(void)
 	install_keyword("garp_master_refresh", &vrrp_garp_refresh_handler);
 	install_keyword("garp_master_repeat", &vrrp_garp_rep_handler);
 	install_keyword("garp_master_refresh_repeat", &vrrp_garp_refresh_rep_handler);
+	install_keyword("garp_lower_prio_delay", &vrrp_garp_lower_prio_delay_handler);
+	install_keyword("garp_lower_prio_repeat", &vrrp_garp_lower_prio_rep_handler);
+	install_keyword("lower_prio_no_advert", &vrrp_lower_prio_no_advert_handler);
+#if defined _WITH_VRRP_AUTH_
 	install_keyword("authentication", NULL);
 	install_sublevel();
 	install_keyword("auth_type", &vrrp_auth_type_handler);
 	install_keyword("auth_pass", &vrrp_auth_pass_handler);
 	install_sublevel_end();
-	install_keyword_root("vrrp_script", &vrrp_script_handler);
+#endif
+	install_keyword_root("vrrp_script", &vrrp_script_handler, active);
 	install_keyword("script", &vrrp_vscript_script_handler);
 	install_keyword("interval", &vrrp_vscript_interval_handler);
 	install_keyword("timeout", &vrrp_vscript_timeout_handler);
 	install_keyword("weight", &vrrp_vscript_weight_handler);
 	install_keyword("rise", &vrrp_vscript_rise_handler);
 	install_keyword("fall", &vrrp_vscript_fall_handler);
+}
+
+vector_t *
+vrrp_init_keywords(void)
+{
+	/* global definitions mapping */
+	global_init_keywords();
+
+	init_vrrp_keywords(true);
+#ifdef _WITH_LVS_
+	init_check_keywords(false);
+#endif
 
 	return keywords;
 }
