@@ -23,18 +23,24 @@
 #include "vrrp_scheduler.h"
 #include "vrrp_ipsecah.h"
 #include "vrrp_if.h"
+#ifdef _HAVE_VRRP_VMAC_
 #include "vrrp_vmac.h"
+#endif
 #include "vrrp.h"
 #include "vrrp_sync.h"
 #include "vrrp_notify.h"
 #include "vrrp_netlink.h"
 #include "vrrp_data.h"
 #include "vrrp_index.h"
+#include "vrrp_arp.h"
+#include "vrrp_ndisc.h"
+#include "vrrp_if.h"
 #include "ipvswrapper.h"
 #include "memory.h"
 #include "notify.h"
 #include "list.h"
 #include "logger.h"
+#include "timer.h"
 #include "main.h"
 #include "smtp.h"
 #include "signals.h"
@@ -43,6 +49,10 @@
 #include "vrrp_snmp.h"
 #endif
 
+/* global vars */
+timeval_t garp_next_time;
+thread_t *garp_thread;
+
 /* VRRP FSM (Finite State Machine) design.
  *
  * The state transition diagram implemented is :
@@ -81,14 +91,16 @@ static int vrrp_script_child_timeout_thread(thread_t * thread);
 static int vrrp_script_child_thread(thread_t * thread);
 static int vrrp_script_thread(thread_t * thread);
 
-struct {
+static int vrrp_read_dispatcher_thread(thread_t *);
+
+static struct {
 	void (*read) (vrrp_t *, char *, int);
-	void (*read_to) (vrrp_t *);
+	void (*read_timeout) (vrrp_t *);
 } VRRP_FSM[VRRP_MAX_FSM_STATE + 1] =
 {
 /*    Stream Read Handlers      |    Stream Read_to handlers   *
  *------------------------------+------------------------------*/
-	{NULL, 				NULL},
+	{NULL,				NULL},
 	{vrrp_backup,			vrrp_goto_master},	/*  BACKUP          */
 	{vrrp_leave_master,		vrrp_master},		/*  MASTER          */
 	{vrrp_leave_fault,		vrrp_fault},		/*  FAULT           */
@@ -126,13 +138,13 @@ struct {
  * FSM since it will speed up convergence to init state.
  * Additionnaly, we have implemented some other handlers into the matrix
  * in order to speed up group synchronization takeover. For instance
- * transitions : 
+ * transitions :
  *    o B->B: To catch wantstate MASTER transition to force sync group
  *            to this transition state too.
  *    o F->F: To speed up FAULT state transition if group is not already
  *            synced to FAULT state.
  */
-struct {
+static struct {
 	void (*handler) (vrrp_t *);
 } VRRP_TSM[VRRP_MAX_TSM_STATE + 1][VRRP_MAX_TSM_STATE + 1] =
 {
@@ -231,13 +243,19 @@ vrrp_init_state(list l)
 					 vrrp, vrrp->adver_int);
 		}
 
-		if (vrrp->wantstate == VRRP_STATE_MAST) {
+		if (vrrp->wantstate == VRRP_STATE_MAST
+			|| vrrp->wantstate == VRRP_STATE_GOTO_MASTER) {
 #ifdef _HAVE_IPVS_SYNCD_
 			/* Check if sync daemon handling is needed */
-			if (vrrp->lvs_syncd_if)
+			if (global_data->lvs_syncd_if)
 				ipvs_syncd_cmd(IPVS_STARTDAEMON,
-					       vrrp->lvs_syncd_if, IPVS_MASTER,
-					       vrrp->vrid);
+					       global_data->lvs_syncd_if,
+					       IPVS_MASTER,
+					       global_data->lvs_syncd_syncid,
+					       false);
+#endif
+#ifdef _WITH_SNMP_RFCV3_
+			vrrp->stats->master_reason = VRRPV3_MASTER_REASON_PREEMPTED;
 #endif
 			vrrp->state = VRRP_STATE_GOTO_MASTER;
 		} else {
@@ -245,20 +263,23 @@ vrrp_init_state(list l)
 			    + VRRP_TIMER_SKEW(vrrp);
 #ifdef _HAVE_IPVS_SYNCD_
 			/* Check if sync daemon handling is needed */
-			if (vrrp->lvs_syncd_if)
+			if (global_data->lvs_syncd_if &&
+			    global_data->lvs_syncd_vrrp == vrrp)
 				ipvs_syncd_cmd(IPVS_STARTDAEMON,
-					       vrrp->lvs_syncd_if, IPVS_BACKUP,
-					       vrrp->vrid);
+					       global_data->lvs_syncd_if,
+					       IPVS_BACKUP,
+					       global_data->lvs_syncd_syncid,
+					       false);
 #endif
 			log_message(LOG_INFO, "VRRP_Instance(%s) Entering BACKUP STATE",
 			       vrrp->iname);
 
 			/* Set BACKUP state */
-			vrrp_restore_interface(vrrp, 0);
+			vrrp_restore_interface(vrrp, false, false);
 			vrrp->state = VRRP_STATE_BACK;
 			vrrp_smtp_notifier(vrrp);
 			notify_instance_exec(vrrp, VRRP_STATE_BACK);
-#ifdef _WITH_SNMP_
+#ifdef _WITH_SNMP_KEEPALIVED_
 			vrrp_snmp_instance_trap(vrrp);
 #endif
 			vrrp->last_transition = timer_now();
@@ -269,12 +290,15 @@ vrrp_init_state(list l)
 					vgroup->state = VRRP_STATE_BACK;
 					vrrp_sync_smtp_notifier(vgroup);
 					notify_group_exec(vgroup, VRRP_STATE_BACK);
-#ifdef _WITH_SNMP_
+#ifdef _WITH_SNMP_KEEPALIVED_
 					vrrp_snmp_group_trap(vgroup);
 #endif
 				}
 			}
 		}
+#ifdef _WITH_SNMP_RFC_
+		vrrp->stats->uptime = timer_now();
+#endif
 	}
 }
 
@@ -426,7 +450,7 @@ already_exist_sock(list l, sa_family_t family, int proto, int ifindex, int unica
 	return 0;
 }
 
-void
+static void
 alloc_sock(sa_family_t family, list l, int proto, int ifindex, int unicast)
 {
 	sock_t *new;
@@ -450,12 +474,17 @@ vrrp_create_sockpool(list l)
 
 	for (e = LIST_HEAD(p); e; ELEMENT_NEXT(e)) {
 		vrrp = ELEMENT_DATA(e);
-		ifindex = (__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags)) ? IF_BASE_INDEX(vrrp->ifp) :
+		ifindex =
+#ifdef _HAVE_VRRP_VMAC_
+			  (__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags)) ? IF_BASE_INDEX(vrrp->ifp) :
+#endif
 										    IF_INDEX(vrrp->ifp);
 		unicast = !LIST_ISEMPTY(vrrp->unicast_peer);
+#if defined _WITH_VRRP_AUTH_
 		if (vrrp->version == VRRP_VERSION_2 && vrrp->auth_type == VRRP_AUTH_AH)
 			proto = IPPROTO_IPSEC_AH;
 		else
+#endif
 			proto = IPPROTO_VRRP;
 
 		/* add the vrrp element if not exist */
@@ -472,7 +501,7 @@ vrrp_open_sockpool(list l)
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		sock = ELEMENT_DATA(e);
-		sock->fd_in = open_vrrp_socket(sock->family, sock->proto,
+		sock->fd_in = open_vrrp_read_socket(sock->family, sock->proto,
 					       sock->ifindex, sock->unicast);
 		if (sock->fd_in == -1)
 			sock->fd_out = -1;
@@ -496,16 +525,21 @@ vrrp_set_fds(list l)
 		sock = ELEMENT_DATA(e_sock);
 		for (e_vrrp = LIST_HEAD(p); e_vrrp; ELEMENT_NEXT(e_vrrp)) {
 			vrrp = ELEMENT_DATA(e_vrrp);
-			ifindex = (__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags)) ? IF_BASE_INDEX(vrrp->ifp) :
+			ifindex =
+#ifdef _HAVE_VRRP_VMAC_
+				  (__test_bit(VRRP_VMAC_XMITBASE_BIT, &vrrp->vmac_flags)) ? IF_BASE_INDEX(vrrp->ifp) :
+#endif
 											    IF_INDEX(vrrp->ifp);
 			unicast = !LIST_ISEMPTY(vrrp->unicast_peer);
+#if defined _WITH_VRRP_AUTH_
 			if (vrrp->version == VRRP_VERSION_2 && vrrp->auth_type == VRRP_AUTH_AH)
 				proto = IPPROTO_IPSEC_AH;
 			else
+#endif
 				proto = IPPROTO_VRRP;
 
 			if ((sock->ifindex == ifindex)	&&
-                (sock->family == vrrp->family) &&
+		(sock->family == vrrp->family) &&
 			    (sock->proto == proto)	&&
 			    (sock->unicast == unicast)) {
 				vrrp->fd_in = sock->fd_in;
@@ -556,7 +590,7 @@ vrrp_dispatcher_init(thread_t * thread)
 void
 vrrp_dispatcher_release(vrrp_data_t *data)
 {
-	free_list(data->vrrp_socket_pool);
+	free_list(&data->vrrp_socket_pool);
 }
 
 static void
@@ -582,7 +616,7 @@ vrrp_backup(vrrp_t * vrrp, char *buffer, int len)
 		if (vrrp->state != VRRP_STATE_FAULT) {
 			notify_instance_exec(vrrp, VRRP_STATE_FAULT);
 			vrrp->state = VRRP_STATE_FAULT;
-#ifdef _WITH_SNMP_
+#ifdef _WITH_SNMP_KEEPALIVED_
 			vrrp_snmp_instance_trap(vrrp);
 #endif
 		}
@@ -631,6 +665,7 @@ vrrp_leave_master(vrrp_t * vrrp, char *buffer, int len)
 	}
 }
 
+#ifdef _WITH_VRRP_AUTH_
 static void
 vrrp_ah_sync(vrrp_t *vrrp)
 {
@@ -643,6 +678,7 @@ vrrp_ah_sync(vrrp_t *vrrp)
 	vrrp->wantstate = VRRP_STATE_BACK;
 	vrrp_state_leave_master(vrrp);
 }
+#endif
 
 static void
 vrrp_leave_fault(vrrp_t * vrrp, char *buffer, int len)
@@ -650,43 +686,33 @@ vrrp_leave_fault(vrrp_t * vrrp, char *buffer, int len)
 	if (!VRRP_ISUP(vrrp))
 		return;
 
-	if (vrrp_state_fault_rx(vrrp, buffer, len)) {
-		if (vrrp->sync) {
-			if (vrrp_sync_leave_fault(vrrp)) {
-				log_message(LOG_INFO,
-				       "VRRP_Instance(%s) prio is higher than received advert",
-				       vrrp->iname);
-				vrrp_become_master(vrrp, buffer, len);
-			}
-		} else {
+	if (vrrp_state_fault_rx(vrrp, buffer, len) == VRRP_STATE_MAST) {
+		if (!vrrp->sync || vrrp_sync_leave_fault(vrrp)) {
 			log_message(LOG_INFO,
 			       "VRRP_Instance(%s) prio is higher than received advert",
 			       vrrp->iname);
 			vrrp_become_master(vrrp, buffer, len);
+#ifdef _WITH_SNMP_RFC_
+#ifdef _WITH_SNMP_RFCV3_
+			vrrp->stats->master_reason = VRRPV3_MASTER_REASON_PREEMPTED;
+#endif
+			vrrp->stats->uptime = timer_now();
+#endif
 		}
 	} else {
-		if (vrrp->sync) {
-			if (vrrp_sync_leave_fault(vrrp)) {
-				log_message(LOG_INFO, "VRRP_Instance(%s) Entering BACKUP STATE",
-				       vrrp->iname);
-				vrrp->state = VRRP_STATE_BACK;
-				vrrp_smtp_notifier(vrrp);
-				notify_instance_exec(vrrp, VRRP_STATE_BACK);
-#ifdef _WITH_SNMP_
-				vrrp_snmp_instance_trap(vrrp);
-#endif
-				vrrp->last_transition = timer_now();
-			}
-		} else {
+		if (!vrrp->sync || vrrp_sync_leave_fault(vrrp)) {
 			log_message(LOG_INFO, "VRRP_Instance(%s) Entering BACKUP STATE",
 			       vrrp->iname);
 			vrrp->state = VRRP_STATE_BACK;
 			vrrp_smtp_notifier(vrrp);
 			notify_instance_exec(vrrp, VRRP_STATE_BACK);
-#ifdef _WITH_SNMP_
+#ifdef _WITH_SNMP_KEEPALIVED_
 			vrrp_snmp_instance_trap(vrrp);
 #endif
 			vrrp->last_transition = timer_now();
+#ifdef _WITH_SNMP_RFC_
+			vrrp->stats->uptime = vrrp->last_transition;
+#endif
 		}
 	}
 }
@@ -703,17 +729,24 @@ vrrp_goto_master(vrrp_t * vrrp)
 		vrrp->state = VRRP_STATE_FAULT;
 		vrrp->ms_down_timer = 3 * vrrp->adver_int + VRRP_TIMER_SKEW(vrrp);
 		notify_instance_exec(vrrp, VRRP_STATE_FAULT);
-#ifdef _WITH_SNMP_
+#ifdef _WITH_SNMP_KEEPALIVED_
 		vrrp_snmp_instance_trap(vrrp);
 #endif
 		vrrp->last_transition = timer_now();
 	} else {
+#if defined _WITH_VRRP_AUTH_
 		/* If becoming MASTER in IPSEC AH AUTH, we reset the anti-replay */
 		if (vrrp->version == VRRP_VERSION_2 && vrrp->ipsecah_counter->cycle) {
 			vrrp->ipsecah_counter->cycle = 0;
 			vrrp->ipsecah_counter->seq_number = 0;
 		}
+#endif
 
+#ifdef _WITH_SNMP_RFCV3_
+		if ((vrrp->version == VRRP_VERSION_2 && vrrp->ms_down_timer >= 3 * vrrp->adver_int) ||
+		    (vrrp->version == VRRP_VERSION_3 && vrrp->ms_down_timer >= 3 * vrrp->master_adver_int))
+			vrrp->stats->master_reason = VRRPV3_MASTER_REASON_MASTER_NO_RESPONSE;
+#endif
 		/* handle master state transition */
 		vrrp->wantstate = VRRP_STATE_MAST;
 		vrrp_state_goto_master(vrrp);
@@ -721,7 +754,7 @@ vrrp_goto_master(vrrp_t * vrrp)
 }
 
 /* Delayed gratuitous ARP thread */
-int
+static int
 vrrp_gratuitous_arp_thread(thread_t * thread)
 {
 	vrrp_t *vrrp = THREAD_ARG(thread);
@@ -732,6 +765,32 @@ vrrp_gratuitous_arp_thread(thread_t * thread)
 	return 0;
 }
 
+/* Delayed gratuitous ARP thread after receiving a lower priority advert */
+int
+vrrp_lower_prio_gratuitous_arp_thread(thread_t * thread)
+{
+	vrrp_t *vrrp = THREAD_ARG(thread);
+
+	/* Simply broadcast the gratuitous ARP */
+	vrrp_send_link_update(vrrp, vrrp->garp_lower_prio_rep);
+
+	return 0;
+}
+
+/* Set effective priorty, issue message on changes */
+void
+vrrp_set_effective_priority(vrrp_t *vrrp, int new_prio)
+{
+	if (vrrp->effective_priority == new_prio)
+		return;
+
+	log_message(LOG_INFO, "VRRP_Instance(%s) Effective priority = %d",
+		    vrrp->iname, new_prio);
+
+	vrrp->effective_priority = new_prio;
+}
+
+
 /* Update VRRP effective priority based on multiple checkers.
  * This is a thread which is executed every adver_int.
  */
@@ -754,16 +813,16 @@ vrrp_update_priority(thread_t * thread)
 
 	if (vrrp->base_priority == VRRP_PRIO_OWNER) {
 		/* we will not run a PRIO_OWNER into a non-PRIO_OWNER */
-		vrrp->effective_priority = VRRP_PRIO_OWNER;
+		vrrp_set_effective_priority(vrrp, VRRP_PRIO_OWNER);
 	} else {
 		/* WARNING! we must compute new_prio on a signed int in order
 		   to detect overflows and avoid wrapping. */
 		new_prio = vrrp->base_priority + prio_offset;
 		if (new_prio < 1)
 			new_prio = 1;
-		else if (new_prio > 254)
-			new_prio = 254;
-		vrrp->effective_priority = new_prio;
+		else if (new_prio >= VRRP_PRIO_OWNER)
+			new_prio = VRRP_PRIO_OWNER - 1;
+		vrrp_set_effective_priority(vrrp, new_prio);
 	}
 
 	/* Register next priority update thread */
@@ -802,13 +861,12 @@ vrrp_master(vrrp_t * vrrp)
 		 * Send the VRRP advert.
 		 * If we catch the master transition
 		 * <=> vrrp_state_master_tx(...) = 1
-		 * register a gratuitous arp thread delayed to 5 secs.
+		 * register a gratuitous arp thread delayed to garp_delay secs.
 		 */
 		if (vrrp_state_master_tx(vrrp, 0)) {
-			thread_add_timer(master, vrrp_gratuitous_arp_thread,
-					 vrrp,
-					 (vrrp->garp_delay) ?
-						vrrp->garp_delay : VRRP_GARP_DELAY);
+			if (vrrp->garp_delay)
+				thread_add_timer(master, vrrp_gratuitous_arp_thread,
+						 vrrp, vrrp->garp_delay);
 			vrrp_smtp_notifier(vrrp);
 		}
 	}
@@ -831,6 +889,7 @@ vrrp_fault(vrrp_t * vrrp)
 	if (new_vrrp_socket(vrrp) < 0)
 		return;
 
+#if defined _WITH_VRRP_AUTH_
 	/*
 	 * We force the IPSEC AH seq_number sync
 	 * to be done in read advert handler.
@@ -840,24 +899,36 @@ vrrp_fault(vrrp_t * vrrp)
 	 */
 	if (vrrp->version == VRRP_VERSION_2 && vrrp->auth_type == VRRP_AUTH_AH) {
 		vrrp_ah_sync(vrrp);
-	} else {
+	} else
+#endif
+	{
 		/* Otherwise, we transit to init state */
 		if (vrrp->init_state == VRRP_STATE_BACK) {
 			vrrp->state = VRRP_STATE_BACK;
 			notify_instance_exec(vrrp, VRRP_STATE_BACK);
-#ifdef _WITH_SNMP_
+			if (vrrp->preempt_delay)
+				vrrp->preempt_time = timer_add_long(timer_now(), vrrp->preempt_delay);
+#ifdef _WITH_SNMP_KEEPALIVED_
 			vrrp_snmp_instance_trap(vrrp);
 #endif
 			vrrp->last_transition = timer_now();
+			log_message(LOG_INFO, "VRRP_Instance(%s): Entering BACKUP STATE", vrrp->iname);
 		} else {
+#ifdef _WITH_SNMP_RFCV3_
+			vrrp->stats->master_reason = VRRPV3_MASTER_REASON_PREEMPTED;
+#endif
+			log_message(LOG_INFO, "VRRP_Instance(%s): Transition to MASTER STATE", vrrp->iname);
 			vrrp_goto_master(vrrp);
 		}
 	}
+#ifdef _WITH_SNMP_RFC_
+	vrrp->stats->uptime = timer_now();
+#endif
 }
 
 /* Handle dispatcher read timeout */
 static int
-vrrp_dispatcher_read_to(int fd)
+vrrp_dispatcher_read_timeout(int fd)
 {
 	vrrp_t *vrrp;
 	int vrid = 0;
@@ -893,7 +964,7 @@ vrrp_dispatcher_read_to(int fd)
 	if (vrrp->quick_sync) {
 		vrrp->sands = timer_add_long(time_now, vrrp->adver_int);
 		vrrp->quick_sync = 0;
-        }
+	}
 
 	return vrrp->fd_in;
 }
@@ -905,14 +976,14 @@ vrrp_dispatcher_read(sock_t * sock)
 	vrrp_t *vrrp;
 	vrrphdr_t *hd;
 	int len = 0, prev_state = 0, proto = 0;
-        struct sockaddr_storage src_addr;
-        socklen_t src_addr_len = sizeof(src_addr);
+	struct sockaddr_storage src_addr;
+	socklen_t src_addr_len = sizeof(src_addr);
 
 	/* Clean the read buffer */
-	memset(vrrp_buffer, 0, VRRP_PACKET_TEMP_LEN);
+	memset(vrrp_buffer, 0, vrrp_buffer_len);
 
 	/* read & affect received buffer */
-	len = recvfrom(sock->fd_in, vrrp_buffer, VRRP_PACKET_TEMP_LEN, 0,
+	len = recvfrom(sock->fd_in, vrrp_buffer, vrrp_buffer_len, 0,
 		       (struct sockaddr *) &src_addr, &src_addr_len);
 	hd = vrrp_get_header(sock->family, vrrp_buffer, &proto);
 
@@ -947,7 +1018,7 @@ vrrp_dispatcher_read(sock_t * sock)
 }
 
 /* Our read packet dispatcher */
-int
+static int
 vrrp_read_dispatcher_thread(thread_t * thread)
 {
 	long vrrp_timer = 0;
@@ -959,7 +1030,7 @@ vrrp_read_dispatcher_thread(thread_t * thread)
 
 	/* Dispatcher state handler */
 	if (thread->type == THREAD_READ_TIMEOUT || sock->fd_in == -1)
-		fd = vrrp_dispatcher_read_to(sock->fd_in);
+		fd = vrrp_dispatcher_read_timeout(sock->fd_in);
 	else
 		fd = vrrp_dispatcher_read(sock);
 
@@ -980,52 +1051,15 @@ static int
 vrrp_script_thread(thread_t * thread)
 {
 	vrrp_script_t *vscript = THREAD_ARG(thread);
-	int status, ret;
-	pid_t pid;
 
 	/* Register next timer tracker */
 	thread_add_timer(thread->master, vrrp_script_thread, vscript,
 			 vscript->interval);
 
-	/* Daemonization to not degrade our scheduling timer */
-	pid = fork();
-
-	/* In case of fork is error. */
-	if (pid < 0) {
-		log_message(LOG_INFO, "Failed fork process");
-		return -1;
-	}
-
-	/* In case of this is parent process */
-	if (pid) {
-		thread_add_child(thread->master, vrrp_script_child_thread,
-				 vscript, pid,
-				 (vscript->timeout) ? vscript->timeout : vscript->interval);
-		return 0;
-	}
-
-	/* Child part */
-	signal_handler_destroy();
-	closeall(0);
-	open("/dev/null", O_RDWR);
-	ret = dup(0);
-	if (ret < 0) {
-		log_message(LOG_INFO, "dup(0) error");
-	}
-
-	ret = dup(0);
-	if (ret < 0) {
-		log_message(LOG_INFO, "dup(0) error");
-	}
-
-	status = system_call(vscript->script);
-
-	if (status < 0 || !WIFEXITED(status))
-		status = 0; /* Script errors aren't server errors */
-	else
-		status = WEXITSTATUS(status);
-
-	exit(status);
+	/* Execute the script in a child process. Parent returns, child doesn't */
+	return system_call_script(thread->master, vrrp_script_child_thread,
+				  vscript, (vscript->timeout) ? vscript->timeout : vscript->interval,
+				  vscript->script);
 }
 
 static int
@@ -1100,7 +1134,99 @@ vrrp_script_child_timeout_thread(thread_t * thread)
 	}
 
 	log_message(LOG_WARNING, "Process [%d] didn't respond to SIGTERM", pid);
-	waitpid(pid, NULL, 0);
+
+	return 0;
+}
+
+/* Delayed ARP/NA thread */
+int
+vrrp_arp_thread(thread_t *thread)
+{
+	element e, a;
+	list l;
+	ip_address_t *ipaddress;
+	timeval_t next_time = {
+		.tv_sec = INT_MAX	/* We're never going to delay this long - I hope! */
+	};
+	interface_t *ifp;
+	vrrp_t *vrrp;
+	enum {
+		VIP,
+		EVIP
+	} i;
+
+	set_time_now();
+
+	for (e = LIST_HEAD(vrrp_data->vrrp); e; ELEMENT_NEXT(e)) {
+		vrrp = ELEMENT_DATA(e);
+
+		if (!vrrp->garp_pending && !vrrp->gna_pending)
+			continue;
+
+		vrrp->garp_pending = false;
+		vrrp->gna_pending = false;
+
+		if (vrrp->state != VRRP_STATE_MAST ||
+		    !vrrp->vipset)
+			continue;
+
+		for (i = VIP; i <= EVIP; i++) {
+			l = (i == VIP) ? vrrp->vip : vrrp->evip;
+
+			if (!LIST_ISEMPTY(l)) {
+				for (a = LIST_HEAD(l); a; ELEMENT_NEXT(a)) {
+					ipaddress = ELEMENT_DATA(a);
+					if (!ipaddress->garp_gna_pending)
+						continue;
+					if (!ipaddress->set) {
+						ipaddress->garp_gna_pending = false;
+						continue;
+					}
+
+					ifp = IF_BASE_IFP(ipaddress->ifp);
+
+					/* This should never happen */
+					if (!ifp->garp_delay) {
+						ipaddress->garp_gna_pending = false;
+						continue;
+					}
+
+					if (!IP_IS6(ipaddress)) {
+						if (timer_cmp(time_now, ifp->garp_delay->garp_next_time) >= 0) {
+							send_gratuitous_arp_immediate(ifp, ipaddress);
+							ipaddress->garp_gna_pending = false;
+						}
+						else {
+							vrrp->garp_pending = true;
+							if (timer_cmp(ifp->garp_delay->garp_next_time, next_time) < 0)
+								next_time = ifp->garp_delay->garp_next_time;
+						}
+					}
+					else {
+						if (timer_cmp(time_now, ifp->garp_delay->gna_next_time) >= 0) {
+							ndisc_send_unsolicited_na_immediate(ifp, ipaddress);
+							ipaddress->garp_gna_pending = false;
+						}
+						else {
+							vrrp->gna_pending = true;
+							if (timer_cmp(ifp->garp_delay->gna_next_time, next_time) < 0)
+								next_time = ifp->garp_delay->gna_next_time;
+						}
+					}
+				}
+			}
+		}
+	}
+
+	if (next_time.tv_sec != INT_MAX) {
+		/* Register next timer tracker */
+		garp_next_time = next_time;
+
+		garp_thread = thread_add_timer(thread->master, vrrp_arp_thread, NULL,
+						 -timer_long(timer_sub_now(next_time)));
+	}
+	else
+		garp_thread = NULL;
 
 	return 0;
 }
