@@ -22,24 +22,46 @@
  * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
  */
 
+#ifdef _MEM_CHECK_
+#include <assert.h>
+#include <unistd.h>
+#include <fcntl.h>
+#include <stdio.h>
+#endif
+
+#include <errno.h>
+#include <string.h>
+
 #include "memory.h"
 #include "utils.h"
 #include "bitops.h"
+#include "logger.h"
 
+#ifdef _MEM_CHECK_
 /* Global var */
-unsigned long mem_allocated;	/* Total memory used in Bytes */
+size_t mem_allocated;		/* Total memory used in Bytes */
+size_t max_mem_allocated;	/* Maximum memory used in Bytes */
+#endif
 
-void *
+static void *
 xalloc(unsigned long size)
 {
 	void *mem = malloc(size);
 
 	if (mem == NULL) {
-		perror("Keepalived");
+		if (__test_bit(DONT_FORK_BIT, &debug))
+			perror("Keepalived");
+		else
+			log_message(LOG_INFO, "Keepalived xalloc() error - %s", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
-	mem_allocated += size;
+#ifdef _MEM_CHECK_
+	mem_allocated += size - sizeof(long);
+	if (mem_allocated > max_mem_allocated)
+		max_mem_allocated = mem_allocated;
+#endif
+
 	return mem;
 }
 
@@ -73,7 +95,7 @@ zalloc(unsigned long size)
  *
  */
 
-#ifdef _DEBUG_
+#ifdef _MEM_CHECK_
 
 typedef struct {
 	int type;
@@ -93,7 +115,9 @@ static int number_alloc_list = 0;
 static int n = 0;		/* Alloc list pointer */
 static int f = 0;		/* Free list pointer */
 
-char *
+static FILE *log_op = NULL;
+
+void *
 keepalived_malloc(unsigned long size, char *file, char *function, int line)
 {
 	void *buf;
@@ -124,10 +148,13 @@ keepalived_malloc(unsigned long size, char *file, char *function, int line)
 	alloc_list[i].csum = check;
 	alloc_list[i].type = 9;
 
-	if (debug & 1)
-		printf("zalloc[%3d:%3d], %p, %4ld at %s, %3d, %s\n",
-		       i, number_alloc_list, buf, size, file, line,
-		       function);
+	fprintf(log_op, "zalloc[%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+	       i, number_alloc_list, buf, size, file, line, function);
+#ifdef _MEM_CHECK_LOG_
+	if (__test_bit(MEM_CHECK_LOG_BIT, &debug))
+		log_message(LOG_INFO, "zalloc[%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+		       i, number_alloc_list, buf, size, file, line, function);
+#endif
 
 	n++;
 	return buf;
@@ -151,9 +178,8 @@ keepalived_free(void *buffer, char *file, char *function, int line)
 		alloc_list[i].func = function;
 		alloc_list[i].line = line;
 		alloc_list[i].type = 2;
-		if (__test_bit(LOG_CONSOLE_BIT, &debug))
-			printf("free NULL in %s, %3d, %s\n", file,
-			       line, function);
+		fprintf(log_op, "free NULL in %s, %3d, %s\n", file,
+		       line, function);
 
 		__set_bit(MEM_ERR_DETECT_BIT, &debug);	/* Memory Error detect */
 
@@ -164,22 +190,20 @@ keepalived_free(void *buffer, char *file, char *function, int line)
 		if (alloc_list[i].type == 9 && alloc_list[i].ptr == buf) {
 			if (*((long *) ((char *) alloc_list[i].ptr + alloc_list[i].size)) == alloc_list[i].csum) {
 				alloc_list[i].type = 0;	/* Release */
-				mem_allocated -= alloc_list[i].size;
+				mem_allocated -= alloc_list[i].size - sizeof(long);
 			} else {
 				alloc_list[i].type = 1;	/* Overrun */
-				if (__test_bit(LOG_CONSOLE_BIT, &debug)) {
-					printf("free corrupt, buffer overrun [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
-					       i, number_alloc_list,
-					       buf, alloc_list[i].size, file,
-					       line, function);
-					dump_buffer(alloc_list[i].ptr,
-						    alloc_list[i].size + sizeof (long));
-					printf("Check_sum\n");
-					dump_buffer((char *) &alloc_list[i].csum,
-						    sizeof(long));
+				fprintf(log_op, "free corrupt, buffer overrun [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+				       i, number_alloc_list,
+				       buf, alloc_list[i].size, file,
+				       line, function);
+				dump_buffer(alloc_list[i].ptr,
+					    alloc_list[i].size + sizeof (long), log_op);
+				fprintf(log_op, "Check_sum\n");
+				dump_buffer((char *) &alloc_list[i].csum,
+					    sizeof(long), log_op);
 
-					__set_bit(MEM_ERR_DETECT_BIT, &debug);
-				}
+				__set_bit(MEM_ERR_DETECT_BIT, &debug);
 			}
 			break;
 		}
@@ -188,7 +212,7 @@ keepalived_free(void *buffer, char *file, char *function, int line)
 
 	/*  Not found */
 	if (i == number_alloc_list) {
-		printf("Free ERROR %p\n", buffer);
+		fprintf(log_op, "Free ERROR %p not found\n", buffer);
 		number_alloc_list++;
 
 		assert(number_alloc_list < MAX_ALLOC_LIST);
@@ -207,10 +231,15 @@ keepalived_free(void *buffer, char *file, char *function, int line)
 	if (buffer != NULL)
 		free(buffer);
 
-	if (__test_bit(LOG_CONSOLE_BIT, &debug))
-		printf("free  [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+	fprintf(log_op, "free  [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
+	       i, number_alloc_list, buf,
+	       alloc_list[i].size, file, line, function);
+#ifdef _MEM_CHECK_LOG_
+	if (__test_bit(MEM_CHECK_LOG_BIT, &debug))
+		log_message(LOG_INFO, "free  [%3d:%3d], %p, %4ld at %s, %3d, %s\n",
 		       i, number_alloc_list, buf,
 		       alloc_list[i].size, file, line, function);
+#endif
 
 	free_list[f].file = file;
 	free_list[f].line = line;
@@ -233,29 +262,29 @@ keepalived_free_final(char *banner)
 	int i, j;
 	i = 0;
 
-	printf("\n---[ Keepalived memory dump for (%s)]---\n\n", banner);
+	fprintf(log_op, "\n---[ Keepalived memory dump for (%s)]---\n\n", banner);
 
 	while (i < number_alloc_list) {
 		switch (alloc_list[i].type) {
 		case 3:
 			badptr++;
-			printf
-			    ("null pointer to realloc(nil,%ld)! at %s, %3d, %s\n",
+			fprintf
+			    (log_op, "null pointer to realloc(nil,%ld)! at %s, %3d, %s\n",
 			     alloc_list[i].size, alloc_list[i].file,
 			     alloc_list[i].line, alloc_list[i].func);
 			break;
 		case 4:
 			badptr++;
-			printf
-			    ("pointer not found in table to free(%p) [%3d:%3d], at %s, %3d, %s\n",
+			fprintf
+			    (log_op, "pointer not found in table to free(%p) [%3d:%3d], at %s, %3d, %s\n",
 			     alloc_list[i].ptr, i, number_alloc_list,
 			     alloc_list[i].file, alloc_list[i].line,
 			     alloc_list[i].func);
 			for (j = 0; j < 256; j++)
 				if (free_list[j].ptr == alloc_list[i].ptr)
 					if (free_list[j].type == 8)
-						printf
-						    ("  -> pointer allready released at [%3d:%3d], at %s, %3d, %s\n",
+						fprintf
+						    (log_op, "  -> pointer allready released at [%3d:%3d], at %s, %3d, %s\n",
 						     (int) free_list[j].csum,
 						     number_alloc_list,
 						     free_list[j].file,
@@ -264,25 +293,25 @@ keepalived_free_final(char *banner)
 			break;
 		case 2:
 			badptr++;
-			printf("null pointer to free(nil)! at %s, %3d, %s\n",
+			fprintf(log_op, "null pointer to free(nil)! at %s, %3d, %s\n",
 			       alloc_list[i].file, alloc_list[i].line,
 			       alloc_list[i].func);
 			break;
 		case 1:
 			overrun++;
-			printf("%p [%3d:%3d], %4ld buffer overrun!:\n",
+			fprintf(log_op, "%p [%3d:%3d], %4ld buffer overrun!:\n",
 			       alloc_list[i].ptr, i, number_alloc_list,
 			       alloc_list[i].size);
-			printf(" --> source of malloc: %s, %3d, %s\n",
+			fprintf(log_op, " --> source of malloc: %s, %3d, %s\n",
 			       alloc_list[i].file, alloc_list[i].line,
 			       alloc_list[i].func);
 			break;
 		case 9:
 			sum += alloc_list[i].size;
-			printf("%p [%3d:%3d], %4ld not released!:\n",
+			fprintf(log_op, "%p [%3d:%3d], %4ld not released!:\n",
 			       alloc_list[i].ptr, i, number_alloc_list,
 			       alloc_list[i].size);
-			printf(" --> source of malloc: %s, %3d, %s\n",
+			fprintf(log_op, " --> source of malloc: %s, %3d, %s\n",
 			       alloc_list[i].file, alloc_list[i].line,
 			       alloc_list[i].func);
 			break;
@@ -290,17 +319,18 @@ keepalived_free_final(char *banner)
 		i++;
 	}
 
-	printf("\n\n---[ Keepalived memory dump summary for (%s) ]---\n", banner);
-	printf("Total number of bytes not freed...: %d\n", sum);
-	printf("Number of entries not freed.......: %d\n", n);
-	printf("Maximum allocated entries.........: %d\n", number_alloc_list);
-	printf("Number of bad entries.............: %d\n", badptr);
-	printf("Number of buffer overrun..........: %d\n\n", overrun);
+	fprintf(log_op, "\n\n---[ Keepalived memory dump summary for (%s) ]---\n", banner);
+	fprintf(log_op, "Total number of bytes not freed...: %d\n", sum);
+	fprintf(log_op, "Number of entries not freed.......: %d\n", n);
+	fprintf(log_op, "Maximum allocated entries.........: %d\n", number_alloc_list);
+	fprintf(log_op, "Maximum memory allocated..........: %zu\n", max_mem_allocated);
+	fprintf(log_op, "Number of bad entries.............: %d\n", badptr);
+	fprintf(log_op, "Number of buffer overrun..........: %d\n\n", overrun);
 
 	if (sum || n || badptr || overrun)
-		printf("=> Program seems to have some memory problem !!!\n\n");
+		fprintf(log_op, "=> Program seems to have some memory problem !!!\n\n");
 	else
-		printf("=> Program seems to be memory allocation safe...\n\n");
+		fprintf(log_op, "=> Program seems to be memory allocation safe...\n\n");
 }
 
 void *
@@ -312,7 +342,7 @@ keepalived_realloc(void *buffer, unsigned long size, char *file, char *function,
 	long check;
 
 	if (buffer == NULL) {
-		printf("realloc %p %s, %3d %s\n", buffer, file, line, function);
+		fprintf(log_op, "realloc %p %s, %3d %s\n", buffer, file, line, function);
 		i = number_alloc_list++;
 
 		assert(number_alloc_list < MAX_ALLOC_LIST);
@@ -336,7 +366,7 @@ keepalived_realloc(void *buffer, unsigned long size, char *file, char *function,
 
 	/* not found */
 	if (i == number_alloc_list) {
-		printf("realloc ERROR no matching zalloc %p \n", buffer);
+		fprintf(log_op, "realloc ERROR no matching zalloc %p \n", buffer);
 		number_alloc_list++;
 
 		assert(number_alloc_list < MAX_ALLOC_LIST);
@@ -363,12 +393,11 @@ keepalived_realloc(void *buffer, unsigned long size, char *file, char *function,
 	*(long *) ((char *) buf + size) = check;
 	alloc_list[i].csum = check;
 
-	if (__test_bit(LOG_CONSOLE_BIT, &debug))
-		printf("realloc [%3d:%3d] %p, %4ld %s %d %s -> %p %4ld %s %d %s\n",
-		       i, number_alloc_list, alloc_list[i].ptr,
-		       alloc_list[i].size, file, line, function, buf, size,
-		       alloc_list[i].file, alloc_list[i].line,
-		       alloc_list[i].func);
+	fprintf(log_op, "realloc [%3d:%3d] %p, %4ld %s %d %s -> %p %4ld %s %d %s\n",
+	       i, number_alloc_list, alloc_list[i].ptr,
+	       alloc_list[i].size, file, line, function, buf, size,
+	       alloc_list[i].file, alloc_list[i].line,
+	       alloc_list[i].func);
 
 	alloc_list[i].ptr = buf;
 	alloc_list[i].size = size;
@@ -379,4 +408,48 @@ keepalived_realloc(void *buffer, unsigned long size, char *file, char *function,
 	return buf;
 }
 
+void
+mem_log_init(const char* prog_name)
+{
+	size_t log_name_len;
+	char *log_name;
+
+	if (__test_bit(LOG_CONSOLE_BIT, &debug)) {
+		log_op = stderr;
+		return;
+	}
+
+	if (log_op)
+		fclose(log_op);
+
+	log_name_len = 5 + strlen(prog_name) + 5 + 5 + 4 + 1;	/* "/tmp/" + prog_name + "_mem." + PID + ".log" + '\0" */
+	log_name = malloc(log_name_len);
+	if (!log_name) {
+		log_message(LOG_INFO, "Unable to malloc log file name");
+		log_op = stderr;
+		return;
+	}
+
+	snprintf(log_name, log_name_len, "/tmp/%s_mem.%d.log", prog_name, getpid());
+	log_op = fopen(log_name, "a");
+	if (log_op == NULL) {
+		log_message(LOG_INFO, "Unable to open %s for appending", log_name);
+		log_op = stderr;
+	}
+	else {
+		int fd = fileno(log_op);
+
+		/* We don't want any children to inherit the log file */
+		fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
+
+		/* Make the log output line buffered. This was to ensure that
+		 * children didn't inherit the buffer, but the CLOEXEC above
+		 * should resolve that. */
+		setlinebuf(log_op);
+
+		fprintf(log_op, "\n");
+	}
+
+	free(log_name);
+}
 #endif
