@@ -1,14 +1,14 @@
-/* 
+/*
  * Soft:        Keepalived is a failover program for the LVS project
  *              <www.linuxvirtualserver.org>. It monitor & manipulate
  *              a loadbalanced server pool using multi-layer checks.
- * 
+ *
  * Part:        Configuration file parser/reader. Place into the dynamic
  *              data structure representation the conf file representing
  *              the loadbalanced server pool.
- *  
+ *
  * Author:      Alexandre Cassen, <acassen@linux-vs.org>
- *              
+ *
  *              This program is distributed in the hope that it will be useful,
  *              but WITHOUT ANY WARRANTY; without even the implied warranty of
  *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,22 +26,32 @@
 #include <unistd.h>
 #include <libgen.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <stdbool.h>
 #include "parser.h"
 #include "memory.h"
 #include "logger.h"
+#include "rttables.h"
+
+#define DUMP_KEYWORDS	0
 
 /* global vars */
 vector_t *keywords;
-vector_t *current_keywords;
-FILE *current_stream;
-char *current_conf_file;
 int reload = 0;
 
 /* local vars */
+static char *current_conf_file;
+static vector_t *current_keywords;
+static FILE *current_stream;
 static int sublevel = 0;
+static int skip_sublevel = 0;
 
-void
-keyword_alloc(vector_t *keywords_vec, char *string, void (*handler) (vector_t *))
+/* Forward references */
+static void process_stream(vector_t *, int);
+
+static void
+keyword_alloc(vector_t *keywords_vec, const char *string, void (*handler) (vector_t *), bool active)
 {
 	keyword_t *keyword;
 
@@ -49,13 +59,14 @@ keyword_alloc(vector_t *keywords_vec, char *string, void (*handler) (vector_t *)
 
 	keyword = (keyword_t *) MALLOC(sizeof(keyword_t));
 	keyword->string = string;
-	keyword->handler = handler;
+	keyword->handler = (active) ? handler : NULL;
+	keyword->active = active;
 
 	vector_set_slot(keywords_vec, keyword);
 }
 
-void
-keyword_alloc_sub(vector_t *keywords_vec, char *string, void (*handler) (vector_t *))
+static void
+keyword_alloc_sub(vector_t *keywords_vec, const char *string, void (*handler) (vector_t *))
 {
 	int i = 0;
 	keyword_t *keyword;
@@ -73,7 +84,7 @@ keyword_alloc_sub(vector_t *keywords_vec, char *string, void (*handler) (vector_
 		keyword->sub = vector_alloc();
 
 	/* add new sub keyword */
-	keyword_alloc(keyword->sub, string, handler);
+	keyword_alloc(keyword->sub, string, handler, true);
 }
 
 /* Exported helpers */
@@ -90,13 +101,13 @@ install_sublevel_end(void)
 }
 
 void
-install_keyword_root(char *string, void (*handler) (vector_t *))
+install_keyword_root(const char *string, void (*handler) (vector_t *), bool active)
 {
-	keyword_alloc(keywords, string, handler);
+	keyword_alloc(keywords, string, handler, active);
 }
 
 void
-install_keyword(char *string, void (*handler) (vector_t *))
+install_keyword(const char *string, void (*handler) (vector_t *))
 {
 	keyword_alloc_sub(keywords, string, handler);
 }
@@ -117,27 +128,41 @@ install_sublevel_end_handler(void (*handler) (void))
 	keyword->sub_close_handler = handler;
 }
 
-void
-dump_keywords(vector_t *keydump, int level)
+#if DUMP_KEYWORDS
+static void
+dump_keywords(vector_t *keydump, int level, FILE *fp)
 {
-	int i, j;
+	unsigned int i;
 	keyword_t *keyword_vec;
+	char file_name[21];
+
+	if (!level) {
+		sprintf(file_name, "/tmp/keywords.%d", getpid());
+		snprintf(file_name, sizeof(file_name), "/tmp/keywords.%d", getpid());
+		fp = fopen(file_name, "w");
+		if (!fp)
+			return;
+	}
+
+	vector_dump(fp, keywords);
 
 	for (i = 0; i < vector_size(keydump); i++) {
 		keyword_vec = vector_slot(keydump, i);
-		for (j = 0; j < level; j++)
-			printf("  ");
-		printf("Keyword : %s\n", keyword_vec->string);
+		fprintf(fp, "%*sKeyword : %s (%s)\n", level * 2, "", keyword_vec->string, keyword_vec->active ? "active": "disabled");
 		if (keyword_vec->sub)
-			dump_keywords(keyword_vec->sub, level + 1);
+			dump_keywords(keyword_vec->sub, level + 1, fp);
 	}
-}
 
-void
+	if (!level)
+		fclose(fp);
+}
+#endif
+
+static void
 free_keywords(vector_t *keywords_vec)
 {
 	keyword_t *keyword_vec;
-	int i;
+	unsigned int i;
 
 	for (i = 0; i < vector_size(keywords_vec); i++) {
 		keyword_vec = vector_slot(keywords_vec, i);
@@ -177,19 +202,25 @@ alloc_strvec(char *string)
 
 	while (1) {
 		start = cp;
+
+		/* Save a quoted string without the "s as a single string */
 		if (*cp == '"') {
+			start++;
+			if (!(cp = strchr(start, '"'))) {
+				log_message(LOG_INFO, "Unmatched quote: '%s'", string);
+				return strvec;
+			}
+			str_len = cp - start;
 			cp++;
-			token = MALLOC(2);
-			*(token) = '"';
-			*(token + 1) = '\0';
 		} else {
-			while (!isspace((int) *cp) && *cp != '\0' && *cp != '"')
+			while (!isspace((int) *cp) && *cp != '\0' && *cp != '"'
+						   && *cp != '!' && *cp != '#')
 				cp++;
 			str_len = cp - start;
-			token = MALLOC(str_len + 1);
-			memcpy(token, start, str_len);
-			*(token + str_len) = '\0';
 		}
+		token = MALLOC(str_len + 1);
+		memcpy(token, start, str_len);
+		token[str_len] = '\0';
 
 		/* Alloc & set the slot */
 		vector_alloc_slot(strvec);
@@ -202,29 +233,45 @@ alloc_strvec(char *string)
 	}
 }
 
-void read_conf_file(char *conf_file)
+static void
+read_conf_file(const char *conf_file)
 {
 	FILE *stream;
 	char *path;
 	int ret;
-
 	glob_t globbuf;
+	int	res;
+	struct stat stb;
 
 	globbuf.gl_offs = 0;
-	glob(conf_file, 0, NULL, &globbuf);
+	res = glob(conf_file, 0, NULL, &globbuf);
+
+	if (res) {
+		log_message(LOG_INFO, "Unable to find config file(s) '%s'.", conf_file);
+		return;
+	}
 
 	int i;
 	for(i = 0; i < globbuf.gl_pathc; i++){
 		log_message(LOG_INFO, "Opening file '%s'.", globbuf.gl_pathv[i]);
 		stream = fopen(globbuf.gl_pathv[i], "r");
 		if (!stream) {
-			log_message(LOG_INFO, "Configuration file '%s' open problem (%s)..."
+			log_message(LOG_INFO, "Configuration file '%s' open problem (%s) - skipping"
 				       , globbuf.gl_pathv[i], strerror(errno));
-			return;
+			continue;
 		}
+
+		/* Make sure what we have opened is a regular file, and not for example a directory */
+		if (fstat(fileno(stream), &stb) ||
+		    !S_ISREG(stb.st_mode)) {
+			log_message(LOG_INFO, "Configuration file '%s' is not a regular file - skipping", globbuf.gl_pathv[i]);
+			fclose(stream);
+			continue;
+		}
+
 		current_stream = stream;
 		current_conf_file = globbuf.gl_pathv[i];
-		
+
 		char prev_path[MAXBUF];
 		path = getcwd(prev_path, MAXBUF);
 		if (!path) {
@@ -240,7 +287,7 @@ void read_conf_file(char *conf_file)
 					    , confpath, strerror(errno));
 		}
 		free(confpath);
-		process_stream(current_keywords);
+		process_stream(current_keywords, 0);
 		fclose(stream);
 
 		ret = chdir(prev_path);
@@ -253,7 +300,48 @@ void read_conf_file(char *conf_file)
 	globfree(&globbuf);
 }
 
-int
+bool check_conf_file(const char *conf_file)
+{
+	glob_t globbuf;
+	int i;
+	bool ret = true;
+	int res;
+	struct stat stb;
+
+	globbuf.gl_offs = 0;
+	res = glob(conf_file, 0, NULL, &globbuf);
+	if (res) {
+		log_message(LOG_INFO, "Unable to find configuration file %s (glob returned %d)", conf_file, res);
+		return false;
+	}
+
+	if (globbuf.gl_pathc == 0) {
+		log_message(LOG_INFO, "Unable to find configuration file %s", conf_file);
+		ret = false;
+	} else {
+		for (i = 0; i < globbuf.gl_pathc; i++) {
+			if (access(globbuf.gl_pathv[i], R_OK)) {
+				log_message(LOG_INFO, "Unable to read configuration file %s", globbuf.gl_pathv[i]);
+				ret = false;
+				break;
+			}
+
+			/* Make sure that the file is a regular file, and not for example a directory */
+			if (stat(globbuf.gl_pathv[i], &stb) ||
+			    !S_ISREG(stb.st_mode)) {
+				log_message(LOG_INFO, "Configuration file '%s' is not a regular file", globbuf.gl_pathv[i]);
+				ret = false;
+				break;
+			}
+		}
+	}
+
+	globfree(&globbuf);
+
+	return ret;
+}
+
+static int
 check_include(char *buf)
 {
 	char *str;
@@ -267,7 +355,7 @@ check_include(char *buf)
 		return 0;
 	}
 	str = vector_slot(strvec, 0);
-	
+
 	if (!strcmp(str, EOB)) {
 		free_strvec(strvec);
 		return 0;
@@ -281,7 +369,7 @@ check_include(char *buf)
 		char prev_path[MAXBUF];
 		path = getcwd(prev_path, MAXBUF);
 		if (!path) {
-			log_message(LOG_INFO, "getcwd(%s) error (%s)\n"
+			log_message(LOG_INFO, "getcwd(%s) error (%s)"
 					    , prev_path, strerror(errno));
 		}
 
@@ -290,7 +378,7 @@ check_include(char *buf)
 		current_conf_file = prev_conf_file;
 		ret = chdir(prev_path);
 		if (ret < 0) {
-			log_message(LOG_INFO, "chdir(%s) error (%s)\n"
+			log_message(LOG_INFO, "chdir(%s) error (%s)"
 					    , prev_path, strerror(errno));
 		}
 		free_strvec(strvec);
@@ -303,54 +391,79 @@ check_include(char *buf)
 int
 read_line(char *buf, int size)
 {
-	int ch;
+	size_t len ;
+	bool eof = false;
 
 	do {
-		int count = 0;
-		memset(buf, 0, size);
-		while ((ch = fgetc(current_stream)) != EOF && (int) ch != '\n'
-			   && (int) ch != '\r') {
-			if (count < size)
-				buf[count] = (int) ch;
-			else
-				break;
-			count++;
+		if (fgets(buf, size, current_stream)) {
+			for (len = strlen(buf) - 1; len >= 0; len--) {
+				if (buf[len] != '\r' && buf[len] != '\n')
+					break;
+				buf[len] = '\0';
+			}
+		}
+		else
+		{
+			eof = true;
+			buf[0] = '\0';
 		}
 	} while (check_include(buf) == 1);
-	return (ch == EOF) ? 0 : 1;
+	return (eof) ? 0 : 1;
 }
 
 vector_t *
-read_value_block(void)
+read_value_block(vector_t *strvec)
 {
 	char *buf;
-	int i;
+	unsigned int word;
 	char *str = NULL;
 	char *dup;
 	vector_t *vec = NULL;
 	vector_t *elements = vector_alloc();
+	int first = 1;
+	int need_bob = 1;
+	int got_eob = 0;
 
 	buf = (char *) MALLOC(MAXBUF);
-	while (read_line(buf, MAXBUF)) {
-		vec = alloc_strvec(buf);
+	while (first || read_line(buf, MAXBUF)) {
+		if (first && vector_size(strvec) > 1) {
+			vec = strvec;
+			word = 1;
+		}
+		else {
+			vec = alloc_strvec(buf);
+			word = 0;
+		}
 		if (vec) {
-			str = vector_slot(vec, 0);
-			if (!strcmp(str, EOB)) {
-				free_strvec(vec);
-				break;
+			str = vector_slot(vec, word);
+			if (need_bob) {
+				if (!strcmp(str, BOB))
+					word++;
+				else
+					log_message(LOG_INFO, "'{' missing at beginning of block %s", FMT_STR_VSLOT(strvec,0));
+				need_bob = 0;
 			}
 
-			if (vector_size(vec))
-				for (i = 0; i < vector_size(vec); i++) {
-					str = vector_slot(vec, i);
-					dup = (char *) MALLOC(strlen(str) + 1);
-					memcpy(dup, str, strlen(str));
-					vector_alloc_slot(elements);
-					vector_set_slot(elements, dup);
+			for (; word < vector_size(vec); word++) {
+				str = vector_slot(vec, word);
+				if (!strcmp(str, EOB)) {
+					if (word != vector_size(vec) - 1)
+						log_message(LOG_INFO, "Extra characters after '}' - \"%s\"", buf);
+					got_eob = 1;
+					break;
 				}
-			free_strvec(vec);
+				dup = (char *) MALLOC(strlen(str) + 1);
+				memcpy(dup, str, strlen(str));
+				vector_alloc_slot(elements);
+				vector_set_slot(elements, dup);
+			}
+			if (vec != strvec)
+				free_strvec(vec);
+			if (got_eob)
+				break;
 		}
 		memset(buf, 0, MAXBUF);
+		first = 0;
 	}
 
 	FREE(buf);
@@ -390,57 +503,89 @@ set_value(vector_t *strvec)
 {
 	char *str = vector_slot(strvec, 1);
 	int size = strlen(str);
-	int i = 0;
-	int len = 0;
-	char *alloc = NULL;
-	char *tmp;
+	char *alloc;
 
-	if (*str == '"') {
-		for (i = 2; i < vector_size(strvec); i++) {
-			str = vector_slot(strvec, i);
-			len += strlen(str);
-			if (!alloc)
-				alloc = (char *) MALLOC(len + 1);
-			else {
-				alloc = (char *) REALLOC(alloc, 2 * (len + 1));
-				tmp = vector_slot(strvec, i-1);
-				if (*str != '"' && *tmp != '"')
-					strncat(alloc, " ", 1);
-			}
+	alloc = (char *) MALLOC(size + 1);
+	if (!alloc)
+		return NULL;
+
+	memcpy(alloc, str, size);
 
-			if (i != vector_size(strvec)-1)
-				strncat(alloc, str, strlen(str));
-		}
-	} else {
-		alloc = (char *) MALLOC(size + 1);
-		memcpy(alloc, str, size);
-	}
 	return alloc;
 }
 
+/* Checks for on/true/yes or off/false/no */
+int
+check_true_false(char *str)
+{
+	if (!strcmp(str, "true") || !strcmp(str, "on") || !strcmp(str, "yes"))
+		return true;
+	if (!strcmp(str, "false") || !strcmp(str, "off") || !strcmp(str, "no"))
+		return false;
+
+	return -1;	/* error */
+}
+
+void skip_block(void)
+{
+	/* Don't process the rest of the configuration block */
+	skip_sublevel = 1;
+}
+
 /* recursive configuration stream handler */
 static int kw_level = 0;
-void
-process_stream(vector_t *keywords_vec)
+static void
+process_stream(vector_t *keywords_vec, int need_bob)
 {
-	int i;
+	unsigned int i;
 	keyword_t *keyword_vec;
 	char *str;
 	char *buf;
 	vector_t *strvec;
 	vector_t *prev_keywords = current_keywords;
 	current_keywords = keywords_vec;
+	int bob_needed = 0;
 
-	buf = zalloc(MAXBUF);
+	buf = MALLOC(MAXBUF);
 	while (read_line(buf, MAXBUF)) {
 		strvec = alloc_strvec(buf);
-		memset(buf,0, MAXBUF);
+		memset(buf, 0, MAXBUF);
 
 		if (!strvec)
 			continue;
 
 		str = vector_slot(strvec, 0);
 
+		if (need_bob) {
+			need_bob = 0;
+			if (!strcmp(str, BOB) && kw_level > 0)
+				str[0] = 0;
+			else
+				log_message(LOG_INFO, "Missing '{' at beginning of configuration block");
+		}
+		else if (!skip_sublevel && !strcmp(str, BOB)) {
+			log_message(LOG_INFO, "Unexpected '{' - ignoring");
+			free_strvec(strvec);
+			continue;
+		}
+
+		if (skip_sublevel) {
+			for (i = 0; i < vector_size(strvec); i++) {
+				str = vector_slot(strvec,i);
+				if (!strcmp(str,BOB))
+					skip_sublevel++;
+				else if (!strcmp(str,EOB)) {
+					if (--skip_sublevel == 0)
+						break;
+				}
+			}
+			free_strvec(strvec);
+
+			if (!skip_sublevel)
+				break;
+			continue;
+		}
+
 		if (!strcmp(str, EOB) && kw_level > 0) {
 			free_strvec(strvec);
 			break;
@@ -450,44 +595,62 @@ process_stream(vector_t *keywords_vec)
 			keyword_vec = vector_slot(keywords_vec, i);
 
 			if (!strcmp(keyword_vec->string, str)) {
-				if (keyword_vec->handler)
+				if (keyword_vec->sub) {
+					/* Remove a trailing '{' */
+					char *bob = vector_slot(strvec, vector_size(strvec)-1) ;
+					if (!strcmp(bob, BOB)) {
+						vector_unset(strvec, vector_size(strvec)-1);
+						FREE(bob);
+						bob_needed = 0;
+					}
+					else
+						bob_needed = 1;
+				}
+
+				if (keyword_vec->active && keyword_vec->handler)
 					(*keyword_vec->handler) (strvec);
 
 				if (keyword_vec->sub) {
+					if (!keyword_vec->active)
+						skip_block();
+
 					kw_level++;
-					process_stream(keyword_vec->sub);
+					process_stream(keyword_vec->sub, bob_needed);
 					kw_level--;
-					if (keyword_vec->sub_close_handler)
+					if (keyword_vec->active && keyword_vec->sub_close_handler)
 						(*keyword_vec->sub_close_handler) ();
 				}
 				break;
 			}
 		}
 
+		if (i >= vector_size(keywords_vec))
+			log_message(LOG_INFO, "Unknown keyword '%s'", str );
+
 		free_strvec(strvec);
 	}
 
 	current_keywords = prev_keywords;
-	free(buf);
+	FREE(buf);
 	return;
 }
 
 /* Data initialization */
 void
-init_data(char *conf_file, vector_t * (*init_keywords) (void))
+init_data(const char *conf_file, vector_t * (*init_keywords) (void))
 {
 	/* Init Keywords structure */
 	keywords = vector_alloc();
 	(*init_keywords) ();
 
-#if 0
+#if DUMP_KEYWORDS
 	/* Dump configuration */
-	vector_dump(keywords);
-	dump_keywords(keywords, 0);
+	dump_keywords(keywords, 0, NULL);
 #endif
 
 	/* Stream handling */
 	current_keywords = keywords;
-	read_conf_file((conf_file) ? conf_file : CONF);
+	read_conf_file(conf_file);
 	free_keywords(keywords);
+	clear_rttables();
 }
