@@ -4,14 +4,14 @@
  *              a loadbalanced server pool using multi-layer checks.
  *
  * Part:        Scheduling framework. This code is highly inspired from
- *              the thread management routine (thread.c) present in the 
+ *              the thread management routine (thread.c) present in the
  *              very nice zebra project (http://www.zebra.org).
  *
  * Author:      Alexandre Cassen, <acassen@linux-vs.org>
  *
- *              This program is distributed in the hope that it will be useful, 
- *              but WITHOUT ANY WARRANTY; without even the implied warranty of 
- *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+ *              This program is distributed in the hope that it will be useful,
+ *              but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *              See the GNU General Public License for more details.
  *
  *              This program is free software; you can redistribute it and/or
@@ -31,6 +31,10 @@
 #undef FREE
 #endif
 
+#ifndef _DEBUG_
+#define NDEBUG
+#endif
+#include <assert.h>
 #include <signal.h>
 #include <sys/wait.h>
 #include <sys/select.h>
@@ -40,10 +44,44 @@
 #include "utils.h"
 #include "signals.h"
 #include "logger.h"
+#include "bitops.h"
+
 
 /* global vars */
 thread_master_t *master = NULL;
 
+void
+report_child_status(int status, pid_t pid, const char *prog_name)
+{
+	const char *prog_id;
+	char pid_buf[10];	/* "pid 32767" + '\0' */
+
+	if (prog_name)
+		prog_id = prog_name;
+	else {
+		snprintf(pid_buf, sizeof(pid_buf), "pid %d", pid);
+		prog_id = pid_buf;
+	}
+
+	if (WIFEXITED(status)) {
+		if (WEXITSTATUS(status) != 0)
+			log_message(LOG_INFO, "%s exited with status %d", prog_id, WEXITSTATUS(status));
+		return;
+	}
+	if (WIFSIGNALED(status)) {
+		if (WTERMSIG(status) == SIGSEGV) {
+			log_message(LOG_INFO, "%s exited due to segmentation fault (SIGSEGV).", prog_id);
+			log_message(LOG_INFO, "  Please report a bug at %s", "https://github.com/acassen/keepalived/issues");
+			log_message(LOG_INFO, "  %s", "and include this log from when keepalived started, what happened");
+			log_message(LOG_INFO, "  %s", "immediately before the crash, and your configuration file.");
+		}
+		else
+			log_message(LOG_INFO, "%s exited due to signal %d", prog_id, WTERMSIG(status));
+
+		return;
+	}
+}
+
 /* Make thread master. */
 thread_master_t *
 thread_make_master(void)
@@ -69,7 +107,7 @@ thread_list_add(thread_list_t * list, thread_t * thread)
 }
 
 /* Add a new thread to the list. */
-void
+static void
 thread_list_add_before(thread_list_t * list, thread_t * point, thread_t * thread)
 {
 	thread->next = point;
@@ -83,7 +121,7 @@ thread_list_add_before(thread_list_t * list, thread_t * point, thread_t * thread
 }
 
 /* Add a thread in the list sorted by timeval */
-void
+static void
 thread_list_add_timeval(thread_list_t * list, thread_t * thread)
 {
 	thread_t *tt;
@@ -100,7 +138,7 @@ thread_list_add_timeval(thread_list_t * list, thread_t * thread)
 }
 
 /* Delete a thread from the list. */
-thread_t *
+static thread_t *
 thread_list_delete(thread_list_t * list, thread_t * thread)
 {
 	if (thread->next)
@@ -176,13 +214,14 @@ thread_destroy_list(thread_master_t * m, thread_list_t thread_list)
 }
 
 /* Cleanup master */
-static void
+void
 thread_cleanup_master(thread_master_t * m)
 {
 	/* Unuse current thread lists */
 	thread_destroy_list(m, m->read);
 	thread_destroy_list(m, m->write);
 	thread_destroy_list(m, m->timer);
+	thread_destroy_list(m, m->child);
 	thread_destroy_list(m, m->event);
 	thread_destroy_list(m, m->ready);
 
@@ -193,6 +232,8 @@ thread_cleanup_master(thread_master_t * m)
 
 	/* Clean garbage */
 	thread_clean_unuse(m);
+
+	memset(m, 0, sizeof(*m));
 }
 
 /* Stop thread scheduler. */
@@ -204,7 +245,7 @@ thread_destroy_master(thread_master_t * m)
 }
 
 /* Delete top of the list and return it. */
-thread_t *
+static thread_t *
 thread_trim_head(thread_list_t * list)
 {
 	if (list->head)
@@ -213,7 +254,7 @@ thread_trim_head(thread_list_t * list)
 }
 
 /* Make new thread. */
-thread_t *
+static thread_t *
 thread_new(thread_master_t * m)
 {
 	thread_t *new;
@@ -435,6 +476,7 @@ thread_cancel(thread_t * thread)
 	return 0;
 }
 
+#ifdef _INCLUDE_UNUSED_CODE_
 /* Delete all events which has argument value arg. */
 void
 thread_cancel_event(thread_master_t * m, void *arg)
@@ -455,6 +497,7 @@ thread_cancel_event(thread_master_t * m, void *arg)
 		}
 	}
 }
+#endif
 
 /* Update timer value */
 static void
@@ -484,7 +527,7 @@ thread_compute_timer(thread_master_t * m, timeval_t * timer_wait)
 	thread_update_timer(&m->read, &timer_min);
 	thread_update_timer(&m->child, &timer_min);
 
-	/* Take care about monothonic clock */
+	/* Take care about monotonic clock */
 	if (!timer_isnull(timer_min)) {
 		timer_min = timer_sub(timer_min, time_now);
 		if (timer_min.tv_sec < 0) {
@@ -618,7 +661,8 @@ retry:	/* When thread can't fetch try to find next thread again. */
 			thread_list_delete(&m->child, t);
 			thread_list_add(&m->ready, t);
 			t->type = THREAD_CHILD_TIMEOUT;
-		}
+		} else
+			break;
 	}
 
 	/* Read thead. */
@@ -683,7 +727,8 @@ retry:	/* When thread can't fetch try to find next thread again. */
 			thread_list_delete(&m->timer, t);
 			thread_list_add(&m->ready, t);
 			t->type = THREAD_READY;
-		}
+		} else
+			break;
 	}
 
 	/* Return one event. */
@@ -706,7 +751,7 @@ retry:	/* When thread can't fetch try to find next thread again. */
 }
 
 /* Synchronous signal handler to reap child processes */
-void
+static void
 thread_child_handler(void * v, int sig)
 {
 	thread_master_t * m = v;
@@ -717,7 +762,7 @@ thread_child_handler(void * v, int sig)
 	 */
 	thread_t *thread;
 	pid_t pid;
-	int status = 77;
+	int status;
 	while ((pid = waitpid(-1, &status, WNOHANG))) {
 		if (pid == -1) {
 			if (errno == ECHILD)
@@ -725,6 +770,8 @@ thread_child_handler(void * v, int sig)
 			DBG("waitpid error: %s", strerror(errno));
 			assert(0);
 		} else {
+			report_child_status(status, pid, NULL);
+
 			thread = m->child.head;
 			while (thread) {
 				thread_t *t;
@@ -732,9 +779,9 @@ thread_child_handler(void * v, int sig)
 				thread = t->next;
 				if (pid == t->u.c.pid) {
 					thread_list_delete(&m->child, t);
-					thread_list_add(&m->ready, t);
 					t->u.c.status = status;
 					t->type = THREAD_READY;
+					thread_list_add(&m->ready, t);
 					break;
 				}
 			}
@@ -744,7 +791,7 @@ thread_child_handler(void * v, int sig)
 
 
 /* Make unique thread id for non pthread version of thread manager. */
-unsigned long int
+static unsigned long
 thread_get_id(void)
 {
 	static unsigned long int counter = 0;
@@ -774,8 +821,10 @@ launch_scheduler(void)
 	while (thread_fetch(master, &thread)) {
 		/* Run until error, used for debuging only */
 #ifdef _DEBUG_
-		if ((debug & 520) == 520) {
-			debug &= ~520;
+		if (__test_bit(MEM_ERR_DETECT_BIT, &debug) &&
+		    __test_bit(DONT_RELEASE_VRRP_BIT, &debug)) {
+			__clear_bit(MEM_ERR_DETECT_BIT, &debug);
+			__clear_bit(DONT_RELEASE_VRRP_BIT, &debug);
 			thread_add_terminate_event(master);
 		}
 #endif
