@@ -27,7 +27,6 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <assert.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <syslog.h>
@@ -104,8 +103,10 @@ typedef struct _thread_master {
 extern thread_master_t *master;
 
 /* Prototypes. */
+extern void report_child_status(int, pid_t, const char *);
 extern thread_master_t *thread_make_master(void);
 extern thread_t *thread_add_terminate_event(thread_master_t *);
+extern void thread_cleanup_master(thread_master_t *);
 extern void thread_destroy_master(thread_master_t *);
 extern thread_t *thread_add_read(thread_master_t *, int (*func) (thread_t *), void *, int, long);
 extern thread_t *thread_add_write(thread_master_t *, int (*func) (thread_t *), void *, int, long);
@@ -113,9 +114,7 @@ extern thread_t *thread_add_timer(thread_master_t *, int (*func) (thread_t *), v
 extern thread_t *thread_add_child(thread_master_t *, int (*func) (thread_t *), void *, pid_t, long);
 extern thread_t *thread_add_event(thread_master_t *, int (*func) (thread_t *), void *, int);
 extern int thread_cancel(thread_t *);
-extern void thread_cancel_event(thread_master_t *, void *);
 extern thread_t *thread_fetch(thread_master_t *, thread_t *);
-extern void thread_child_handler(void *, int);
 extern void thread_call(thread_t *);
 extern void launch_scheduler(void);
 
