@@ -25,13 +25,15 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include "utils.h"
+#include "signals.h"
+#include "bitops.h"
 
 /* global vars */
 unsigned long debug = 0;
 
 /* Display a buffer into a HEXA formated output */
 void
-dump_buffer(char *buff, int count)
+dump_buffer(char *buff, int count, FILE* fp)
 {
 	int i, j, c;
 	int printnext = 1;
@@ -44,28 +46,28 @@ dump_buffer(char *buff, int count)
 	for (i = 0; i < c; i++) {
 		if (printnext) {
 			printnext--;
-			printf("%.4x ", i & 0xffff);
+			fprintf(fp, "%.4x ", i & 0xffff);
 		}
 		if (i < count)
-			printf("%3.2x", buff[i] & 0xff);
+			fprintf(fp, "%3.2x", buff[i] & 0xff);
 		else
-			printf("   ");
+			fprintf(fp, "   ");
 		if (!((i + 1) % 8)) {
 			if ((i + 1) % 16)
-				printf(" -");
+				fprintf(fp, " -");
 			else {
-				printf("   ");
+				fprintf(fp, "   ");
 				for (j = i - 15; j <= i; j++)
 					if (j < count) {
 						if ((buff[j] & 0xff) >= 0x20
 						    && (buff[j] & 0xff) <= 0x7e)
-							printf("%c",
+							fprintf(fp, "%c",
 							       buff[j] & 0xff);
 						else
-							printf(".");
+							fprintf(fp, ".");
 					} else
-						printf(" ");
-				printf("\n");
+						fprintf(fp, " ");
+				fprintf(fp, "\n");
 				printnext = 1;
 			}
 		}
@@ -120,6 +122,7 @@ inet_ntop2(uint32_t ip)
 	return buf;
 }
 
+#ifdef _INCLUDE_UNUSED_CODE_
 /*
  * IP network to ascii representation. To use
  * for multiple IP address convertion into the same call.
@@ -136,25 +139,22 @@ inet_ntoa2(uint32_t ip, char *buf)
 
 /* IP string to network mask representation. CIDR notation. */
 uint8_t
-inet_stom(char *addr)
+inet_stom(const char *addr)
 {
 	uint8_t mask = 32;
-	char *cp = addr;
+	const char *cp = addr;
 
-	if (!strstr(addr, "/"))
+	if (!(cp = strchr(addr, '/')))
 		return mask;
-	while (*cp != '/' && *cp != '\0')
-		cp++;
-	if (*cp == '/')
-		return atoi(++cp);
-	return mask;
+	return atoi(cp+1);
 }
+#endif
 
 /* IP string to network range representation. */
 uint8_t
-inet_stor(char *addr)
+inet_stor(const char *addr)
 {
-	char *cp = addr;
+	const char *cp = addr;
 
 	if (!strstr(addr, "-"))
 		return 0;
@@ -165,9 +165,37 @@ inet_stor(char *addr)
 	return 0;
 }
 
+/* Domain to sockaddr_storage */
+int
+domain_stosockaddr(const char *domain, const char *port, struct sockaddr_storage *addr)
+{
+	struct addrinfo *res = NULL;
+
+	if (getaddrinfo(domain, NULL, NULL, &res) != 0 || !res)
+		return -1;
+
+	addr->ss_family = res->ai_family;
+
+	if (addr->ss_family == AF_INET6) {
+		struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
+		*addr6 = *(struct sockaddr_in6 *) res->ai_addr;
+		if (port)
+			addr6->sin6_port = htons(atoi(port));
+	} else {
+		struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
+		*addr4 = *(struct sockaddr_in *) res->ai_addr;
+		if (port)
+			addr4->sin_port = htons(atoi(port));
+	}
+
+	freeaddrinfo(res);
+
+	return 0;
+}
+
 /* IP string to sockaddr_storage */
 int
-inet_stosockaddr(char *ip, char *port, struct sockaddr_storage *addr)
+inet_stosockaddr(char *ip, const char *port, struct sockaddr_storage *addr)
 {
 	void *addr_ip;
 	char *cp = ip;
@@ -175,17 +203,10 @@ inet_stosockaddr(char *ip, char *port, struct sockaddr_storage *addr)
 	addr->ss_family = (strchr(ip, ':')) ? AF_INET6 : AF_INET;
 
 	/* remove range and mask stuff */
-	if (strstr(ip, "-")) {
-		while (*cp != '-' && *cp != '\0')
-			cp++;
-		if (*cp == '-')
-			*cp = 0;
-	} else if (strstr(ip, "/")) {
-		while (*cp != '/' && *cp != '\0')
-			cp++;
-		if (*cp == '/')
-			*cp = 0;
-	}
+	if ((cp = strchr(ip, '-')))
+		*cp = 0;
+	else if ((cp = strchr(ip, '/')))
+		*cp = 0;
 
 	if (addr->ss_family == AF_INET6) {
 		struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
@@ -199,42 +220,41 @@ inet_stosockaddr(char *ip, char *port, struct sockaddr_storage *addr)
 		addr_ip = &addr4->sin_addr;
 	}
 
-	if (!inet_pton(addr->ss_family, ip, addr_ip))
+	if (!inet_pton(addr->ss_family, ip, addr_ip)) {
+		addr->ss_family = AF_UNSPEC;
 		return -1;
+	}
 
 	return 0;
 }
 
 /* IPv4 to sockaddr_storage */
-int
+void
 inet_ip4tosockaddr(struct in_addr *sin_addr, struct sockaddr_storage *addr)
 {
 	struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
 	addr4->sin_family = AF_INET;
 	addr4->sin_addr = *sin_addr;
-	return 0;
 }
 
 /* IPv6 to sockaddr_storage */
-int
+void
 inet_ip6tosockaddr(struct in6_addr *sin_addr, struct sockaddr_storage *addr)
 {
 	struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
 	addr6->sin6_family = AF_INET6;
 	addr6->sin6_addr = *sin_addr;
-	return 0;
 }
 
-int
+void
 inet_ip6scopeid(uint32_t scope_id, struct sockaddr_storage *addr)
 {
 	struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *) addr;
 	addr6->sin6_scope_id = scope_id;
-	return 0;
 }
 
 /* IP network to string representation */
-char *
+static char *
 inet_sockaddrtos2(struct sockaddr_storage *addr, char *addr_str)
 {
 	void *addr_ip;
@@ -273,7 +293,7 @@ inet_sockaddrport(struct sockaddr_storage *addr)
 		struct sockaddr_in *addr4 = (struct sockaddr_in *) addr;
 		port = addr4->sin_port;
 	}
-	
+
 	return port;
 }
 
@@ -295,7 +315,7 @@ inet_sockaddrip4(struct sockaddr_storage *addr)
 {
 	if (addr->ss_family != AF_INET)
 		return -1;
-	
+
 	return ((struct sockaddr_in *) addr)->sin_addr.s_addr;
 }
 
@@ -304,7 +324,7 @@ inet_sockaddrip6(struct sockaddr_storage *addr, struct in6_addr *ip6)
 {
 	if (addr->ss_family != AF_INET6)
 		return -1;
-	
+
 	*ip6 = ((struct sockaddr_in6 *) addr)->sin6_addr;
 	return 0;
 }
@@ -313,45 +333,27 @@ inet_sockaddrip6(struct sockaddr_storage *addr, struct in6_addr *ip6)
 int
 inet_inaddrcmp(int family, void *a, void *b)
 {
+	int64_t addr_diff;
+
 	if (family == AF_INET) {
-		if (ntohl(*((const uint32_t *) a)) >
-		    ntohl(*((const uint32_t *) b)))
+		addr_diff = (int64_t)ntohl(*((const uint32_t *) a)) - (int64_t)ntohl(*((const uint32_t *) b));
+		if (addr_diff > 0)
 			return 1;
-		if (ntohl(*((const uint32_t *) a)) <
-		    ntohl(*((const uint32_t *) b)))
+		if (addr_diff < 0)
 			return -1;
 		return 0;
 	}
 
 	if (family == AF_INET6) {
-		if (ntohl(((const uint32_t *) (a))[0]) >
-		    ntohl(((const uint32_t *) (b))[0]))
-			return 1;
-		if (ntohl(((const uint32_t *) (a))[0]) <
-		    ntohl(((const uint32_t *) (b))[0]))
-			return -1;
-
-		if (ntohl(((const uint32_t *) (a))[1]) >
-		    ntohl(((const uint32_t *) (b))[1]))
-			return 1;
-		if (ntohl(((const uint32_t *) (a))[1]) <
-		    ntohl(((const uint32_t *) (b))[1]))
-			return -1;
-
-		if (ntohl(((const uint32_t *) (a))[2]) >
-		    ntohl(((const uint32_t *) (b))[2]))
-			return 1;
-		if (ntohl(((const uint32_t *) (a))[2]) <
-		    ntohl(((const uint32_t *) (b))[2]))
-			return -1;
-
-		if (ntohl(((const uint32_t *) (a))[3]) >
-		    ntohl(((const uint32_t *) (b))[3]))
-			return 1;
-		if (ntohl(((const uint32_t *) (a))[3]) <
-		    ntohl(((const uint32_t *) (b))[3]))
-			return -1;
+		int i;
 
+		for (i = 0; i < 4; i++ ) {
+			addr_diff = (int64_t)ntohl(((const uint32_t *) (a))[i]) - (int64_t)ntohl(((const uint32_t *) (b))[i]);
+			if (addr_diff > 0)
+				return 1;
+			if (addr_diff < 0)
+				return -1;
+		}
 		return 0;
 	}
 
@@ -419,6 +421,7 @@ inet_ston(const char *addr, uint32_t * dst)
 	return 1;
 }
 
+#ifdef _INCLUDE_UNUSED_CODE_
 /*
  * Return broadcast address from network and netmask.
  */
@@ -441,6 +444,7 @@ inet_cidrtomask(uint8_t cidr)
 		mask |= (1 << (31 - b));
 	return ntohl(mask);
 }
+#endif
 
 /* Getting localhost official canonical name */
 char *
@@ -489,36 +493,61 @@ string_equal(const char *str1, const char *str2)
 	return (*str1 == 0 && *str2 == 0);
 }
 
-int
-fork_exec(char **argv)
+void
+set_std_fd(int force)
 {
-	pid_t pid;
 	int fd;
-	int status;
 
-	pid = fork();
-	if (pid < 0)
-		return -1;
-
-	/* Child */
-	if (pid == 0) {
-		fd = open("/dev/null", O_RDWR, 0);
+	if (force || __test_bit(DONT_FORK_BIT, &debug)) {
+		fd = open("/dev/null", O_RDWR);
 		if (fd != -1) {
 			dup2(fd, STDIN_FILENO);
 			dup2(fd, STDOUT_FILENO);
 			dup2(fd, STDERR_FILENO);
-			if (fd > 2)
+			if (fd > STDERR_FILENO)
 				close(fd);
 		}
+	}
+
+	signal_pipe_close(STDERR_FILENO+1);
+}
+
+#ifndef _HAVE_LIBIPTC_
+int
+fork_exec(char **argv)
+{
+	pid_t pid;
+	int status;
+	struct sigaction act, old_act;
+	int res = 0;
+
+	act.sa_handler = SIG_DFL;
+	sigemptyset(&act.sa_mask);
+	act.sa_flags = 0;
+
+	sigaction(SIGCHLD, &act, &old_act);
+
+	pid = fork();
+	if (pid < 0)
+		res = -1;
+	else if (pid == 0) {
+		/* Child */
+		set_std_fd(false);
+
+		signal_handler_script();
+
 		execvp(*argv, argv);
 		exit(EXIT_FAILURE);
 	} else {
 		/* Parent */
 		while (waitpid(pid, &status, 0) != pid);
 
-		if (WEXITSTATUS(status) != EXIT_SUCCESS)
-			return -1;
+		if (!WIFEXITED(status) || WEXITSTATUS(status) != EXIT_SUCCESS)
+			res = -1;
 	}
 
-	return 0;
+	sigaction(SIGCHLD, &old_act, NULL);
+
+	return res;
 }
+#endif
