@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <string.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
@@ -48,18 +49,16 @@
 extern unsigned long debug;
 
 /* Prototypes defs */
-extern void dump_buffer(char *, int);
+extern void dump_buffer(char *, int, FILE *);
 extern u_short in_csum(u_short *, int, int, int *);
 extern char *inet_ntop2(uint32_t);
-extern char *inet_ntoa2(uint32_t, char *);
-extern uint8_t inet_stom(char *);
-extern uint8_t inet_stor(char *);
-extern int inet_stosockaddr(char *, char *, struct sockaddr_storage *);
-extern int inet_ip4tosockaddr(struct in_addr *, struct sockaddr_storage *);
-extern int inet_ip6tosockaddr(struct in6_addr *, struct sockaddr_storage *);
-extern int inet_ip6scopeid(uint32_t, struct sockaddr_storage *);
+extern uint8_t inet_stor(const char *);
+extern int domain_stosockaddr(const char *, const char *, struct sockaddr_storage *);
+extern int inet_stosockaddr(char *, const char *, struct sockaddr_storage *);
+extern void inet_ip4tosockaddr(struct in_addr *, struct sockaddr_storage *);
+extern void inet_ip6tosockaddr(struct in6_addr *, struct sockaddr_storage *);
+extern void inet_ip6scopeid(uint32_t, struct sockaddr_storage *);
 extern char *inet_sockaddrtos(struct sockaddr_storage *);
-extern char *inet_sockaddrtos2(struct sockaddr_storage *, char *);
 extern char *inet_sockaddrtopair(struct sockaddr_storage *addr);
 extern uint16_t inet_sockaddrport(struct sockaddr_storage *);
 extern uint32_t inet_sockaddrip4(struct sockaddr_storage *);
@@ -67,10 +66,11 @@ extern int inet_sockaddrip6(struct sockaddr_storage *, struct in6_addr *);
 extern int inet_inaddrcmp(int, void *, void *);
 extern int inet_sockaddrcmp(struct sockaddr_storage *, struct sockaddr_storage *);
 extern int inet_ston(const char *, uint32_t *);
-uint32_t inet_broadcast(uint32_t, uint32_t);
-uint32_t inet_cidrtomask(uint8_t);
 extern char *get_local_name(void);
 extern int string_equal(const char *, const char *);
+extern void set_std_fd(int);
+#ifndef _HAVE_LIBIPTC_
 extern int fork_exec(char **argv);
+#endif
 
 #endif
